/*
*  program name: Rubrica <http://digilander.iol.it/nfragale/download.html>
*  file: card.c
*  
*  Copyright (C) 2000-2002 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <config.h>

#include <gnome.h>
#include <gconf/gconf-client.h>
#include <libxml/tree.h>
#include <libxml/parser.h>

#include "rubrica.h"
#include "rubrica_type.h"
#include "form_card.h"
#include "item.h"
#include "card.h"
#include "main.h"
#include "callbacks.h"
#include "data_view.h"
#include "view.h"
#include "ui.h"
#include "dialog.h"
#include "prefer.h"
#include "prefer_configure.h"
#include "sort.h"
#include "browse.h"
#include "utils.h"





RubricaMenu columns[] = {
  RUBRICA_MENU_CHECK(N_("Card"), on_pop_check_item_clicked, 
                     RUBRICA_KEY_SCC_CARD),
  RUBRICA_MENU_CHECK(N_("First name"), on_pop_check_item_clicked,
		     RUBRICA_KEY_SCC_FIRST_NAME),
  RUBRICA_MENU_CHECK(N_("Last name"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCC_LAST_NAME),
  RUBRICA_MENU_CHECK(N_("Profession"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCC_PROFESSION),
  RUBRICA_MENU_CHECK(N_("City"), on_pop_check_item_clicked,
		     RUBRICA_KEY_SCC_CITY),
  RUBRICA_MENU_CHECK(N_("Country"), on_pop_check_item_clicked,
		     RUBRICA_KEY_SCC_COUNTRY),
  RUBRICA_MENU_CHECK(N_("Web"), on_pop_check_item_clicked,
		     RUBRICA_KEY_SCC_WEB),
  RUBRICA_MENU_CHECK(N_("Email"), on_pop_check_item_clicked,
		     RUBRICA_KEY_SCC_EMAIL),
  RUBRICA_MENU_CHECK(N_("Telephone"), on_pop_check_item_clicked,
		     RUBRICA_KEY_SCC_TELEPHONE),
  RUBRICA_MENU_CHECK(N_("Company"), on_pop_check_item_clicked,
		     RUBRICA_KEY_SCC_COMPANY),
  RUBRICA_MENU_CHECK(N_("Assigment"), on_pop_check_item_clicked,
		     RUBRICA_KEY_SCC_ASSIGMENT),
  RUBRICA_MENU_END
};

RubricaMenu personal[] = {  
  RUBRICA_MENU_CHECK(N_("First name"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_FIRST_NAME),
  RUBRICA_MENU_CHECK(N_("Middle name"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_MIDDLE_NAME),
  RUBRICA_MENU_CHECK(N_("Last name"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_LAST_NAME),
  RUBRICA_MENU_CHECK(N_("Nickname"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_NICKNAME),
  RUBRICA_MENU_CHECK(N_("Profession"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_PROFESSION),
  RUBRICA_MENU_CHECK(N_("Name prefix"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_NAME_PREFIX),
  RUBRICA_MENU_CHECK(N_("Title"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_TITLE),
  RUBRICA_MENU_CHECK(N_("Birthday"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_BIRTHDAY),
  RUBRICA_MENU_END
};

RubricaMenu address[] ={
  RUBRICA_MENU_CHECK(N_("Street"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_STREET),
  RUBRICA_MENU_CHECK(N_("Street number"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_STREET_NUM),
  RUBRICA_MENU_CHECK(N_("Zip code"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_ZIP_CODE),
  RUBRICA_MENU_CHECK(N_("City"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_CITY),
  RUBRICA_MENU_CHECK(N_("Province"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_PROVINCE),
  RUBRICA_MENU_CHECK(N_("State"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_STATE),
  RUBRICA_MENU_CHECK(N_("Country"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COUNTRY),
  RUBRICA_MENU_END
};


RubricaMenu communication[] = {
  RUBRICA_MENU_CHECK(N_("Web urls"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_WEB),
  RUBRICA_MENU_CHECK(N_("Email addresses"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_EMAIL),
  RUBRICA_MENU_CHECK(N_("Telephone numbers"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_TELEPHONE),
  RUBRICA_MENU_END
};

RubricaMenu company[] = {
  RUBRICA_MENU_CHECK(N_("Company"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COMPANY),
  RUBRICA_MENU_CHECK(N_("Street"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COMPANY_STREET),
  RUBRICA_MENU_CHECK(N_("Street number"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COMPANY_STREET_NUM),
  RUBRICA_MENU_CHECK(N_("Zip code"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COMPANY_ZIP_CODE),
  RUBRICA_MENU_CHECK(N_("City"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COMPANY_CITY),
  RUBRICA_MENU_CHECK(N_("Province"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COMPANY_PROVINCE),
  RUBRICA_MENU_CHECK(N_("Country"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COMPANY_COUNTRY),
  RUBRICA_MENU_CHECK(N_("Web url"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COMPANY_WEB),
  RUBRICA_MENU_CHECK(N_("Email"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COMPANY_EMAIL),
  RUBRICA_MENU_CHECK(N_("Operator"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_OPERATOR),
  RUBRICA_MENU_CHECK(N_("Fax"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_FAX),
  RUBRICA_MENU_CHECK(N_("Green"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_GREEN),
  RUBRICA_MENU_CHECK(N_("Customer care"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_CUSTOMER_CARE),
  RUBRICA_MENU_CHECK(N_("Notes"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_COMPANY_NOTE),
  RUBRICA_MENU_END
};


RubricaMenu assigment[] = {
  RUBRICA_MENU_CHECK(N_("Assigment"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_ASSIGMENT),
  RUBRICA_MENU_CHECK(N_("Organization"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_ORG),
  RUBRICA_MENU_CHECK(N_("Department"), on_pop_check_item_clicked,
		     RUBRICA_KEY_SCD_DEP),
  RUBRICA_MENU_CHECK(N_("Sub department"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_SUB_DEP),
  RUBRICA_MENU_CHECK(N_("Secretary name"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_SECRETARY),
  RUBRICA_MENU_CHECK(N_("Secretary phone"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_SECR_PHONE),
  RUBRICA_MENU_END
};


RubricaMenu note[] = {
  RUBRICA_MENU_CHECK(N_("Contact is married"), 
		     on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_MARRIED),
  RUBRICA_MENU_CHECK(N_("Spouse name"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_SPOUSE),
  RUBRICA_MENU_CHECK(N_("Spouse birthday"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_SPOUSE_BIRTH),
  RUBRICA_MENU_CHECK(N_("Anniversary"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_ANNIVERSARY),
  RUBRICA_MENU_CHECK(N_("Contact has children"), 
		     on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_HAS_CHILDREN),
  RUBRICA_MENU_CHECK(N_("Children notes"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_CHILDREN),
  RUBRICA_MENU_CHECK(N_("Notes"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_NOTE),
  RUBRICA_MENU_CHECK(N_("Public key"), on_pop_check_item_clicked, 
		     RUBRICA_KEY_SCD_PUBLIC_KEY),     
  RUBRICA_MENU_END
};


RubricaMenu card_show[] = {
  RUBRICA_MENU_SUBTREE(N_("Columns"), columns),
  RUBRICA_MENU_SUBTREE(N_("Personal"), personal),
  RUBRICA_MENU_SUBTREE(N_("Address"), address),
  RUBRICA_MENU_SUBTREE(N_("Telephone and net"), communication),
  RUBRICA_MENU_SUBTREE(N_("Company"), company),
  RUBRICA_MENU_SUBTREE(N_("Assigment"), assigment),
  RUBRICA_MENU_SUBTREE(N_("Notes"), note),  
  RUBRICA_MENU_CHECK(N_("Groups"), on_pop_check_item_clicked,
		     RUBRICA_KEY_SHOW_GROUPS),
  RUBRICA_MENU_END
};


RubricaMenu popcardmenu[] = {
  RUBRICA_MENU_ICONED_ITEM(N_("Cut"), RMP_STOCK, GTK_STOCK_CUT, 
			   on_pop_cut_activate, NULL),
  RUBRICA_MENU_ICONED_ITEM(N_("Copy"), RMP_STOCK, GTK_STOCK_COPY,
			   on_pop_copy_activate, NULL),
  RUBRICA_MENU_ICONED_ITEM(N_("Paste"), RMP_STOCK, GTK_STOCK_PASTE,
			   on_pop_paste_activate, NULL),
  RUBRICA_MENU_SEPARATOR,
  RUBRICA_MENU_ICONED_ITEM(N_("_Add"), RMP_FILE, "add-menu.png",
			   on_pop_add_activate, NULL),
  RUBRICA_MENU_ICONED_ITEM(N_("_Remove"), RMP_FILE, "remove-menu.png",
			   on_pop_remove_activate, NULL),
  RUBRICA_MENU_ICONED_ITEM(N_("_Modify"), RMP_FILE, "mod-menu.png",
			   on_pop_modify_activate, NULL),
  RUBRICA_MENU_SEPARATOR,
  RUBRICA_MENU_ICONED_SUBTREE(N_("Show"), RMP_FILE, "view.png", card_show),
  RUBRICA_MENU_END
};  




GtkTreeModel*
rubrica_card_create_model(void)
{
  GtkListStore *model;
  gint i;

  /*      create list store 
   */
  model = gtk_list_store_new(COLUMNS, 
			     G_TYPE_STRING,       /* card          */
			     G_TYPE_STRING,       /* first name    */ 
			     G_TYPE_STRING,       /* last name     */ 
			     G_TYPE_STRING,       /* profession    */  
			     G_TYPE_STRING,       /* city          */
			     G_TYPE_STRING,       /* country       */
			     G_TYPE_STRING,       /* web           */
			     G_TYPE_STRING,       /* email         */ 
			     G_TYPE_STRING,       /* telephone     */ 
			     G_TYPE_STRING,       /* company       */  
			     G_TYPE_STRING,       /* assigment     */
			     G_TYPE_INT,          /* creation date */ 
			     G_TYPE_INT,          /* change date   */
			     G_TYPE_ULONG);       /* rubrica's id  */      

  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 
				       CARD_COLUMN, 
				       GTK_SORT_ASCENDING);

  gtk_tree_sortable_set_default_sort_func(GTK_TREE_SORTABLE(model), 
					  (GtkTreeIterCompareFunc) 
					  rubrica_sort_alfa,
					  GINT_TO_POINTER(CARD_COLUMN), NULL);
  
  for (i = FIRST_NAME_COLUMN; i < CREATED_COLUMN; i++)
    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), i, 
				    (GtkTreeIterCompareFunc) 
				    rubrica_sort_alfa, 
				    GINT_TO_POINTER(i), NULL);

  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), CREATED_COLUMN,
				  (GtkTreeIterCompareFunc) rubrica_sort_date, 
				  GINT_TO_POINTER(CREATED_COLUMN), NULL);

  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), CHANGED_COLUMN,
				  (GtkTreeIterCompareFunc) rubrica_sort_date, 
				  GINT_TO_POINTER(CHANGED_COLUMN), NULL);

  return GTK_TREE_MODEL(model);
}


void 
rubrica_card_add_columns(GtkTreeView *view)
{
  GtkTreeViewColumn *column;
  GtkCellRenderer *renderer;
  gint col;
  GConfClient *client;

  client = gconf_client_get_default(); 
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes(view, -1, _("Card"), 
						    renderer, "text", 
						    CARD_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					  CARD_COLUMN);
  rubrica_view_set_column_attributes(client, view, 
				RUBRICA_KEY_SCC_CARD, col-1);

  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes(view, -1, _("First Name"),
						    renderer, "text",
						    FIRST_NAME_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					  FIRST_NAME_COLUMN);
  rubrica_view_set_column_attributes(client, view, 
				     RUBRICA_KEY_SCC_FIRST_NAME, col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Last Name"),
						     renderer, "text", 
						     LAST_NAME_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					  LAST_NAME_COLUMN);
  rubrica_view_set_column_attributes(client, view, 
				     RUBRICA_KEY_SCC_LAST_NAME, col-1);

  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Profession"),
						     renderer, "text",
						     PROFESSION_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					  PROFESSION_COLUMN);
  rubrica_view_set_column_attributes(client, view, RUBRICA_KEY_SCC_PROFESSION,
				     col-1);

  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("City"), 
						     renderer, "text", 
						     CITY_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					  CITY_COLUMN);
  rubrica_view_set_column_attributes(client, view, RUBRICA_KEY_SCC_CITY, 
				     col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Country"), 
						     renderer, "text", 
						     COUNTRY_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column),
					  COUNTRY_COLUMN);
  rubrica_view_set_column_attributes(client, view, RUBRICA_KEY_SCC_COUNTRY, 
				     col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Web"),
						     renderer, "text",
						     WEB_CARD_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					  WEB_CARD_COLUMN);
  rubrica_view_set_column_attributes(client, view, RUBRICA_KEY_SCC_WEB, col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Email"),
						     renderer, "text",
						     EMAIL_CARD_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					  EMAIL_CARD_COLUMN);
  rubrica_view_set_column_attributes(client, view, RUBRICA_KEY_SCC_EMAIL, 
				     col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Telephone"),
						     renderer, "text", 
						     TELEPHONE_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column),
					  TELEPHONE_COLUMN);
  rubrica_view_set_column_attributes(client, view, RUBRICA_KEY_SCC_TELEPHONE, 
				     col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Company"), 
						     renderer, "text",
						     COMPANY_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column),
					  COMPANY_COLUMN);
  rubrica_view_set_column_attributes(client, view, RUBRICA_KEY_SCC_COMPANY, 
				     col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Assigment"),
						     renderer, "text",
						     ASSIGMENT_COLUMN, NULL);
  column = gtk_tree_view_get_column(view, col-1);
  gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column),
					  ASSIGMENT_COLUMN);
  rubrica_view_set_column_attributes(client, view, RUBRICA_KEY_SCC_ASSIGMENT, 
				     col-1); 
}



/*  add into addressbook's notebook a new page with the 
    content of file "filename"

    return the number of the new page
*/
RubricaDoc*
rubrica_card_add_page_view(gchar *filename)
{
  RubricaDoc *rubrica;      /* the new addressbook item */
  RubricaDocTab *tab;
  GtkWidget* scroolwin;
  GtkTreeModel* model;
  GtkTreeView* view;        /* the cards list */
  GtkWidget* tab_box;
  GtkWidget* popup;
  gchar *path; 
  gchar *fname = NULL;

  popup   = rubrica_menu_popup_new(popcardmenu);
  rubrica = rubrica_doc_new();   

  if (filename)
    {
      path  = g_path_get_dirname(filename);
      fname = g_path_get_basename(filename);
    }
  else
    {
      fname = g_strdup(_("no name")); 
      path  = rubrica_preferences_get_home();
    }

  /*    build new addressbook. 
	It will be related to new notebook page  
  */
  tab = rubrica_doc_tab_new(fname);
  rubrica_doc_set_doc_tab(rubrica, tab);
  tab_box = rubrica_doc_get_tab_box(rubrica);

  /*    make notebook page's child 
   */
  model     = rubrica_card_create_model();
  scroolwin = rubrica_view_new_view(rubrica, model);
  view      = GTK_TREE_VIEW(rubrica_doc_get_view(rubrica));

  /*      store infos about this addressbook/page 
   */
  rubrica_doc_set_name(rubrica, fname);
  rubrica_doc_set_path(rubrica, path);
  rubrica_doc_set_child(rubrica, scroolwin);
    
  rubrica_menu_attach_popup(GTK_WIDGET(view), GTK_MENU(popup));

  g_signal_connect(G_OBJECT(rubrica), "item_added", 
 	 	   G_CALLBACK(rubrica_card_insert_item), NULL);
  g_signal_connect(G_OBJECT(rubrica), "item_added", 
 	 	   G_CALLBACK(rubrica_data_view_update), NULL);


  /*      append a new addressbook view 
	  and increase the pages number 
   */
  rubrica_doc_append_doc(rubrica, scroolwin, tab_box);

  return rubrica;
}


RubricaItem* 
rubrica_card_get_nth_item(GtkTreeView *view, gint n)
{
  RubricaDoc *doc;
  gulong id;
  GtkTreeModel *model;
  GtkTreeIter iter; 
  gchar *string;
  gboolean bool;

  doc    = rubrica_doc_get_current_document();
  model  = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
  string = g_strdup_printf("%d", n);  
  
  /* iter initialization */
  bool   = gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(model), 
					       &iter, string);
  g_free(string);

  if (bool)
    gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, ID_COLUMN, &id, -1);
  else
    return NULL;

  return rubrica_doc_get_item_by_id(doc, id);
}

/*
void 
rubrica_card_append_item(RubricaDoc *rubrica, 
			 RubricaItem *item, 
			 gpointer data)
{
  GtkTreeView *view;
  GtkTreeIter iter;
  GtkTreeModel *model;
  gchar *web, *email, *number, *city, *country;

  g_return_if_fail(rubrica != NULL);
  g_return_if_fail(item != NULL);
  
  view = GTK_TREE_VIEW(rubrica_doc_get_view(rubrica));
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));

  if (item->addresses && item->addresses->data)
    {
      RubricaCardAddress* address;

      address = item->addresses->data;
      city    = g_strdup(address->city);
      country = g_strdup(address->country);
    }
  else
    {
      city = country = "";
    }

  if (item->web && item->web->data)
    web = g_strdup(item->web->data);
  else
    web = "";

  if (item->email && item->email->data)
    email = g_strdup(item->email->data);
  else
    email = "";

  if (item->phone && item->phone->data)
    {
      RubricaCardTelNum *tn;
      
      tn = item->phone->data;
      number = g_strdup(tn->number);
    }
  else
    number = ""; 

  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		     CARD_COLUMN,       item->card, 
		     FIRST_NAME_COLUMN, item->personal.first, 
		     LAST_NAME_COLUMN,  item->personal.last,
		     PROFESSION_COLUMN, item->personal.prof,
		     CITY_COLUMN,       city,
		     COUNTRY_COLUMN,    country,
		     WEB_CARD_COLUMN,   web,
		     EMAIL_CARD_COLUMN, email,
		     TELEPHONE_COLUMN,  number,
		     COMPANY_COLUMN,    item->company.name,
		     ASSIGMENT_COLUMN,  item->work.assigment,
		     CREATED_COLUMN,    item->created,
		     CHANGED_COLUMN,    item->changed,
		     ID_COLUMN,         item->id,
		     -1);  
}
*/





void 
rubrica_card_insert_item(RubricaDoc *rubrica,
			 RubricaItem *item, 
			 gint index,
			 gpointer user_data)
{
  GtkTreeView *view;
  GtkTreeIter iter;
  GtkTreeModel *model;
  gchar *web, *email, *number, *city, *country;

  g_return_if_fail(rubrica != NULL);
  g_return_if_fail(item != NULL);

  view = GTK_TREE_VIEW(rubrica_doc_get_view(rubrica));
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));

  if (item->addresses && item->addresses->data)
    {
      RubricaCardAddress* address;

      address = item->addresses->data;
      city    = g_strdup(address->city);
      country = g_strdup(address->country);
    }
  else
    {
      city = country = "";
    }

  if (item->web && item->web->data)
    web = g_strdup(item->web->data);
  else
    web = "";

  if (item->email && item->email->data)
    email = g_strdup(item->email->data);
  else
    email = "";

  if (item->phone && item->phone->data)
    {
      RubricaCardTelNum *tn;
      
      tn = item->phone->data;
      number = g_strdup(tn->number);
    }
  else
    number = ""; 

  if (index == -1)
    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  else
    gtk_list_store_insert(GTK_LIST_STORE(model), &iter, index);

  gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		     CARD_COLUMN,       item->card, 
		     FIRST_NAME_COLUMN, item->personal.first, 
		     LAST_NAME_COLUMN,  item->personal.last,
		     PROFESSION_COLUMN, item->personal.prof,
		     CITY_COLUMN,       city,
		     COUNTRY_COLUMN,    country,
		     WEB_CARD_COLUMN,   web,
		     EMAIL_CARD_COLUMN, email,
		     TELEPHONE_COLUMN,  number,
		     COMPANY_COLUMN,    item->company.name,
		     ASSIGMENT_COLUMN,  item->work.assigment,
		     CREATED_COLUMN,    RUBRICA_ITEM(item)->created,
		     CHANGED_COLUMN,    RUBRICA_ITEM(item)->changed,
		     ID_COLUMN,         RUBRICA_ITEM(item)->id,
		     -1);  
}


gulong 
rubrica_card_get_selected_item_id(RubricaDoc* doc)
{  
  GtkTreeIter iter;
  GtkTreeView *view;
  GtkTreeModel *model;
  GtkTreeSelection *select;
  guint id;

  g_return_val_if_fail(IS_RUBRICA_DOC(doc), -1L);

  view   = GTK_TREE_VIEW(rubrica_doc_get_view(doc));
  model  = gtk_tree_view_get_model (view);
  select = gtk_tree_view_get_selection(view);  

  if (rubrica_doc_is_empty(doc))
    {
      rubrica_dialog_message(_("Address book is empty. I can't continue\n"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 

      return -1L;
    }

  if (gtk_tree_selection_get_selected(select, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, ID_COLUMN, &id, -1);
      
      return id;
    }
  /* else */
  rubrica_dialog_message(_("I can't continue if you don't select a card"), 
			 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);   
  
  return -1L;
}

void
rubrica_card_modify_item(RubricaDoc *doc, RubricaItem* new, gint index)
{
  GtkTreeIter iter;
  GtkTreeView *view;
  GtkTreeModel *model;

  g_return_if_fail(IS_RUBRICA_DOC(doc));
  g_return_if_fail(IS_RUBRICA_ITEM(new));
  
  view  = GTK_TREE_VIEW(rubrica_doc_get_view(doc));
  g_return_if_fail(view != NULL);
  
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
  
  gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
  rubrica_card_insert_item(doc, new, index, NULL);  
}


gboolean rubrica_card_remove_selected_item(RubricaDoc *doc)
{
  GtkTreeIter iter;
  GtkTreeView *view;
  GtkTreeModel *model;
  GtkTreeSelection *selection;  
  RubricaItem *item;
  gulong id;

  g_return_val_if_fail(doc != NULL, FALSE);
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), FALSE);

  if (rubrica_doc_is_empty(doc))
    {
      rubrica_dialog_message(_("Address book is empty. I can't continue\n"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 
      
      return FALSE;
    }

  view  = GTK_TREE_VIEW(rubrica_doc_get_view(doc));
  g_return_val_if_fail(view != NULL, FALSE);
  
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
    {      
      gchar *card;

      if (rubrica_utils_confirm_remove())
	{
	  GtkWidget *dialog;
	  gchar *str;
	  gint choose;

	  gtk_tree_model_get (model, &iter, 
			      CARD_COLUMN, &card, 
			      ID_COLUMN,   &id,     
			      -1);

	  str = g_strdup_printf(_("Do you want delete the %s card?"), card);

	  dialog = rubrica_dialog_query(str, GTK_BUTTONS_YES_NO);
	  gtk_widget_show(dialog);
	  g_free(str);
	  
	  choose = gtk_dialog_run(GTK_DIALOG(dialog));
	  gtk_widget_destroy(dialog);

	  switch(choose)
	    {
	    case GTK_RESPONSE_YES:
	      item = rubrica_doc_get_item_by_id(doc, id);
	      break;
	      
	    case GTK_RESPONSE_NO:
	    default:
	      return FALSE;
	      break;
	    }	

	  if (rubrica_item_is_deletable(RUBRICA_ITEM(item)))
	    {
	      rubrica_item_set_deleted(RUBRICA_ITEM(item), TRUE);
	      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);      
	      
	      return TRUE;
	    }
	  else
	    {
	      gchar *msg;
	      
	      msg = g_strdup_printf(_("The %s card is marked undeleteble.\n"
				      "Please modify the card status and "
				      "then delete it"), card);
	      rubrica_dialog_error(msg);
	      
	      g_free(msg);

	      return FALSE;
	    }    	
	}
    }
  else
    {
      rubrica_dialog_message(_("I can't continue if you don't select a card"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);
    }
  
  return FALSE;    
}


gboolean 
rubrica_card_remove_last_item(RubricaDoc *doc)
{
  RubricaItem *item;
  GtkTreeIter iter;
  GtkTreeView *view;
  GtkTreeModel *model;
  gint n;

  g_return_val_if_fail(doc != NULL, FALSE);

  if (rubrica_doc_is_empty(doc))
    {
      rubrica_dialog_message(_("Address book is empty. I can't continue\n"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 

      return FALSE;
    }

  view  = GTK_TREE_VIEW(rubrica_doc_get_view(doc));
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
  
  n = gtk_tree_model_iter_n_children(model, NULL); 
  if (gtk_tree_model_iter_nth_child(model, &iter, NULL, n-1))
    {
      gtk_list_store_remove(GTK_LIST_STORE(model), &iter); 
      
      item = rubrica_doc_get_last_item(doc);
      rubrica_item_set_deleted(RUBRICA_ITEM(item), TRUE);
      
      return TRUE;
    }

  return FALSE;
}

gboolean 
rubrica_card_remove_nth_item(RubricaDoc *doc, gint n)
{
  RubricaItem *item;
  GtkTreeIter iter;
  GtkTreeView *view;
  GtkTreeModel *model;
  gchar *str;
  
  g_return_val_if_fail(doc != NULL, FALSE);
  
  if (rubrica_doc_is_empty(doc))
    {
      rubrica_dialog_message(_("Address book is empty. I can't continue\n"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 

      return FALSE;
    }

  view  = GTK_TREE_VIEW(rubrica_doc_get_view(doc));
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
  str = g_strdup_printf("%d", n);
  
  if (gtk_tree_model_get_iter_from_string(model, &iter, str))
    {
      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);  
      g_free(str);
      
      item = rubrica_doc_get_nth_item(doc, n);
      rubrica_item_set_deleted(RUBRICA_ITEM(item), TRUE);      

      return TRUE;
    }

  g_free(str);
  return FALSE;  
}




/*
gboolean 
rubrica_card_free_memory(GtkTreeModel *model, GtkTreePath *path, 
			 GtkTreeIter *iter, gpointer data)
{
  RubricaDoc *doc = data;
  RubricaItem* item;
  gulong id;
  
  if (rubrica_doc_is_empty(doc))
    return TRUE;
  
  gtk_tree_model_get(model, iter, ID_COLUMN, &id, -1);
  item = rubrica_doc_get_item_by_id(doc, id);

  rubrica_card_item_free(item);

  return FALSE;
}
*/

void rubrica_card_cut_item(RubricaDoc *doc)
{
  GtkTreeIter iter;
  GtkTreeView *view;
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  
  view      = GTK_TREE_VIEW(rubrica_doc_get_view(doc));
  model     = gtk_tree_view_get_model(view);
  selection = gtk_tree_view_get_selection(view);
  
  if (gtk_tree_selection_get_selected(selection, NULL, &iter))    
    {
      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);      
    }
  else
    rubrica_dialog_message(_("I can't continue if you don't select a card"), 
			   GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);       
}



/***
 *               CALLBACKS
 ***/


/* show data stored in selected item
*/
void 
rubrica_card_row_selected(GtkTreeSelection *select, gpointer data)
{
  GtkTreeModel *model = gtk_tree_view_get_model(data);
  RubricaDoc *doc;
  RubricaItem *item;
  GtkTreePath *path;
  GtkTreeIter iter;
  gint index;
  gulong id;
  
  doc = rubrica_doc_get_current_document();
  
  if (gtk_tree_selection_get_selected(select, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, ID_COLUMN, &id, -1);
      item = (RubricaItem*) rubrica_doc_get_item_by_id(doc, id);
      path = gtk_tree_model_get_path(model, &iter);
      rubrica_browse_save_path(path);

      if (gtk_tree_selection_path_is_selected(select, path))
	{
	  index = gtk_tree_path_get_indices(path)[0];
	  rubrica_doc_set_index(doc, index);
	}      
      
      rubrica_data_view_show_item(item);
    }
}

/*  double click on card row (edit the selected item)
*/
void 
rubrica_card_row_activated(GtkTreeView *tree_view,
			   GtkTreePath *path,
			   GtkTreeViewColumn *column)
{
  RubricaDoc *doc;
  RubricaItem *item;
  gint index;

  doc   = rubrica_doc_get_current_document();
  item  = rubrica_doc_get_selected_item(doc);
  index = gtk_tree_path_get_indices (path)[0];

  rubrica_browse_save_path(path);

  rubrica_doc_set_index(doc, index);
  rubrica_doc_modify_item(doc, item);
}

