/*
*  Rubrica
*  file: data_view.c
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <config.h>
#include <gnome.h>

#include "ui.h"
#include "rubrica.h"
#include "main.h"
#include "prefer.h"
#include "utils.h"
#include "init.h"
#include "data_view.h"
#include "item.h"


GtkWidget*       text_view;
GtkTextBuffer*   text_buffer;
GtkTextTagTable* tag_table;
GtkTextIter      iter;

gboolean   hovering_over_link = FALSE;
GdkCursor* hand_cursor        = NULL;
GdkCursor* regular_cursor     = NULL;

GtkWidget* statusbar   = NULL;
GtkWidget* label_path  = NULL;
GtkWidget* label_items = NULL;

// gchar* font_header = NULL;
// gchar* font_data   = NULL;

gint items = 0;

struct _count {
  gint personal;
  gint communicate;
  gint notes;
  gint assigment;
  gint company;
} count = {0,0,0,0,0};

gboolean loading = FALSE;


void 
rubrica_data_view_set_loading(gboolean bool)
{
  loading = bool;
}


void 
rubrica_data_view_inc_personal(gboolean bool)
{
  if (bool) 
    count.personal++;
  else 
    if (count.personal > 0) 
      count.personal--;
}


void 
rubrica_data_view_inc_communicate(gboolean bool)
{
  if (bool) 
    count.communicate++;
  else 
    if (count.communicate > 0) 
      count.communicate--;
}


void 
rubrica_data_view_inc_company(gboolean bool)
{
  if (bool) 
    count.company++;
  else
    if (count.company > 0) count.company--;
}


void 
rubrica_data_view_inc_assigment(gboolean bool)
{
  if (bool)
    count.assigment++;
  else 
    if (count.assigment > 0)
      count.assigment--;
}


void 
rubrica_data_view_inc_notes(gboolean bool)
{
 if (bool)
   count.notes++;
 else 
   if(count.notes > 0)
     count.notes--;
}


/* 
   TODO:
   - add full pango support
   - add refresh of data 
   - other ?
*/


GtkWidget*
rubrica_data_view_interface_creation(void)
{
  GtkWidget* scrollwin;
  GConfClient *client;
  gchar *font;
  gchar *tmp = NULL; 
  GdkColor color; 

  client = gconf_client_get_default();

  hand_cursor = gdk_cursor_new (GDK_HAND2);
  regular_cursor = gdk_cursor_new (GDK_XTERM);

  scrollwin = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin), 
				  GTK_POLICY_AUTOMATIC, 
				  GTK_POLICY_AUTOMATIC);  
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollwin), 
				      GTK_SHADOW_IN);
  gtk_container_set_border_width(GTK_CONTAINER(scrollwin), 0);
  gtk_widget_show (scrollwin);
  
  text_view   = gtk_text_view_new ();

  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
  tag_table   = gtk_text_buffer_get_tag_table (GTK_TEXT_BUFFER(text_buffer));

  /*      init tags 
   */
  rubrica_data_view_init_tags();

  /*      init user fonts 
   */
  font = gconf_client_get_string(client, RUBRICA_KEY_FONT_HEADER, NULL);
  rubrica_data_view_set_header_font(font);
  
  font = gconf_client_get_string(client, RUBRICA_KEY_FONT_DATA, NULL);
  rubrica_data_view_set_data_font(font);


  /*      init user colours 
  */
  tmp = gconf_client_get_string(client, RUBRICA_KEY_COLOUR_HEADER, NULL); 
  if (tmp)
    {
      gdk_color_parse(tmp, &color);
      rubrica_data_view_set_header_colour(&color); 
    }
  
  tmp = gconf_client_get_string(client, RUBRICA_KEY_COLOUR_DATA, NULL);
  if (tmp)
    {
      gdk_color_parse(tmp, &color); 
      rubrica_data_view_set_data_colour(&color);  
    }

  gtk_container_add (GTK_CONTAINER (scrollwin), text_view);
  gtk_text_view_set_left_margin (GTK_TEXT_VIEW (text_view), 10);
  gtk_text_view_set_right_margin (GTK_TEXT_VIEW (text_view), 10);
//  gtk_text_view_set_pixels_above_lines(GTK_TEXT_VIEW (text_view), 10);
//  gtk_text_view_set_pixels_below_lines(GTK_TEXT_VIEW (text_view), 10);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (text_view), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text_view), FALSE);

  gtk_widget_show (text_view);
 
  g_object_unref(client);

  g_signal_connect (G_OBJECT (text_view), "key-press-event", 
		    G_CALLBACK (key_press_event), NULL);
  g_signal_connect (G_OBJECT (text_view), "event-after", 
		    G_CALLBACK (event_after), NULL);
  g_signal_connect (G_OBJECT (text_view), "motion-notify-event", 
		    G_CALLBACK (motion_notify_event), NULL);
  g_signal_connect (G_OBJECT (text_view), "visibility-notify-event", 
		    G_CALLBACK (visibility_notify_event), NULL);  

  return (GtkWidget *) scrollwin;
}


void 
rubrica_data_view_init_tags(void)
{
  PangoFontDescription* pfd;   /* pango font description */
//  gchar* font;

  gtk_text_buffer_create_tag (text_buffer, "bold", "weight", 
			      PANGO_WEIGHT_BOLD, NULL);
  
  g_print("\nRubrica: setting default fonts");
//  font = g_strdup("Times 12");
  pfd  = pango_font_description_from_string ("Tines 12");    
  gtk_text_buffer_create_tag (text_buffer, "header_font", "font-desc", 
			      pfd, NULL);
  gtk_text_buffer_create_tag (text_buffer, "data_font", "font-desc",
			      pfd, NULL);
  pango_font_description_free (pfd);   
//  g_free(font);
}


void 
rubrica_data_view_set_header_font(gchar *font)
{
  PangoFontDescription *pfd;
  GtkTextTag* tag;

  if (!font)
    return;

  g_print("\nRubrica: setting %s as header font", font);
  
  pfd = pango_font_description_from_string (font);  
  tag = gtk_text_tag_table_lookup (GTK_TEXT_TAG_TABLE(tag_table), 
				   "header_font");
  if (tag)
    gtk_text_tag_table_remove(GTK_TEXT_TAG_TABLE(tag_table), tag);
    
  gtk_text_buffer_create_tag (text_buffer, "header_font", 
			      "font-desc", pfd, NULL);
  pango_font_description_free (pfd);   
}


void 
rubrica_data_view_set_data_font(gchar *font)
{
  PangoFontDescription *pfd;
  GtkTextTag* tag;

  if (!font)
    return;

  g_print("\nRubrica: setting %s as data font", font);
  
  pfd = pango_font_description_from_string (font);  
  tag = gtk_text_tag_table_lookup (GTK_TEXT_TAG_TABLE(tag_table), "data_font");

  if (tag)
    gtk_text_tag_table_remove(GTK_TEXT_TAG_TABLE(tag_table), tag);

  gtk_text_buffer_create_tag (text_buffer, "data_font", 
			      "font-desc", pfd, NULL);
  pango_font_description_free (pfd);
}

void 
rubrica_data_view_set_data_colour(GdkColor *color) 
{
  GtkTextTag* tag; 
  
  tag = gtk_text_tag_table_lookup (GTK_TEXT_TAG_TABLE(tag_table),
				   "data_colour"); 

  if (tag)
    gtk_text_tag_table_remove(GTK_TEXT_TAG_TABLE(tag_table), tag);
  
  gtk_text_buffer_create_tag (text_buffer, "data_colour", 
			      "foreground-gdk", color, NULL); 

  g_print("\nRubrica: setting data colour");
} 

void 
rubrica_data_view_set_header_colour(GdkColor *color) 
{
  GtkTextTag* tag; 
  tag = gtk_text_tag_table_lookup (GTK_TEXT_TAG_TABLE(tag_table), 
				   "header_colour"); 

  if (tag)
    gtk_text_tag_table_remove(GTK_TEXT_TAG_TABLE(tag_table), tag);
  
  gtk_text_buffer_create_tag (text_buffer, "header_colour", 
			      "foreground-gdk", color, NULL); 

  g_print("\nRubrica: setting header colour");
}

/*
gchar *
rubrica_data_view_get_header_font(void)
{
  return (gchar *) font_header;
}

gchar *
rubrica_data_view_get_data_font(void)
{
  return (gchar *) font_data;
}
*/

void 
rubrica_data_view_clean_view(void)
{
  rubrica_ui_clean_text_view(text_view);
}

void 
rubrica_data_view_set_statusbar(GtkWidget *sbar)
{
  statusbar = sbar;
}

GtkWidget*
rubrica_data_view_get_statusbar(void)
{
  return (GtkWidget*) statusbar;
}

GtkWidget*
rubrica_data_view_label_items_new(void)
{
  label_items = gtk_label_new(NULL);

  return label_items;
}


GtkWidget*  
rubrica_data_view_label_path_new(void)
{
  label_path = gtk_label_new(NULL);

  return label_path;
}


void 
rubrica_data_view_show_path(gboolean bool)
{
  if (bool)
    gtk_widget_show(label_path);
  else
    gtk_widget_hide(label_path);
}


void 
rubrica_data_view_set_items(gint n)
{
  gchar* str;

  if (loading) return;

  items = n;
  str = g_strdup_printf("%d", n);

  gtk_label_set_label(GTK_LABEL(label_items), str);
}

gint rubrica_data_view_get_items(void)  
{
  return (gint) items;  
}

void rubrica_data_view_update(RubricaDoc *rubrica,
			      RubricaItem *item, 
			      gint index,
			      gpointer user_data)
{
//  g_print("\nfunzione di callback -rubrica_data_view_update- richiamata dall'emissione del segnale add_item");
  rubrica_data_view_set_items(rubrica->items);
}

void 
rubrica_data_view_set_path(gchar *str)
{
  gtk_label_set_text(GTK_LABEL(label_path),str);		   
}





void 
rubrica_data_view_show_item(RubricaItem *item)
{
  GConfClient *client;
 
  g_return_if_fail(IS_RUBRICA_ITEM(item));

  client = gconf_client_get_default();
  rubrica_ui_clean_text_view(text_view);

  gtk_text_buffer_get_iter_at_offset(text_buffer, &iter, 0);

  rubrica_data_view_write_card_header(_("Card"), item->card, 
				      item->personal.photo);
  rubrica_data_view_write_personal_data(item, client);
  rubrica_data_view_write_addresses_data(item, client);
  rubrica_data_view_write_net_data(item, client);
  rubrica_data_view_write_telephone_data(item, client);
  rubrica_data_view_write_company_data(item, client);
  rubrica_data_view_write_assigment_data(item, client);
  rubrica_data_view_write_note_data(item, client);

  g_object_unref(client);
}


void 
rubrica_data_view_write_header(gchar *header,  // header label
			       gchar *data)    // data (card name or NULL)
{
  gchar *str;

  str = g_strdup_printf("\n%s", header);

  gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, str, -1, 
					   "header_font",
					   "header_colour",
					   "bold", 
					   NULL);
  g_free(str);

  if (data)
    {
      gchar *text;

      text = g_strdup_printf("%s\n", data);

      gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, ": ", -1, 
					       "header_font",
					       "header_colour",
					       "bold", 
					       NULL);       

      gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, text, -1, 
					       "data_font",
					       "data_colour",
					       NULL);

      g_free(text);
    }  
}


void 
rubrica_data_view_write_item(GConfClient *client,
			     gchar *key,   // GConf key
			     gchar *str,   // header label
			     gchar *label) // data
{
  gchar *string;

  if (!label || (g_ascii_strcasecmp(label, "") == 0)) return;

  string = g_strdup_printf("\n%s: ", str);
  
  /* check if user wants to see this data */
  if (gconf_client_get_bool(client, key, NULL))
  {        
    gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, string, -1, 
					     "header_font",
					     "header_colour",
					     "bold", NULL);  

    gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, label, -1, 
					     "data_font",
					     "data_colour",
					     NULL); 
  }
  
  g_free(string);
}


/*  TODO: impostare il formato della data
*/
void 
rubrica_data_view_write_date_item(GConfClient *client, 
				  gchar *key,    // GConf key
				  gchar *str,    // header label
				  gboolean bool, // known date?
				  gchar *label)  // date data
{
  if (bool)
    {
      gchar *string;
      
      string = g_strdup_printf("\n%s: ", str);
      gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, string, -1, 
					       "header_font",
					       "header_colour",
					       "bold", 
					       NULL);   

      gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, 
					       label, -1, 
					       "data_font",
					       "data_colour",
					       NULL);
  
      g_free(string);
    }
}


/*  write company's web and email 
*/
void 
rubrica_data_view_write_net_item(GConfClient *client, 
				 gchar *key, 
				 gchar *str,    // header label 
				 gchar *label,  // net data
				 RubricaViewNetType type)  // data type
{
  gchar *string;
  gchar *uri;
  
  if (gconf_client_get_bool(client, key, NULL))
  {
    string = g_strdup_printf("\n%s:\n", str);
    gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, string, -1, 
					     "header_font",
					     "header_colour",
					     "bold",
					     NULL);  
    g_free(string);
    
    if (type == RUBRICA_VIEW_EMAIL)
      {
	string = g_strdup(label);
	uri = g_strdup_printf("mailto:%s", label);
      }
    else
      {
	string = g_strdup(label);
	uri = g_strdup(label);
      }

    insert_link(text_buffer, &iter, string, uri);
    
    g_free(string);
    g_free(uri);
  }  
}

/* write web and email urls
 */
void 
rubrica_data_view_net_data(GList *net, RubricaViewNetType type)
{
  GConfClient *client;
  GList* alias;

  if (!net) return;
  if (!net->data) return;

  client = gconf_client_get_default();

  if (type == RUBRICA_VIEW_EMAIL)
    rubrica_data_view_write_header(_("Email: "), NULL); 
  else
    rubrica_data_view_write_header(_("Web: "), NULL); 

  alias = net;
  for (; alias; alias = alias->next)
    {
      gchar *uri;
      gchar *str;
      
      if (type == RUBRICA_VIEW_EMAIL)
	{
	  str = g_strdup(alias->data);
	  uri = g_strdup_printf("mailto:%s", (gchar*) alias->data);
	}
      else
	{
	  str = g_strdup(alias->data);
	  uri = g_strdup(alias->data);
	}

      insert_link(text_buffer, &iter, str, uri);

      g_free(uri);
      g_free(str);
    }
}


void rubrica_data_view_write_card_header(gchar* header, gchar* data, 
					 gchar* uri_photo)
{
  GdkPixbuf *pixbuf = NULL;
  GdkPixbuf *scaled;

  gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, header, -1, 
					   "header_font", 
					   "header_colour",
					   "bold", 
					   NULL);

  gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, ": ", -1, 
					   "header_font",
					   "header_colour",
					   "bold", NULL);       
  
  if (data)
    gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, data, -1, 
					     "data_font",
					     "data_colour", NULL);
  else
    gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, 
					     _("no name card"), -1, 
					     "data_font",
					     "data_colour", NULL);      

  if (uri_photo)
    {
      pixbuf = gdk_pixbuf_new_from_file (uri_photo, NULL);
      
      if (pixbuf) 
	{
	  scaled = gdk_pixbuf_scale_simple (pixbuf, 128, 128, 
					    GDK_INTERP_BILINEAR);

	  gtk_text_buffer_insert_pixbuf (text_buffer, &iter, scaled);  

	  g_object_unref (scaled);
	}

      g_object_unref (pixbuf);
    }
}


void rubrica_data_view_write_personal_data(RubricaItem* item,
					   GConfClient* client)
{
  if (count.personal)
    {
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_FIRST_NAME, 
				   _("First Name"), item->personal.first);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_MIDDLE_NAME, 
				   _("Middle Name"), item->personal.middle);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_NICKNAME, 
				   _("Nickname"), item->personal.nick);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_LAST_NAME, 
				   _("Last Name"), item->personal.last);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_PROFESSION, 
				   _("Profession"), item->personal.prof);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_NAME_PREFIX, 
				   _("Name Prefix"), item->personal.prefix);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_TITLE, 
				   _("Title"), item->personal.title);
      rubrica_data_view_write_date_item(client, RUBRICA_KEY_SCD_BIRTHDAY, 
					_("BirthDay"), 
					item->personal.birth,
					RUBRICA_DATE(item->personal.bdate, 
						     RT_MDY));
    }
}

  
void rubrica_data_view_write_addresses_data(RubricaItem* item, 
					    GConfClient* client)
{
  GList* alias;
  
  alias = item->addresses;
  for (; alias; alias = g_list_next(alias))
    {
      RubricaCardAddress* address;
      
      if ((address = alias->data))
	{
	  rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_STREET, 
				       _("Street"), address->street);
	  rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_STREET_NUM, 
				       _("Street Number"),address->number);
	  rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_ZIP_CODE, 
				       _("Zip Code"), address->zip);
	  rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_CITY, 
				       _("City"), address->city);
	  rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_PROVINCE, 
				       _("Province"), address->province);
	  rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_STATE, 
				       _("State"), address->state);
	  rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COUNTRY, 
				       _("Country"), address->country);
	}
    }
}


void rubrica_data_view_write_net_data(RubricaItem* item,
				      GConfClient* client)
{
  if (gconf_client_get_bool(client, RUBRICA_KEY_SCD_WEB, NULL) || 
      gconf_client_get_bool(client, RUBRICA_KEY_SCD_EMAIL, NULL))
    {
      if (item->web || item->email)
	{
	  rubrica_data_view_write_header(_("\nNet Data"), NULL);
	  
	  if (gconf_client_get_bool(client, RUBRICA_KEY_SCD_WEB, NULL))
	    rubrica_data_view_net_data(item->web, RUBRICA_VIEW_WEB);
	  
	  if (gconf_client_get_bool(client, RUBRICA_KEY_SCD_EMAIL, NULL))
	    rubrica_data_view_net_data(item->email, RUBRICA_VIEW_EMAIL);
	}
    }
}


void 
rubrica_data_view_write_telephone_data(RubricaItem *item,
				       GConfClient *client)
{
  GList* alias;

  if (gconf_client_get_bool(client, RUBRICA_KEY_SCD_TELEPHONE, NULL))
    {
      if (item->phone && item->phone->data)
	{
	  rubrica_data_view_write_header(_("\n\nTelephone Data"), NULL);
	  
	  alias = item->phone;
	  for (; alias; alias = alias->next)
	    {
	      RubricaCardTelNum *tn;
	      gchar *str;
	      
	      tn = (RubricaCardTelNum *) alias->data;
	      str = g_strdup_printf("%s\t%s", tn->number, tn->type);
	      
	      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_TELEPHONE, 
					   _("Telephone"), str);     
	      
	      g_free(str);
	    }
	}
    }
}

void
rubrica_data_view_write_company_data(RubricaItem *item, 
				     GConfClient *client)
{
  if (count.company)
    {
      rubrica_data_view_write_header(_("\n\nCompany Data"), NULL);
      
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY, 
				   _("Company"), item->company.name);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_STREET, 
				   _("Street"), item->company.street);
      rubrica_data_view_write_item(client, 
				   RUBRICA_KEY_SCD_COMPANY_STREET_NUM, 
				   _("Street Number"), item->company.number);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_ZIP_CODE,
				   _("Zip Code"), item->company.zip);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_CITY, 
				   _("City"), item->company.city);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_PROVINCE, 
				   _("Province"), item->company.province);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_COUNTRY, 
				   _("Country"), item->company.country);
      rubrica_data_view_write_net_item(client, RUBRICA_KEY_SCD_COMPANY_WEB, 
				       _("Web"), item->company.web, 
				       RUBRICA_VIEW_WEB);
      rubrica_data_view_write_net_item(client,
				       RUBRICA_KEY_SCD_COMPANY_EMAIL, 
				       _("Email"), item->company.email, 
				       RUBRICA_VIEW_EMAIL);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_OPERATOR, 
				   _("Operator"), item->company.operator);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_FAX, 
				   _("Fax"), item->company.fax);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_GREEN, 
				   _("Green"), item->company.green);
      rubrica_data_view_write_item(client,
				   RUBRICA_KEY_SCD_CUSTOMER_CARE, 
				   _("Customer Care"), 
				   item->company.ccare);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_NOTE, 
				   _("Note"), item->company.notes);
      
    }
}

void rubrica_data_view_write_assigment_data(RubricaItem *item, 
					    GConfClient *client)
{
  if (count.assigment)
    {
      rubrica_data_view_write_header(_("\n\nWork Data\n"), NULL); 
	
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_ASSIGMENT, 
				   _("Assigment"), item->work.assigment);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_ORG, 
				   _("Organization"), item->work.org);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_DEP, 
				   _("Department"), item->work.dep);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_SUB_DEP, 
				   _("Sub Department"), item->work.subdep);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_SECRETARY, 
				   _("Secretary"), item->work.secretary);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_SECR_PHONE, 
				   _("Secretary Telephone"), 
				   item->work.telephone);
    }
}

void rubrica_data_view_write_note_data(RubricaItem *item, 
				       GConfClient *client)
{
  if (count.notes)
    {
      rubrica_data_view_write_header(_("\n\nNotes"), NULL);
      
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_MARRIED,
				   _("Contact"), 
				   item->notes.married ? 
				   _("is married") : _("is not married"));
      
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_SPOUSE, 
				   _("Spouse"), item->notes.spouse);
      
      rubrica_data_view_write_date_item(client, 
					RUBRICA_KEY_SCD_SPOUSE_BIRTH, 
					_("Spouse birthday"),
					item->notes.birth,
					RUBRICA_DATE(item->notes.bdate, 
						     RT_MDY));
      
      rubrica_data_view_write_date_item(client, RUBRICA_KEY_SCD_ANNIVERSARY,
					_("Marriage anniversary"), 
					item->notes.ann,
					RUBRICA_DATE(item->notes.adate, 
						     RT_MDY));
      
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_CHILDREN, 
				   _("Children note"), item->notes.children);
        
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_NOTE, _("Note"), 
				   item->notes.notes);
      
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_PUBLIC_KEY, 
				   _("Public key"), item->notes.pubkey);
    }
}



/* Inserts a piece of text into the buffer, giving it the usual
 * appearance of a hyperlink in a web browser: blue and underlined.
 * Additionally, attaches some data on the tag, to make it recognizable
 * as a link. 
 */
void insert_link (GtkTextBuffer *buffer, 
		  GtkTextIter   *iter, 
		  gchar         *text,
		  gchar         *uri)
{
  GtkTextTag *tag;
  
  g_print("\ninsert link::  uri = %s", uri);
  tag = gtk_text_buffer_create_tag (buffer, NULL, 
				    "foreground", "blue", 
				    "underline", PANGO_UNDERLINE_SINGLE, 
				    NULL);
  g_object_set_data (G_OBJECT (tag), "uri", uri);
  gtk_text_buffer_insert_with_tags (buffer, iter, text, -1, tag, NULL);
}


/* Looks at all tags covering the position of iter in the text view, 
 * and if one of them is a link, follow it by showing the page identified
 * by the data attached to it.
 */
void
follow_if_link (GtkWidget   *text_view, 
		GtkTextIter *iter)
{
  GSList *tags = NULL, *tagp = NULL;

  tags = gtk_text_iter_get_tags (iter);
  for (tagp = tags;  tagp != NULL;  tagp = tagp->next)
    {
      GtkTextTag *tag = tagp->data;
      gchar *str = NULL;
      
      str = (gchar* ) g_object_get_data (G_OBJECT (tag), "uri");

      if (str)
        {
	  g_print("\n\a********** URI: %s **********\n", str);
	  break;
        }
    }

  if (tags) 
    g_slist_free (tags);
}

/* Links can be activated by pressing Enter.
 */
gboolean
key_press_event (GtkWidget *text_view,
		 GdkEventKey *event)
{
  GtkTextIter iter;
  GtkTextBuffer *buffer;

  switch (event->keyval)
    {
      case GDK_Return: 
      case GDK_KP_Enter:
        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
        gtk_text_buffer_get_iter_at_mark (buffer, &iter, 
                                          gtk_text_buffer_get_insert (buffer));
        follow_if_link (text_view, &iter);
        break;

      default:
        break;
    }

  return FALSE;
}

/* Links can also be activated by clicking.
 */
gboolean
event_after (GtkWidget *text_view,
	     GdkEvent  *ev)
{
  GtkTextIter start, end, iter;
  GtkTextBuffer *buffer;
  GdkEventButton *event;
  gint x, y;

  if (ev->type != GDK_BUTTON_RELEASE)
    return FALSE;

  event = (GdkEventButton *)ev;

  if (event->button != 1)
    return FALSE;

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));

  /* we shouldn't follow a link if the user has selected something */
  gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
  if (gtk_text_iter_get_offset (&start) != gtk_text_iter_get_offset (&end))
    return FALSE;

  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), 
                                         GTK_TEXT_WINDOW_WIDGET,
                                         event->x, event->y, &x, &y);

  gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (text_view), &iter, x, y);

  follow_if_link (text_view, &iter);

  return FALSE;
}


/* Looks at all tags covering the position (x, y) in the text view, 
 * and if one of them is a link, change the cursor to the "hands" cursor
 * typically used by web browsers.
 */
void
set_cursor_if_appropriate (GtkTextView    *text_view,
                           gint            x,
                           gint            y)
{
  GSList *tags = NULL, *tagp = NULL;
  GtkTextBuffer *buffer;
  GtkTextIter iter;
  gboolean hovering = FALSE;

  buffer = gtk_text_view_get_buffer (text_view);

  gtk_text_view_get_iter_at_location (text_view, &iter, x, y);
  
  tags = gtk_text_iter_get_tags (&iter);
  for (tagp = tags;  tagp != NULL;  tagp = tagp->next)
    {
      GtkTextTag *tag = tagp->data;
      gchar* str = NULL;
	
      str = g_object_get_data (G_OBJECT (tag), "uri");
      if (str) 
        {
          hovering = TRUE;
          break;
        }
    }

  if (hovering != hovering_over_link)
    {
      hovering_over_link = hovering;

      if (hovering_over_link)
        gdk_window_set_cursor(gtk_text_view_get_window(text_view, 
						       GTK_TEXT_WINDOW_TEXT),
			      hand_cursor);
      else
        gdk_window_set_cursor(gtk_text_view_get_window(text_view, 
						       GTK_TEXT_WINDOW_TEXT),
			      regular_cursor);
    }

  if (tags) 
    g_slist_free (tags);
}


/* Update the cursor image if the pointer moved. 
 */
gboolean
motion_notify_event (GtkWidget      *text_view,
		     GdkEventMotion *event)
{
  gint x, y;

  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), 
                                         GTK_TEXT_WINDOW_WIDGET,
                                         event->x, event->y, &x, &y);

  set_cursor_if_appropriate (GTK_TEXT_VIEW (text_view), x, y);

  gdk_window_get_pointer (text_view->window, NULL, NULL, NULL);
  return FALSE;
}

/* Also update the cursor image if the window becomes visible
 * (e.g. when a window covering it got iconified).
 */
gboolean
visibility_notify_event (GtkWidget          *text_view,
			 GdkEventVisibility *event)
{
  gint wx, wy, bx, by;
  
  gdk_window_get_pointer (text_view->window, &wx, &wy, NULL);
  
  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), 
                                         GTK_TEXT_WINDOW_WIDGET,
                                         wx, wy, &bx, &by);

  set_cursor_if_appropriate (GTK_TEXT_VIEW (text_view), bx, by);

  return FALSE;
}
