/*
*  Rubrica
*  file: data_view.h
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_DATA_VIEW_H
#define _RUBRICA_DATA_VIEW_H

#include <gnome.h>
#include <gconf/gconf-client.h>

#include "rubrica.h"
#include "rubrica_type.h"
#include "item.h"


/*  inizializzazione e prelievo dei dati della vista 
*/

typedef enum {
  RUBRICA_VIEW_EMAIL,
  RUBRICA_VIEW_WEB,
} RubricaViewNetType;

void rubrica_data_view_update(RubricaDoc *rubrica,
			      RubricaItem *item, 
			      gint index,
			      gpointer user_data);

GtkWidget*      rubrica_data_view_interface_creation(void);
void            rubrica_data_view_init_tags(void);

void            rubrica_data_view_set_loading(gboolean bool);

void            rubrica_data_view_inc_personal(gboolean bool);
void            rubrica_data_view_inc_communicate(gboolean bool);
void            rubrica_data_view_inc_assigment(gboolean bool);
void            rubrica_data_view_inc_company(gboolean bool);
void            rubrica_data_view_inc_notes(gboolean bool);

void            rubrica_data_view_init_view_type(void);
void            rubrica_data_view_set_view_type(void);

void            rubrica_data_view_set_header_font(gchar *font);
void            rubrica_data_view_set_section_font(gchar *font);
void            rubrica_data_view_set_field_font(gchar *font);
void            rubrica_data_view_set_data_font(gchar *font);

//gchar*          rubrica_data_view_get_header_font(void);
gchar*          rubrica_data_view_get_section_font(void);
gchar*          rubrica_data_view_get_field_font(void);
//gchar*          rubrica_data_view_get_data_font(void);

void            rubrica_data_view_set_header_colour(GdkColor *color);
void            rubrica_data_view_set_data_colour(GdkColor *color);

void            rubrica_data_view_clean_view(void);

GtkWidget*      rubrica_data_view_get_statusbar(void);
void            rubrica_data_view_set_statusbar(GtkWidget *sbar);

GtkWidget*      rubrica_data_view_label_items_new (void);
GtkWidget*      rubrica_data_view_label_path_new  (void);
void            rubrica_data_view_show_path       (gboolean bool);

void            rubrica_data_view_set_items(gint n);
gint            rubrica_data_view_get_items(void);
void            rubrica_data_view_set_path(gchar *str);

void            rubrica_data_view_change_font(RubricaFontType type);

void rubrica_data_view_write_card_header(gchar* header, gchar* data, 
					 gchar* uri_photo);

void            rubrica_data_view_write_data(gchar *data);
void            rubrica_data_view_write_header(gchar *header, gchar *data);
void            rubrica_data_view_write_header_section(gchar *header);
void            rubrica_data_view_write_field(gchar *field,gchar *data);
void            rubrica_data_view_show_photo(gchar* photo_path);
void            rubrica_data_view_write_personal_data(RubricaItem* item,
						      GConfClient* client);
void            rubrica_data_view_write_addresses_data(RubricaItem* item, 
						       GConfClient* client);
void            rubrica_data_view_write_net_data(RubricaItem* item,
						 GConfClient* client);
void            rubrica_data_view_write_telephone_data(RubricaItem *item,
						       GConfClient *client);
void            rubrica_data_view_write_company_data(RubricaItem *item, 
						     GConfClient *client);
void            rubrica_data_view_write_assigment_data(RubricaItem *item, 
						       GConfClient *client);
void            rubrica_data_view_write_note_data(RubricaItem *item, 
						  GConfClient *client);

void            insert_link               (GtkTextBuffer *buffer, 
					   GtkTextIter *iter, 
					   gchar* text, gchar* uri);
void            follow_if_link            (GtkWidget *text_view,
					   GtkTextIter *iter);
gboolean        key_press_event           (GtkWidget *text_view,
					   GdkEventKey *event);
gboolean        event_after               (GtkWidget *text_view, GdkEvent *ev);
void            set_cursor_if_appropriate (GtkTextView *text_view, 
					   gint x, gint y);
gboolean        motion_notify_event       (GtkWidget *text_view, 
					   GdkEventMotion *event);
gboolean        visibility_notify_event   (GtkWidget *text_view,
					   GdkEventVisibility *event);


void            rubrica_data_view_write_net_item(GConfClient *client, 
						 gchar *key, 
						 gchar *str, gchar *label,
						 RubricaViewNetType type);
void            rubrica_data_view_net_data      (GList *net, 
						 RubricaViewNetType type);
void            rubrica_data_view_show_item     (RubricaItem *item);


#endif










