/*
*  Rubrica
*  file: doctab.c
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "doctab.h"
#include "ui.h"
#include "utils.h"
  

/*
  crea un box da inserire come tab nella pagina del notebook.
  Contiene una lampada rossa, una verde, 
  un'etichetta con il nome del file e un bottone per
  chiudere la pagina del notebook.
*/
RubricaDocTab*
rubrica_doc_tab_new(gchar* name)
{  
  RubricaDocTab* doc_tab = NULL;
  GtkWidget* image;
  gchar* str;

  if (!name)
    g_warning("\nRubrica ** rubrica_doc_tab_new: name is NULL");

  doc_tab = (RubricaDocTab*) g_malloc(sizeof(RubricaDocTab));
  if (!doc_tab)
    g_error("\nCan't allocate enougth memory");

  image = rubrica_ui_image_new("close.png");

  doc_tab->ttips = gtk_tooltips_new();
  doc_tab->box   = gtk_hbox_new(FALSE, 2);
  doc_tab->green = rubrica_ui_image_new("ledgreen.png");
  doc_tab->red   = rubrica_ui_image_new("ledred.png");
  doc_tab->label = gtk_label_new(name);
  doc_tab->close = gtk_button_new();

  gtk_container_add(GTK_CONTAINER(doc_tab->close), image);  
  gtk_box_pack_start(GTK_BOX(doc_tab->box), doc_tab->green, FALSE, FALSE, 1);
  gtk_box_pack_start(GTK_BOX(doc_tab->box), doc_tab->red,   FALSE, FALSE, 1);
  gtk_box_pack_start(GTK_BOX(doc_tab->box), doc_tab->label, TRUE,  FALSE, 1);
  gtk_box_pack_start(GTK_BOX(doc_tab->box), doc_tab->close, FALSE, FALSE, 1);

  gtk_button_set_relief(GTK_BUTTON(doc_tab->close), GTK_RELIEF_NONE);
  
  gtk_widget_show(doc_tab->box);
  gtk_widget_show(doc_tab->green);
  gtk_widget_hide(doc_tab->red);
  gtk_widget_show(doc_tab->label);
  gtk_widget_show(doc_tab->close);
  rubrica_utils_add_tooltip(doc_tab->ttips);

  str = g_strdup_printf(_("close the addressbook %s"), name);
  gtk_tooltips_set_tip(GTK_TOOLTIPS(doc_tab->ttips), doc_tab->close, 
		       str, NULL);
  g_free(str);

  return doc_tab;
}


/*  modifica la stringa visualizzata sull'etichetta
*/
void 
rubrica_doc_tab_set_label(RubricaDocTab* tab, gchar *label)
{
  gchar* str;

  str = g_strdup_printf(_("close the addressbook %s"), label);
  gtk_tooltips_set_tip(GTK_TOOLTIPS(tab->ttips), tab->close, 
		       str, NULL);
  
  gtk_label_set_text(GTK_LABEL(tab->label), label);

  g_free(str);
}


/*  spegne la luce rossa (errore/modifica), 
    accende la luce verde (ok)
*/  
void 
rubrica_doc_tab_enable_green(RubricaDocTab* tab)
{
  gtk_widget_hide(tab->red);
  gtk_widget_show(tab->green);
}


/*  spegne la luce verde
    accende la luce rosssa
*/
void 
rubrica_doc_tab_enable_red(RubricaDocTab* tab)
{
  gtk_widget_hide(tab->green);
  gtk_widget_show(tab->red);
}

