/*
*  Rubrica
*  file: file.c
*  
*  Copyright (C) 2000-2002 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <gnome.h>
#include <libxml/tree.h>
#include <libxml/parser.h>

#include "rubrica_type.h"
#include "rubrica.h"
#include "card.h"
#include "item.h"
#include "form_card.h"
#include "prefer.h"
#include "main.h"
#include "file.h"
#include "file_utils.h"
#include "import_int.h"
#include "import.h"
#include "export.h"
#include "utils.h"
#include "dialog.h"
#include "ui.h"
#include "log.h"
#include "data_view.h"

/*  RubricaDoc file structure (fileformat #4) (addressbook type):
    <?xml version="1.0" encoding="UTF-8"?>
    <Rubrica version="" fileformat="" doctype="">  root
       <Card name="" deletable="" deleted="" 
             rank="" id="" 
	     created="" last_change="">            root child
          <Groups>                                 card child 
             <Group>...</Group>                    groups children
	     ...
          </Groups>       
	  <Data>                                   card child
             <FirstName>....</FirstName>           data child 
	     <MiddleName>...</MiddleName>            "    "
	     <LastName>.....</LastName>                    
	     <Nickname>.....</NickName>                    
	     <Profession>...</Profession>                  
	     <NamePrefix>...</NamePrefix>                  
	     <Title>........</Title>                       
	     <Birthday known="" day="" month="" year=""/>  
          </Data>
	  <Addresses>                              card child
             <Address type="">                     addresses children
                <Street number="">...</Street>     address child
		<City zip="">........</City>
		<Province>...........</Province>
		<State>..............</State>
		<Country>............</Country>
	     </Address>
	     ...
          </Addresses>
	  <WebAddresses>                            card child
             <Web>...</Web>                         webaddresses children
	     ...
	  </WebAddresses>
	  <EmailAddresses>                          card child
             <Email>...</Email>                     emailaddresses children
	     ...
          </EmailAddresses>
	  <TelephoneNumbers>
	     <Telephone type="">...</Telephone>
	     ...
	  </TelephoneNumbers>
	  <Company>
             <CompanyName>........</CompanyName>
	     <Street number="">...</Street>
	     <City zip="">........</City>
	     <Province>...........</province>
	     <Country>............</Country>
	     <Web>................</Web>
	     <Email>..............</Email>
	     <Operator>...........</Operator>
	     <Fax>................</Fax>
	     <Green>..............</Green>
	     <CustomerCare>.......</CustomerCare>
	     <Notes>..............</Notes>
          </Company>
	  <Work>
	     <Assigment>..........</Assigment>
	     <Organization>.......</Organization>
             <Department>.........</Department>
	     <SubDepartment>......</SubDepartment>
	     <SecretaryName>......</SecretaryName>
	     <SecretaryPhone>.....</SecretaryPhone>
          </Work>
	  <Notes married="">
             <SpouseName>.........</SpouseName>
	     <SpouseBirthday known="" day="" month="" year=""/>
	     <Anniversary known="" day="" month="" year""/>
	     <Child>..............</Child>
	     <OtherNotes>.........</OtherNotes>
	     <PublicKey>..........</PublicKey>
          </Notes>
       </Card>
       <Card>
          ....
       </Card>
       ...
    </Rubrica>


  Rubrica
     |
     | children
     |
     v   next         next         next 
   Card-------->Card-------->....------->Card
     |
     | children
     |
     v   next                
   Groups-->Data->Addresses->WebAddresses->Email....phone->Company->Work->Notes
     |        \       |           |          |        |       |      |      |
     | child  | child | child     |          |        |       |      |      |
     |        |       v           v          v        v       v      v      v
     |        |     ...         ...         ...      ...     ...    ...    ...
     v        v
   Group    FirstName--->MiddleName--->...
*/


#undef DEBUG

void DEBUG_ITEM(gchar* str, RubricaDoc* rubrica, RubricaItem* item)
{
  GList *l;                                           

  g_print("\n\t\t DEBUG ELEMENTO");                   

  if (str) g_print("\n*** %s ***", str);

  g_print("\n\nRubricaDoc %p", rubrica);                 
  g_print("\n**** id: %ld", rubrica->id);             
  g_print("\n**** new: %d", rubrica->new );           
  g_print("\n**** modified: %d", rubrica->modified);  
  g_print("\n**** empty: %d", rubrica->empty);        
  g_print("\n**** name: %s", rubrica->name);          
  g_print("\n**** path: %s", rubrica->path);          
  g_print("\n**** tmpfile: %s ", rubrica->tmpfile);   
  g_print("\n\nRubricaItem: %p ", item);          
  g_print("\n**** card: %s", item->card);             
  g_print("\n**** father: %p", item->father);         
  g_print("\n**** id: %ld", item->id);                
  g_print("\n**** deletable: %d", item->deletable);   
  g_print("\n**** deleted: %d", item->deleted);       
  g_print("\n**** marked: %d", item->marked);         
  g_print("\n**** rank: %d", item->rank);             

  g_print("\n\n**** Card group");
  for(l = item->groups; l; l = l->next) 
    {             			
      g_print("\n**** group: %s", rubrica_item_get_group_label(l->data));
    }                                             

  g_print("\n\n**** Personal data");
  g_print("\n**** first: %s", item->personal.first); 
  g_print("\n**** middle: %s", item->personal.middle);
  g_print("\n**** last: %s", item->personal.last);
  g_print("\n**** nick: %s", item->personal.nick);
  g_print("\n**** prof: %s", item->personal.prof);
  g_print("\n**** prefix :%s", item->personal.prefix);
  g_print("\n**** title :%s", item->personal.title);
  g_print("\n**** photo uri: %s", item->personal.photo);
  g_print("\n**** birthday: %ld", item->personal.bdate);

  for (l = item->addresses; l; l=l->next)
    {
      RubricaCardAddress* a = l->data;

      g_print("\n\n**** Address");
      g_print("\n**** address type : %d", a->type);
      g_print("\n**** street: %s", a->street);
      g_print("\n**** number: %s", a->number);
      g_print("\n**** zip: %s", a->zip);
      g_print("\n**** city: %s", a->city);
      g_print("\n**** province: %s", a->province);
      g_print("\n**** state: %s", a->state);
      g_print("\n**** country: %s", a->country);
    }
 
  g_print("\n\n**** Web Uri");
  for (l = item->web; l; l=l->next)
    {
      g_print("\n**** web url : %s", (gchar* )l->data);
    }

  g_print("\n\n**** Emails");
  for (l = item->email; l; l=l->next)
    {
      g_print("\n\n**** email: %s", (gchar*)l->data);
    }

  g_print("\n\n**** Telephone numbers");
  for (l = item->phone; l; l=l->next)
    {
      RubricaCardTelNum* tn;

      tn = l->data;
      g_print("\n**** telephone (%s): %s", tn->type, tn->number);
    }

  g_print("\n\n**** Company");
  g_print("\n**** company name: %s", item->company.name);
  g_print("\n**** street: %s", item->company.street); 
  g_print("\n**** number: %s", item->company.number); 
  g_print("\n**** zip: %s", item->company.zip); 
  g_print("\n**** city: %s", item->company.province); 
  g_print("\n**** country: %s", item->company.country); 
  g_print("\n**** web: %s", item->company.web); 
  g_print("\n**** email: %s", item->company.email); 
  g_print("\n**** operator: %s", item->company.operator); 
  g_print("\n**** fax: %s", item->company.fax); 
  g_print("\n**** green: %s", item->company.green); 
  g_print("\n**** customer care: %s", item->company.ccare); 
  g_print("\n**** notes: %s", item->company.notes); 

  g_print("\n\n**** Work");
  g_print("\n**** assigment: %s", item->work.assigment);
  g_print("\n**** organization: %s", item->work.org);
  g_print("\n**** department: %s", item->work.dep);
  g_print("\n**** sub department: %s", item->work.subdep);
  g_print("\n**** secretary (assistent): %s", item->work.secretary);
  g_print("\n**** secretary's telephone: %s", item->work.telephone);
  
  g_print("\n\n**** Notes");
  g_print("\n**** contact is married: %d", item->notes.married);
  g_print("\n**** know birthday: %d", item->notes.birth);
  g_print("\n**** know anniversary day: %d", item->notes.ann);
  g_print("\n**** birthday: %ld", item->notes.bdate);
  g_print("\n**** anniversary: %ld", item->notes.adate);
  g_print("\n**** spouse name: %s", item->notes.spouse);
  g_print("\n**** children notes: %s", item->notes.children);
  g_print("\n**** notes: %s", item->notes.notes);
  g_print("\n**** public key: %s", item->notes.pubkey); 
}


#define RUBRICA_IMPORT_FROM(x) (RubricaImportType) gtk_option_menu_get_history(GTK_OPTION_MENU(x))

#define RUBRICA_EXPORT_TO(x)   (RubricaExportType) gtk_option_menu_get_history(GTK_OPTION_MENU(x))


GtkWidget* loadmenu;
GtkWidget* savemenu;


/* File Selection Dialogs
 */
void 
rubrica_file_load_file(void)
{
  GtkWidget* fs;
  GtkWidget* mainbox;
  GtkWidget* hbox;
  GtkWidget* label;
  GtkWidget* app;
	
  app = rubrica_get_app();
	
  fs = gtk_file_selection_new(_("Open file..."));
  mainbox = GTK_FILE_SELECTION (fs)->main_vbox; 
  
  hbox = gtk_hbox_new(FALSE, 12);
  gtk_box_pack_start(GTK_BOX(mainbox), hbox, FALSE, FALSE, 12);
  gtk_widget_show(hbox);
  
  label = gtk_label_new(_("<b>File type</b>"));
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);    
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_widget_show(label);
	
  loadmenu = rubrica_import_build_options();
  gtk_box_pack_start(GTK_BOX(hbox), loadmenu, TRUE, TRUE, 0);
  gtk_option_menu_set_history(GTK_OPTION_MENU(loadmenu), OPEN_RUBRICA); 
	
  if (rubrica_preferences_get_have_default_folder())
    {
      gchar *path;
      
      path = rubrica_preferences_get_default_folder();
      if (path)
	{
	  gtk_file_selection_set_filename (GTK_FILE_SELECTION(fs), path);      
	  
	  g_free(path);
	}
    }
  
  g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fs)->ok_button), "clicked",
		   G_CALLBACK(rubrica_file_open_file), (gpointer) fs);
  
  g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button),
			   "clicked", G_CALLBACK(gtk_widget_destroy), 
			   (gpointer)fs);

  
  gtk_window_set_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);
  
  /* il genitore della finestra fileselection: app 
   */
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));
  
  gtk_widget_show(GTK_WIDGET(fs));
}


/* function called when the OK button 
   in load dialog has clicked
*/
void 
rubrica_file_open_file(GtkWidget *w, gpointer data)
{
  GtkWidget *fs = data;
  gchar *filename;
  gchar* msg;
  gboolean loaded = FALSE;
  RubricaErrorType err;
  RubricaImportType type;

  filename = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
  type = RUBRICA_IMPORT_FROM(loadmenu);

  msg = g_strdup_printf("Rubrica *** loading file: %s", filename);
  rubrica_log_write_log(msg);
  g_free(msg);

  while (!loaded)
    {
      switch(type)
	{
	case OPEN_RUBRICA:
	  loaded = rubrica_file_load_xmlfile(filename, &err);
	  
	  if (!loaded)
	    rubrica_dialog_error_loading_file(filename, err);
	  break;
	  
	case OPEN_OLD_RUBRICA:
	  
	  break;
	  
	case OPEN_GNOMECARD:
	case OPEN_KAB:
	case OPEN_VCARD:
	case OPEN_PINE:
	  g_print("\n OPEN_VCARD");
	  break;
	  
	case OPEN_CSV_MOZILLA:
	case OPEN_CSV_NETSCAPE:
	  g_print("\n OPEN_CSV");
	  break;
	  
	default:
	  break;
	}
    }
  
  gtk_widget_destroy(fs);
}


/* save_as dialog window
 */
void 
rubrica_file_save_as(RubricaDoc *doc)
{
  GtkWidget *app;
  GtkWidget *fs;
  GtkWidget* mainbox;
  GtkWidget* hbox;
  GtkWidget* label;
  gchar *filename; 
  
  app      = rubrica_get_app();
  filename = rubrica_doc_get_name(doc);

  fs = gtk_file_selection_new(_("Rubrica Save as"));
  mainbox = GTK_FILE_SELECTION (fs)->main_vbox; 

  hbox = gtk_hbox_new(FALSE, 12);
  gtk_box_pack_start(GTK_BOX(mainbox), hbox, FALSE, FALSE, 12);
  gtk_widget_show(hbox);
  
  label = gtk_label_new(_("<b>File type</b>"));
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);    
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_widget_show(label);

  savemenu = rubrica_export_build_options();
  gtk_box_pack_start(GTK_BOX(hbox), savemenu, TRUE, TRUE, 0);
  gtk_option_menu_set_history(GTK_OPTION_MENU(savemenu), SAVE_RUBRICA);   

  gtk_window_set_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));

  g_object_set_data(G_OBJECT(fs), "rubrica", doc);

  if (rubrica_preferences_get_have_default_folder())
    {      
      gchar *path;
			
      path = rubrica_preferences_get_default_folder();
      if (path)
	{
	  gtk_file_selection_set_filename (GTK_FILE_SELECTION(fs), path);
	  
	  g_free(path);
	}
    }
  
  /*    if I have a file name, show it into the entry
   */
  if (filename)
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), filename);
  
  g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fs)->ok_button), "clicked",
		   G_CALLBACK(rubrica_file_write_file), fs);
  
  g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button),
			   "clicked", G_CALLBACK(gtk_widget_destroy), 
			   (gpointer) fs);
  
  gtk_widget_show(fs);
}


/* called when document file has a name
 */
void 
rubrica_file_save(RubricaDoc *doc)
{
  GtkWidget *app;
  gchar *filename;
  gchar *tmp;

  app      = rubrica_get_app();
  filename = rubrica_doc_get_name(doc);

  /*    call rubrica_file_save_as if filename is NULL
  */
  if (!filename || (g_ascii_strcasecmp(filename, N_("no name")) == 0))
    {    
      rubrica_file_save_as(doc);
    }
  else 
    {
      gchar *name = NULL;
			
      if (rubrica_preferences_get_have_default_folder())
	{      
	  gchar *path;
	  
	  path = rubrica_preferences_get_default_folder();
	  if (path)
	    {
	      name = g_strdup_printf("%s/%s", path, filename);
	      
	      g_free(path);
	    }
	  else
	    g_print("\nRubrica error ** Can't read default folder");
	}
      else
	name = g_strdup(filename);
      
      if (!rubrica_file_write_xmlfile(doc, name))  
	{      
	  tmp = g_strdup_printf(_("I can't save the file:\n%s"), name);
	  
	  rubrica_dialog_error(tmp);
	  g_free(tmp);
	}
      else  /* file has been saved */
	{
	  rubrica_doc_set_modified(doc, FALSE);
	  // TODO: messaggio nella status bar -- File Salvato con successo
	}
      
      g_free(name);
    }
}


/* function called when the OK button 
   in save dialog has clicked
*/
void
rubrica_file_write_file(GtkWidget *ok_button, gpointer data)
{
  RubricaDoc *doc;
  GtkWidget *fs;
  gchar *tmp;
  gchar *fname = NULL;

  fs  = (GtkWidget *) data;
  doc = g_object_get_data(G_OBJECT(fs), "rubrica");

  tmp = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));

  /*      if file hasn't extension or extension isn'r .rub
   */
  if (!rubrica_file_utils_has_rub_extension(tmp)) 
    fname = g_strdup_printf("%s.rub",tmp);  
  else 
    fname = g_strdup(tmp);
  
  /*    ERROR if file was not selected, or can't open the file
  */  
  if (!fname || !rubrica_file_write_xmlfile(doc, fname)) 
    {
      gchar *msg;
      
      if (fname)
	msg = g_strdup_printf(_("I can't save the file:\n%s"), fname); 
      else
	msg = g_strdup_printf(_("I can't save a file without name.")); 
      
      rubrica_dialog_error(msg);

      g_free(msg);
    }
  else /* file has been saved */
    {      
      gchar *path = NULL;
      gchar *name = NULL;
      gchar *newname = NULL;

      name = g_path_get_basename(fname);
      if (rubrica_preferences_get_have_default_folder())  
	path  = rubrica_preferences_get_default_folder();
      else	
	path  = g_path_get_dirname(fname);
      
      newname = g_strdup_printf("%s/%s", path, name);
          
      /* set the saved file as last opened/showed
      */
      rubrica_preferences_set_last_book(newname);
      rubrica_doc_set_path(doc, path);
      rubrica_doc_set_name(doc, name);
      rubrica_doc_set_modified(doc, FALSE);
          
      g_free(newname);
      g_free(name);
      g_free(path);
    }

  gtk_widget_destroy(fs);
  g_free(fname);
}


/**
 **
 **  Save Routines
 **
 **/
gboolean
rubrica_file_write_xmlfile(RubricaDoc *doc, gchar *name)
{
  xmlDocPtr xmldoc;     /* pointer to xml doc */
  gchar *path = NULL, *fname;
  gchar *msg;
  
  g_return_val_if_fail(name != NULL, FALSE);

  path  = g_path_get_dirname(name);
  fname = g_strdup(name);

  xmldoc = rubrica_doc_get_xml_tree(doc);

  /* test if exist a file with requested name
     if asked, make backup of old file
     otherwise delete the old file and write the new one
  */
  if (g_file_test(fname, G_FILE_TEST_EXISTS))
    {
      if (rubrica_preferences_get_do_backup())
	rubrica_file_utils_do_backup(doc, fname);
      else
	{
	  remove(fname);
	}
    }
  
  /* do compress doc?
  */
  if (rubrica_preferences_get_compress())
    {
      xmlSetCompressMode(rubrica_preferences_get_compress_rate());
      xmlSetDocCompressMode(xmldoc, rubrica_preferences_get_compress_rate());
    }
  else
    {
      xmlSetDocCompressMode(xmldoc, 0);     
      xmlSetCompressMode(0);
    }
  
  /*  write xml tree
  */
  if (xmlSaveFormatFileEnc(fname, xmldoc, "UTF-8", 1) == -1) 
    {
      msg = g_strdup_printf(_("An error occurred during file saving.\n"
			      "I can't save the file %s"), fname);
      rubrica_log_write_log(msg);
      rubrica_dialog_error(msg);

      xmlFreeDoc(xmldoc);
      g_free(msg);
      g_free(path);
      g_free(fname);

      return FALSE;
    }  

  /* if we are here the addressbook has been writen 
   */
  msg = g_strdup_printf("\naddressbook saved as %s in folder: %s\n", 
			name, path);
  rubrica_log_write_log(msg);
  
  rubrica_data_view_set_path(path);

  /* xmlFreeDoc do a segfault, why? */
//  xmlFreeDoc(doc);    
  g_free(fname);
  g_free(path);
  g_free(msg);

  return TRUE;
}



/**
 **
 **  Load Routines
 **
 **/

/*  this function is called when user want start rubrica whit a file
*/
RubricaErrorType 
rubrica_file_start_with_files(gchar *files[])
{
  gboolean loaded = FALSE;
  RubricaErrorType err = RE_NO_ERROR;
  gchar* file;

  rubrica_utils_set_start(TRUE);
  
  while (*files)
    {
      /* load files */
      file = *files;
      g_print("\nloading %s", file);
      loaded = rubrica_file_load_xmlfile(file, &err);
      if (!loaded)
	{
	  rubrica_dialog_error_loading_file(file, err);     

	  return err; 
	}
      *files++;
    }

  rubrica_utils_set_start(FALSE);
  return RE_NO_ERROR;
}


gboolean 
rubrica_file_load_xmlfile(gchar *filename, RubricaErrorType* err)
{
  RubricaDoc *doc;
  gchar *fpath, *fname;
  
  if (!g_file_test(filename, G_FILE_TEST_EXISTS))
    {
      *err = RE_FILE_NOT_EXIST;
            
      return FALSE;
    }

  rubrica_card_add_page_view(filename); 
  doc = rubrica_doc_get_current_document();

  /* real file load 
   */
  if (!rubrica_file_real_load_xmlfile(doc, filename, err))
    { 
      return FALSE;
    }

  /* if we are here, the file has been loaded, 
     now we need to put in memory some information 
     about this file and addressbook      
  */
  fpath = g_path_get_dirname(filename);
  fname = g_path_get_basename(filename);
  
  rubrica_doc_set_path(doc, fpath);
  rubrica_doc_set_name(doc, fname);
  rubrica_doc_set_new(doc, FALSE);
  rubrica_doc_set_empty(doc, FALSE);
  rubrica_doc_set_modified(doc, FALSE);

  g_free(fname);
  
  *err = RE_NO_ERROR;
  return TRUE;
}



/* 
 *  read the addressbook file 
 */
gboolean 
rubrica_file_real_load_xmlfile(RubricaDoc *doc, gchar *fname, 
				    RubricaErrorType *err)
{
  xmlDocPtr xmldoc = NULL;
  gint fileformat  = 0;
  gchar* msg;

  if((xmldoc = rubrica_file_utils_open_doc(fname)) == NULL)
    return FALSE;

  if(!rubrica_file_utils_is_rubrica_doc(xmldoc, fname))
    return FALSE;

  /* get the file format
   */
  fileformat = rubrica_file_utils_get_fileformat(xmldoc);
  if (fileformat < RUBRICA_FILE_FORMAT)  /* this is an old rubrica file */
    {
      /* close opened file */
      xmlFreeDoc(xmldoc);

      if (fileformat == 3)
	{
	  if (!rubrica_file_utils_load_ff3_doc(doc, err))
	    return FALSE;
	  else
	    return TRUE;
	}      
      
      if (!rubrica_file_utils_import_old_file(fname))
	return FALSE;

      if((xmldoc = rubrica_file_utils_open_doc(fname)) == NULL)
	return FALSE;
      
      if(!rubrica_file_utils_is_rubrica_doc(xmldoc, fname))
	return FALSE;     
    }

  /* parse doc, extract address from file and put them in memory 
   */
  rubrica_file_parse_doc(doc, xmldoc, err);
  
  /* correggere id */
//  rubrica_card_item_test_id(doc);
  
  msg = g_strdup_printf(_("document %s succesfully opened"), fname);
  g_print("\n%s", msg);  
  g_free(msg);

  /* addressbook has been parsed. Free memory */
  xmlFreeDoc(xmldoc);
  
  return TRUE;  
}


void 
rubrica_file_parse_doc(RubricaDoc *doc, xmlDocPtr xmldoc, 
		       RubricaErrorType *err)
{  
  xmlNodePtr node;
  gint n = 0;
      
  node = xmldoc->children;
  if (xmlIsBlankNode(node))
    node = node->next;
  
  if (node && node->name && xmlStrcmp(node->name, "Rubrica") == 0) 
    {
      node = node->children;
      
      if (xmlIsBlankNode(node))
	node = node->next;
      
      /* node is the head of a Card's list */
      rubrica_data_view_set_loading(TRUE);
      while (node)
	{
	  rubrica_file_load_from_xmlnode(doc, node);  
	  n++;
	  
	  /* parse next Card */
	  node = node->next;
	  if(xmlIsBlankNode(node))
	    node = node->next;
	}

      rubrica_data_view_set_loading(FALSE);
    }
  else
    {
      *err = RE_XML_DOC_NO_RUBRICA_HEADER;
      return;
    }
  
  *err = RE_NO_ERROR;  
  rubrica_data_view_set_items(n);
  g_print("\nRubrica reads %d card%s", n, (n > 1 ? "s" : ""));  
}


void
rubrica_file_load_from_xmlnode(RubricaDoc *doc, xmlNodePtr xmlcard)
{
  RubricaItem *item;  
  xmlNodePtr child;

  /* gli xmlnode sono nodi cardxml da cui estrarre i nodi figli */

  item = rubrica_item_new(doc);
#ifdef DEBUG
  DEBUG_ITEM("elemento allocato con successo", doc, item);  
#endif

  /*      Get data in Card node
  */
  if(xmlIsBlankNode(xmlcard))
    xmlcard = xmlcard->next;
  rubrica_file_load_card_data(item, xmlcard);

  /*      Get data in Card childres   */

  /*    first child: group list
   */     
  child = xmlcard->children;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_load_groups_data(item, child);  

  /*    next child: personal data
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_load_personal_data(item, child);

  /*    next child: addresses
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_load_address_data(item, child);  

  /*    next child: web 
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_load_web_data(item, child);


  /*    next child: email 
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_load_email_data(item, child);

  
  /*    next child: telephone 
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_load_telephone_data(item, child);

  
  /*    next child: company 
   */
  child = child->next; 
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_load_company_data(item, child);

  
  /*    next child: work 
   */
  child = child->next; 
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_load_work_data(item, child);


  /*    next child: note 
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_load_note_data(item, child);

#ifdef DEBUG
  DEBUG_ITEM("valori caricati nell' elemento", doc, item);  
  g_print("\n\t\tAPPENDING %s", item->card);
#endif

  rubrica_doc_append_item(doc, item);

#ifdef DEBUG
  g_print("\n\t\tELEMENTO APPESO");
#endif
}




void 
rubrica_file_load_card_data(RubricaItem *item, xmlNodePtr node)
{
  gchar *tmp = NULL;  

  if (node &&                                        /* if node is valid and */
      node->name &&                                  /* node has a name and  */
      xmlStrcmp(node->name, (xmlChar*) "Card") == 0) /* the name is Card     */
    {
      item->card = rubrica_file_utils_get_card_name(node);
      
      g_print("\nreading card: %s", item->card);
      
      item->deletable = rubrica_file_utils_get_boolean_property(node, 
								"deletable");
      item->deleted   = rubrica_file_utils_get_boolean_property(node, 
								"deleted");

      tmp = rubrica_file_utils_get_node_property(node, "rank");      
      if (tmp)
	{
	  item->rank = (RubricaRankType) atoi(tmp);
	  g_free(tmp);
	}
      else item->rank = (RubricaRankType) 2;
      
      tmp = rubrica_file_utils_get_node_property(node, "id");
      if (tmp)
	{
	  item->id = atol(tmp);
	  g_free(tmp);
	}
      else item->id = rubrica_item_ask_id(item);

      item->created = rubrica_file_utils_get_date_property(node, "created");
      item->changed = rubrica_file_utils_get_date_property(node,
							   "last_change");

/*      
      tmp = rubrica_file_get_node_property(node, "created");
      if (tmp)
	{
	 item->created = atol(tmp); 
	 g_free(tmp);
	}
      else item->created = time(NULL);

      tmp = rubrica_file_get_node_property(node, "last_change");
      if (tmp)
	{
	  item->changed = atol(tmp); 
	  g_free(tmp);
	}
      else item->changed = time(NULL);
*/
    }
  else
    {
      /* TODO 
	 
      segnalare l'errore, far scegliere all'utente se 
      continuare a caricare automaticamente il file 
      o vuole un caricamento assistito
      */	 
    }
}


void 
rubrica_file_load_groups_data(RubricaItem *item, xmlNodePtr xmlgroups)
{
  xmlNodePtr child;

  if (xmlgroups && 
      xmlgroups->name && 
      xmlStrcmp(xmlgroups->name, (xmlChar*) "Groups") == 0)
    {
      child = xmlgroups->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      
      while (child)
	{
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  
	  if (child)
	    {
	      RubricaGroup* group;

	      group = (RubricaGroup*) g_malloc(sizeof(RubricaGroup));
	      if (!group) return;
	
	      group->label = rubrica_file_utils_get_node_content(child);
	      group->type  = rubrica_form_card_get_option_type(group->label);  
	      item->groups = g_list_append(item->groups, group);
	      
	      child = child->next;
	    }
	}
    }
}


void 
rubrica_file_load_personal_data(RubricaItem *item, xmlNodePtr xmldata)
{
  xmlNodePtr child;
    
  if (xmldata && 
      xmldata->name &&
      xmlStrcmp(xmldata->name, (xmlChar*) "Data") == 0)  
    {
      /* carico i figli di datixml
       */   
      child = xmldata->children;
      if (xmlIsBlankNode(child))
	child = child->next;      
      item->personal.first = rubrica_file_utils_get_node_content(child);

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->personal.middle = rubrica_file_utils_get_node_content(child);

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;      
      item->personal.last = rubrica_file_utils_get_node_content(child);

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->personal.nick = rubrica_file_utils_get_node_content(child);
        
      child = child->next;  
      if (xmlIsBlankNode(child))
	child = child->next;    
      item->personal.prof = rubrica_file_utils_get_node_content(child);

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->personal.prefix = rubrica_file_utils_get_node_content(child);

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->personal.title = rubrica_file_utils_get_node_content(child);

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;      
      item->personal.birth = rubrica_file_utils_get_boolean_property(child, 
							     "know_birthday");
      if (item->personal.birth)
	item->personal.bdate = rubrica_file_utils_get_calendar_property(child);
      else
	item->personal.bdate = time (NULL);
      

/*
      tmp = xmlGetProp(child, "know_birthday");      
      item->personal.birth = (xmlStrcmp(tmp, "true") == 0) ? TRUE : FALSE;
      item->personal.bdate = ((tmp = xmlGetProp(child, "birthday")) ? 
			      atol(tmp) : time(NULL));
*/
   }
  else 
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"Data");
      
      item->personal.first  = item->personal.middle = 
	item->personal.last = item->personal.prefix = 
	item->personal.title = "";
      item->personal.bdate = time(NULL);
    }
}  



void 
rubrica_file_load_address_data(RubricaItem *item, xmlNodePtr xmladd)
{
  xmlNodePtr xmladdress;
  xmlNodePtr child;
  gchar *tmp;
  
  if (xmladd &&
      xmladd->name &&
      xmlStrcmp(xmladd->name, (xmlChar* ) "Addresses") == 0)
    {
      /* child is the head of an <address> list */
      xmladdress = xmladd->children;  
      if (xmlIsBlankNode(xmladdress))
	xmladdress = xmladdress->next;
      
      while(xmladdress)
	{
	  RubricaCardAddress* address;

	  address = (RubricaCardAddress*) g_malloc(sizeof(RubricaCardAddress));
	  
	  tmp = xmlGetProp(xmladdress, "type");
	  address->type = RUBRICA_STR2ADDRESS_TYPE(tmp);

	  /* get fist <address> child */
	  child = xmladdress->children;  
	  if (xmlIsBlankNode(child))
	    child = child->next;

	  address->street = ((tmp = xmlNodeGetContent(child)) ? 
			     g_strdup(tmp) : "");
	  address->number = (tmp = xmlGetProp(child, "number") ? 
			     g_strdup(tmp) : "");

	  child = child->next;
	  if (xmlIsBlankNode(child))
	    child = child->next;

	  /* loading city values, try to avoid segfault */
	  tmp = NULL;
	  if (child && child->content)
	    tmp = xmlNodeGetContent(child);
	  address->city = tmp ? tmp : "";
	  address->zip = (tmp = xmlGetProp(child, "zip") ? 
			  g_strdup(tmp) : "");

	  child = child->next;
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  address->province = ((tmp = xmlNodeGetContent(child)) ? 
			       g_strdup(tmp) : "");
	  
	  child = child->next;
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  address->state = ((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");
	  
	  child = child->next;
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  address->country = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");

	  item->addresses = g_list_append(item->addresses, address);
	  
	  xmladdress = xmladdress->next;
	  if (xmlIsBlankNode(xmladdress))
	    xmladdress = xmladdress->next;
	}
    }
  else 
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"Address");
      
      item->addresses = NULL;
    }
}

  
void 
rubrica_file_load_web_data(RubricaItem *item, xmlNodePtr node)
{
  GList *web = NULL;
  xmlNodePtr child;
  gchar *buffer;
  
  if (node &&
      node->name &&
      xmlStrcmp(node->name, (xmlChar *) "WebAddresses") == 0)
    {
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      while(child)
	{
	  gchar *tmp;
	  
	  if (xmlIsBlankNode(child))
	    child = child->next;
    	  
	  tmp = xmlNodeGetContent(child);
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      web = g_list_append(web, buffer);
	      child = child->next;
	    }	  
	}
      item->web = web;
    }
  else
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"WebAddresses");
      
      item->web = NULL;
    }
}


void 
rubrica_file_load_email_data(RubricaItem *item, xmlNodePtr node)
{
  GList *email = NULL;
  xmlNodePtr child;
  gchar *buffer;

  if (node &&
      node->name &&
      xmlStrcmp(node->name, (xmlChar*) "EmailAddresses") == 0)
    {
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      while(child)
	{
	  gchar *tmp;
	  
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  
	  tmp = xmlNodeGetContent(child);
	  
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      email = g_list_append(email, buffer);
	      child = child->next;
	    }	
	}
      item->email = email;
    }  
  else
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"EmailAddresses");
      
      item->email = NULL;
    }
}


void
rubrica_file_load_telephone_data(RubricaItem *item, xmlNodePtr node)
{
  GList *tel = NULL;
  xmlNodePtr child;
  RubricaCardTelNum *tn;

  if (node &&
      node->name &&
      xmlStrcmp(node->name, (xmlChar*) "TelephoneNumbers") == 0)
    {
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;

      while(child)
	{
	  gchar *tmp1=NULL, *tmp2=NULL;
	  
	  if (child)
	    {
	      if ((tmp1= xmlNodeGetContent(child)) &&
		  (xmlStrcmp(tmp1, "") != 0))
		{	
		  tn = g_malloc(sizeof(RubricaCardTelNum));
		  
		  if (xmlHasProp(child, "type"))
		    tmp2 = xmlGetProp(child, "type");
		  else 
		    tmp2 = "???";
		  
		  tn->number = g_strdup(tmp1);
		  tn->type   = g_strdup(tmp2);	  	     
		  
		  tel = g_list_append(tel, tn);
		  
		  child = child->next;
		  if(child && xmlIsBlankNode(child))
		    child = child->next;	      
		}
	      else
		{
		  
		  child = child->next;
		  if(child && xmlIsBlankNode(child))
		    child = child->next;
		}
	    }
	} 
      
      item->phone = tel;
    }
  else
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"TelephoneNumbers");
      
      item->phone = NULL;
    }
}


void 
rubrica_file_load_company_data(RubricaItem *item, xmlNodePtr node)
{
  xmlNodePtr child;
  gchar *tmp;

  if (node &&
      node->name && 
      xmlStrcmp(node->name, (xmlChar*) "Company") == 0)
    {
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.name = ((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.street = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");
      
      if (xmlHasProp(child, "number"))
	item->company.number = xmlGetProp(child, "number");
      
      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.city = ((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");
      if (xmlHasProp(child, "zip"))
	item->company.zip = xmlGetProp(child, "zip");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.province = ((tmp = xmlNodeGetContent(child)) ? 
				g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.country = ((tmp = xmlNodeGetContent(child)) ? 
			       g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.web = ((tmp = xmlNodeGetContent(child)) ?
			   g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.email = ((tmp = xmlNodeGetContent(child)) ? 
			     g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.operator = ((tmp = xmlNodeGetContent(child)) ? 
				g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.fax = ((tmp = xmlNodeGetContent(child)) ? 
			   g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.green = ((tmp = xmlNodeGetContent(child)) ? 
			     g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.ccare =((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.notes = ((tmp = xmlNodeGetContent(child)) ? 
			     g_strdup(tmp) : "");
    }
  else   
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"Company");
      
      item->company.name = item->company.street = item->company.number 
	= item->company.zip = item->company.city  = item->company.province
	= item->company.country = item->company.web = item->company.email
	= item->company.operator = item->company.fax = item->company.green 
	= item->company.ccare = item->company.notes = "";
    }
}


void 
rubrica_file_load_work_data(RubricaItem *item, xmlNodePtr node)
{
  xmlNodePtr child;
  gchar *tmp;

  if (node &&
      node->name && 
      xmlStrcmp(node->name, (xmlChar*) "Work") == 0)
    {     
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.assigment = ((tmp = xmlNodeGetContent(child)) ? 
			   g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.org = ((tmp = xmlNodeGetContent(child)) ? g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.dep = ((tmp = xmlNodeGetContent(child)) ? g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.subdep = ((tmp = xmlNodeGetContent(child)) ? 
			g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.secretary = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.telephone = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");
       
    }
  else
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"Work");

      item->work.assigment    = item->work.org       = 
	item->work.dep        = item->work.subdep    = 
	item->work.secretary  = item->work.telephone = "";
    }
}


void 
rubrica_file_load_note_data(RubricaItem *item, xmlNodePtr node)
{
  xmlNodePtr child;
  gchar *tmp;

  if (node &&
      node->name &&
      xmlStrcmp(node->name, (xmlChar*) "Notes") == 0)
    {      
      tmp = xmlGetProp(node, "married");
      item->notes.married = ((xmlStrcmp(tmp, "true") == 0) ? TRUE : FALSE);

      tmp = xmlGetProp(node, "know_birthday");
      item->notes.birth = ((xmlStrcmp(tmp, "true") == 0) ? TRUE : FALSE);

      tmp = xmlGetProp(node, "know_anniversary");
      item->notes.ann = ((xmlStrcmp(tmp, "true") == 0) ? TRUE : FALSE);
      
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->notes.spouse = ((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");

      if (item->notes.birth)
	item->notes.bdate = ((tmp = xmlGetProp(child, "birthday")) ? 
			     atol(tmp) : time(NULL));
      else
	item->notes.bdate = time(NULL);

      if (item->notes.ann)    
	item->notes.adate = ((tmp= xmlGetProp(child, "anniversary")) ? 
				atol(tmp) : time(NULL));
      else
	item->notes.adate = time(NULL);	

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->notes.children = ((tmp = xmlNodeGetContent(child)) ? 
			   g_strdup(tmp) : "");

      child = child->next;
       if (xmlIsBlankNode(child))
	child = child->next;
       item->notes.notes = ((tmp = xmlNodeGetContent(child)) ? 
			 g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->notes.pubkey = ((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");
    } 
  else
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"Notes");

      item->notes.bdate   = item->notes.adate    = time(NULL);
      item->notes.married = item->notes.birth    = item->notes.ann = FALSE;
      item->notes.spouse  = item->notes.children =
	item->notes.notes = item->notes.pubkey   = "";
    }
}


/* richiesto da autosave */
void     rubrica_file_build_xmlnode       (xmlNodePtr xmlnode, 
					   RubricaItem *item)
{
}
