/*
*  program name: Rubrica <http://digilander.iol.it/nfragale/download.html>
*  file:interface.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <config.h>
#include <gnome.h>
#include <glade/glade.h>
#include <gconf/gconf-client.h>

#include "rubrica.h"
#include "callbacks.h"
#include "interface.h"
#include "prefer.h"
#include "prefer_configure.h"
#include "data_view.h"
#include "menu.h"
#include "menubar.h"
#include "rubrica_type.h"
#include "ui.h"
#include "card.h"

GtkAccelGroup *accel_group;

GtkWidget* group_scrollwin;
GtkWidget* group_view;             /* group tree widgets   */
GtkListStore* group_model;
GtkTreeSelection *group_selection;

RubricaOptionType groups[] = {
  {"all.png",          N_("all groups"),    ALL_GROUP},
  {"favorites.png",    N_("favorites"),     FAVORITES_GROUP},
  {"hot-contacts.png", N_("hot contact"),   HOT_GROUP},
  {"friends.png",      N_("friends"),       FRIENDS_GROUP},
  {"heart.png",        N_("heart"),         HEART_GROUP},
  {"home.png",         N_("relatives"),     RELATIVES_GROUP},
  {"holiday.png",      N_("holidays"),      HOLIDAYS_GROUP},
  {"hobby.png",        N_("hobbies"),       HOBBIES_GROUP},
  {"football.png",     N_("sport"),         SPORT_GROUP},
  {"money.png",        N_("bussines"),      BUSSINES_GROUP},
  {"company2.png",     N_("company"),       COMPANY_GROUP},
  {"genius.png",       N_("work"),          WORK_GROUP},
  {"customer.png",     N_("customer"),      CUSTOMER_GROUP},
  {"strategies.png",   N_("strategies"),    STRATEGIES_GROUP},
  {"penguin.png",      N_("programmers"),   PROGRAMMERS_GROUP},
  {"freebsd.png",      N_("BSD friends"),   BSD_GROUP},
  {"pj.png",           N_("journal"),       PLUTO_GROUP},
  {NULL, NULL}
};

enum {
  PIX_GROUP_COLUMN,
  TEXT_GROUP_COLUMN,
  TYPE_GROUP_COLUMN,
  LAST_GROUP_COLUMN
};


void rubrica_interface_populate_model(GtkListStore* model)
{
  GtkTreeIter iter;  
  RubricaOptionType* op = groups;

  while (op->label)
    {
      GdkPixbuf* pixbuf;

      pixbuf = rubrica_ui_pixbuf_new(op->pixmap);

      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
			 PIX_GROUP_COLUMN,  pixbuf, 
			 TEXT_GROUP_COLUMN, op->label,
			 TYPE_GROUP_COLUMN, op->type,
			 -1);

      op++;
    }
  
}

void          
rubrica_interface_add_group_columns(GtkWidget *view)
{
  GtkTreeViewColumn* column;
  GtkCellRenderer* renderer;
  
  column = gtk_tree_view_column_new();
  gtk_tree_view_column_set_title(column, _("Groups"));

  renderer = gtk_cell_renderer_pixbuf_new();
  gtk_tree_view_column_pack_start(column, renderer, FALSE);
  gtk_tree_view_column_set_attributes (column, renderer, 
				       "pixbuf", PIX_GROUP_COLUMN,
				       NULL);

  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_column_pack_start(column, renderer, TRUE);
  gtk_tree_view_column_set_attributes (column, renderer, 
				       "text", TEXT_GROUP_COLUMN,
				       NULL);

  gtk_tree_view_append_column (GTK_TREE_VIEW(view), column);
}

GtkWidget *
rubrica_interface_create_buttons_box(void)
{
  GtkWidget *buttonbox;
  GtkWidget *hbox;
  GtkWidget *hbox2;

  GtkWidget *button_prev;
  GtkWidget *button_next;

  GtkWidget *alignment1;
  GtkWidget *alignment2; 

  GtkWidget *label1;
  GtkWidget *label2;

  GtkWidget *image1;
  GtkWidget *image2; 

  buttonbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (buttonbox), GTK_BUTTONBOX_START);
  gtk_box_set_spacing (GTK_BOX (buttonbox), 18);
  gtk_widget_show (buttonbox);
  
  /*    Prev button
  */
  button_prev = gtk_button_new ();
  gtk_container_add (GTK_CONTAINER (buttonbox), button_prev);
  gtk_widget_show (button_prev);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_container_add (GTK_CONTAINER (button_prev), alignment1);
  gtk_widget_show (alignment1);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox);
  gtk_widget_show (hbox);

  image1 = gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_BUTTON);
  gtk_box_pack_start (GTK_BOX (hbox), image1, FALSE, FALSE, 0);
  gtk_widget_show (image1);

  label1 = gtk_label_new_with_mnemonic (_("_Previous card"));
  gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 0);
  gtk_widget_show (label1);

  /*    Next button
   */
  button_next = gtk_button_new ();
  gtk_container_add (GTK_CONTAINER (buttonbox), button_next);
  GTK_WIDGET_SET_FLAGS (button_next, GTK_CAN_DEFAULT);
  gtk_widget_show (button_next);

  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_container_add (GTK_CONTAINER (button_next), alignment2);
  gtk_widget_show (alignment2);

  hbox2 = gtk_hbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox2);
  gtk_widget_show (hbox2);

  image2 = gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_BUTTON);
  gtk_box_pack_start (GTK_BOX (hbox2), image2, FALSE, FALSE, 0);
  gtk_widget_show (image2);

  label2 = gtk_label_new_with_mnemonic (_("Ne_xt card"));
  gtk_box_pack_start (GTK_BOX (hbox2), label2, FALSE, FALSE, 0);
  gtk_widget_show (label2);

  g_signal_connect (G_OBJECT (button_prev), "clicked",
		    G_CALLBACK (button_browse_cb), 
		    GINT_TO_POINTER(RB_PREV_CARD));
  g_signal_connect (G_OBJECT (button_next), "clicked",
		    G_CALLBACK (button_browse_cb),
		    GINT_TO_POINTER(RB_NEXT_CARD));

  return buttonbox;
}

GtkWidget *
rubrica_interface_create_info_box(GConfClient *client)
{
  GtkWidget *info_box;
  GtkWidget *path_label, *items_label, *path, *items;

  info_box = gtk_hbox_new (FALSE, 18);
  gtk_container_set_border_width (GTK_CONTAINER (info_box), 0);
  
  /*   labels
   */
  items_label = gtk_label_new(_("<b>Items in this address book:</b>"));
  gtk_label_set_use_markup(GTK_LABEL(items_label), TRUE);
  gtk_box_pack_start (GTK_BOX (info_box), items_label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (items_label), GTK_JUSTIFY_LEFT);
  gtk_widget_show(items_label);

  items = rubrica_data_view_label_items_new();
  gtk_box_pack_start (GTK_BOX (info_box), items, FALSE, FALSE, 0);
  gtk_label_set_use_markup(GTK_LABEL(items), TRUE);
  gtk_label_set_justify (GTK_LABEL (items), GTK_JUSTIFY_LEFT);
  gtk_widget_show(items);
  
  path_label = gtk_label_new (_("<b>path:</b>"));
  gtk_label_set_use_markup(GTK_LABEL(path_label), TRUE);
  gtk_box_pack_start (GTK_BOX (info_box), path_label, FALSE, FALSE, 10);
  gtk_label_set_justify (GTK_LABEL (path_label), GTK_JUSTIFY_LEFT);
  gtk_widget_show (path_label);

  path = rubrica_data_view_label_path_new();
  gtk_label_set_use_markup(GTK_LABEL(path), TRUE);
  gtk_box_pack_start (GTK_BOX (info_box), path, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (path), GTK_JUSTIFY_LEFT);
  gtk_widget_show (path);

  rubrica_preferences_configure_path(client, path, RUBRICA_KEY_SHOW);

  return (GtkWidget *) info_box;  
}



GtkWidget* 
rubrica_interface_create_rubrica_app (GConfClient *client) 
{
  GtkWidget* app;
  GtkWidget* main_hbox;
  GtkWidget* data_vbox;
  GtkWidget* paned;

  GtkWidget *bonobodock;
  GtkWidget* appbar;

  GtkWidget* box_books;
  GtkWidget* books_label;
  GtkWidget* books_notebook;

  GtkWidget* box_data;         /* data view widgets */
  GtkWidget* data_label;
  GtkWidget* data_view;

  GtkWidget* buttons_box;
  GtkWidget *info_box; 
  gint width, length;

  g_print("\nRubrica: begin interface creation"); 

  app         = gnome_app_new ("Rubrica", _("Rubrica"));  
  accel_group = gtk_accel_group_new ();  

  rubrica_ui_set_default_icon(app);

  gtk_window_set_title (GTK_WINDOW (app), _("Rubrica"));  
  gtk_window_set_position (GTK_WINDOW (app), GTK_WIN_POS_CENTER); 

  bonobodock = GNOME_APP (app)->dock; 
  gtk_widget_show (bonobodock); 

  /*    Build Toolbars
   */
  rubrica_menu_create_toolbar(app, accel_group);  
  gnome_app_create_menus(GNOME_APP(app), menubar_uiinfo); 

  /*    Build GUI
   */
  main_hbox = gtk_hbox_new(FALSE, 12);
  gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 12); 
  gnome_app_set_contents (GNOME_APP(app), main_hbox);   
  gtk_widget_show (main_hbox); 

  
  /*    Group TreeView
  */
  group_scrollwin = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (group_scrollwin);
  gtk_box_pack_start (GTK_BOX (main_hbox), group_scrollwin, FALSE, FALSE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (group_scrollwin), 
				  GTK_POLICY_NEVER, 
				  GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (group_scrollwin), 
				       GTK_SHADOW_IN);
  
  group_view  = gtk_tree_view_new();
  group_model = gtk_list_store_new(LAST_GROUP_COLUMN, 
				   GDK_TYPE_PIXBUF, 
				   G_TYPE_STRING, 
				   G_TYPE_INT);
  gtk_tree_view_set_model(GTK_TREE_VIEW(group_view), 
			  GTK_TREE_MODEL(group_model)); 
  group_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(group_view));
  gtk_tree_selection_set_mode(group_selection, GTK_SELECTION_SINGLE);
  rubrica_interface_populate_model(group_model);
  rubrica_interface_add_group_columns(group_view);
  gtk_container_add(GTK_CONTAINER(group_scrollwin), group_view);
  gtk_widget_show(group_view);

  
  /*    Vbox
   */
  data_vbox = gtk_vbox_new(FALSE, 18);
  gtk_box_pack_start(GTK_BOX(main_hbox), data_vbox, TRUE, TRUE, 0);
  gtk_widget_show(data_vbox);

  /*    Paned
  */
  paned = gtk_hpaned_new (); 
  gtk_container_add (GTK_CONTAINER (data_vbox), paned);
  gtk_container_set_border_width (GTK_CONTAINER (paned), 0); 
  gtk_paned_set_position (GTK_PANED (paned), 400); 
  gtk_widget_show (paned); 


  /*    AddressBooks notebook
  */
  g_print("\nRubrica: begin addressbooks view initialization...");

  box_books = gtk_vbox_new(FALSE, 6);
  gtk_paned_pack1 (GTK_PANED (paned), box_books, TRUE, TRUE); 
  gtk_widget_show(box_books);  
  
  /*    Note for translators:
	Leave unchanged <b>, </b>, <span>, </span>, ect. Don't translate 
	words into brackets, for example: <b>Cards</b>, translate only 
	Cards. 
  */
  books_label = gtk_label_new("<b>Cards</b>"); 
  gtk_label_set_use_markup(GTK_LABEL(books_label), TRUE);
  gtk_box_pack_start(GTK_BOX(box_books), books_label, FALSE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (books_label), 0, 0.5);
  gtk_widget_show(books_label);  

  books_notebook  = gtk_notebook_new();
  gtk_box_pack_start(GTK_BOX(box_books), books_notebook, TRUE, TRUE, 0); 
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK (books_notebook), GTK_POS_BOTTOM);
  gtk_notebook_set_scrollable(GTK_NOTEBOOK (books_notebook), TRUE);
  gtk_widget_show(books_notebook);

  rubrica_doc_store_notebook(books_notebook);

  g_print("\nRubrica: addressbooks view initialization done.");

  g_print("\nRubrica: begin data view initialization...");

  /*    Data view
  */
  box_data = gtk_vbox_new(FALSE, 6);
  gtk_paned_pack2 (GTK_PANED (paned), box_data, TRUE, TRUE);
  gtk_widget_show(box_data);

  data_label = gtk_label_new("<b>Data</b>");
  gtk_label_set_use_markup(GTK_LABEL(data_label), TRUE);
  gtk_box_pack_start(GTK_BOX(box_data), data_label, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (data_label), 0, 0.5);
  gtk_widget_show(data_label);
  
  data_view = rubrica_data_view_interface_creation();
  gtk_box_pack_start(GTK_BOX(box_data), data_view, TRUE, TRUE, 0);
  gtk_widget_show(data_view);

  g_print("\nRubrica: data view initialization done");

  /*    info boxes
  */
  buttons_box = rubrica_interface_create_buttons_box();
  gtk_box_pack_start (GTK_BOX (data_vbox), buttons_box, FALSE, FALSE, 0);
  gtk_widget_show (buttons_box);

  info_box = rubrica_interface_create_info_box(client);
  gtk_box_pack_start (GTK_BOX (data_vbox), info_box, FALSE, FALSE, 0);
  gtk_widget_show(info_box);


  /*    Appbar
   */
  appbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
  gnome_app_set_statusbar (GNOME_APP (app), appbar);
  gtk_widget_show (appbar);

  gnome_app_install_menu_hints (GNOME_APP (app), menubar_uiinfo);

  rubrica_data_view_set_statusbar(appbar);
  rubrica_preferences_configure_widget_view (client, group_scrollwin,
					     RUBRICA_KEY_SHOW_GROUPS);

  /*     CALLBACKS
   */
  g_signal_connect (G_OBJECT(group_selection), "changed", 
		    G_CALLBACK(rubrica_group_row_selected), group_view);
  
  g_signal_connect (G_OBJECT (app), "destroy", 
		    G_CALLBACK(rubrica_destroy), app); 

  g_signal_connect(G_OBJECT(books_notebook), "switch-page",  
		   G_CALLBACK(document_notebook_switch_pages_cb), 
		   NULL); 


//  gtk_signal_connect (GTK_OBJECT (statusbar), "text_popped",
//                      GTK_SIGNAL_FUNC (on_statusbar_text_popped),
//                      NULL);
//  gtk_signal_connect (GTK_OBJECT (statusbar), "text_pushed",
//                      GTK_SIGNAL_FUNC (on_statusbar_text_pushed),
//                      NULL);


  gtk_window_add_accel_group (GTK_WINDOW (app), accel_group); 

  width  = gconf_client_get_int(client, RUBRICA_KEY_WIDTH, NULL);
  length = gconf_client_get_int(client, RUBRICA_KEY_LENGTH, NULL);
  gtk_window_resize(GTK_WINDOW (app), width, length);

  g_print("\nRubrica: Interface done...\n"); 

  return app; 
}

void 
rubrica_group_row_selected(GtkTreeSelection *select, gpointer data)
{
  GtkTreeModel *model;
  GtkWidget* cview;
  GtkListStore* cmodel;
  GtkTreeIter iter;
  RubricaDoc* doc;
  RubricaGroupType type;
  RubricaItem* item;
  
  g_return_if_fail(data != NULL);
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(data));

  doc = rubrica_doc_get_current_document();

  if (!doc)
    return;

  cview  = rubrica_doc_get_view(doc);
  cmodel = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(cview)));

  if (gtk_tree_selection_get_selected(select, &model, &iter))
    {
      gint items;
      gint i = 0;
      
      gtk_tree_model_get(model, &iter, TYPE_GROUP_COLUMN, &type, -1); 
      gtk_list_store_clear(GTK_LIST_STORE(cmodel));
      
      items = rubrica_doc_get_items(doc);
      for (i = 0; i < items; i++)
	{
	  item = rubrica_doc_get_nth_item(doc, i);
	  
	  if (rubrica_item_belong_to_group(item, type))
	    rubrica_card_insert_item(doc, item, -1, NULL);
	}
    }
}



