/*
*  Rubrica
*  file: card_item.h
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_ITEM_H
#define _RUBRICA_ITEM_H

#include <gnome.h>

#include "item.h"
#include "rubrica.h"
#include "rubrica_type.h"


#define RUBRICA_ITEM_TYPE            (rubrica_item_get_type())
#define RUBRICA_ITEM(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), RUBRICA_ITEM_TYPE, RubricaItem))
#define RUBRICA_ITEM_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), RUBRICA_ITEM_TYPE, RubricaItemClass))
#define IS_RUBRICA_ITEM(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), RUBRICA_ITEM_TYPE))
#define IS_RUBRICA_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), RUBRICA_ITEM_TYPE))
#define RUBRICA_ITEM_GET_CLASS(obj)  (G_TYPE_CHECK_INSTANCE_GET_CLASS((obj), RUBRICA_ITEM_TYPE, RubricaItemClass))


typedef struct _RubricaItem        RubricaItem;
typedef struct _RubricaItemClass   RubricaItemClass;
typedef struct _RubricaItemPrivate RubricaItemPrivate;

typedef struct _RubricaCardPersonal  RubricaCardPersonal;
typedef struct _RubricaCardAddress   RubricaCardAddress;
typedef struct _RubricaCardTelNum    RubricaCardTelNum;
typedef struct _RubricaCardCompany   RubricaCardCompany;
typedef struct _RubricaCardWork      RubricaCardWork;
typedef struct _RubricaCardNotes     RubricaCardNotes;

typedef GList* RubricaGroupList;
typedef GList* RubricaCardAddresses;
typedef GList* RubricaCardNet;
typedef GList* RubricaCardTelephone;


struct _RubricaCardPersonal
{
  /* name */
  gchar*   first;
  gchar*   middle;
  gchar*   last;
  gchar*   nick;

  gchar*   prof;
  gchar*   prefix;
  gchar*   title;

  gchar*   photo;    /* contact's photo (uri)     */

  gboolean birth;    /* is contact birthday know? */
  time_t   bdate;    /* contact birthday          */
};


struct _RubricaCardAddress
{
  RubricaAddressType type;

  gchar* street;
  gchar* number;
  gchar* zip;
  gchar* city;
  gchar* province;
  gchar* state;
  gchar* country;
};


struct _RubricaCardTelNum {
  gchar* number;
  gchar* type;
};

struct _RubricaCardCompany
{
  gchar* name;
  gchar* street;
  gchar* number;
  gchar* zip;
  gchar* city;
  gchar* province;
  gchar* country;
  gchar* web;
  gchar* email;
  gchar* operator;              /* company's operator */
  gchar* fax;
  gchar* green;                 /* free phone number  */
  gchar* ccare;                 /* customer care phone number */
  gchar* notes;                 
};

struct _RubricaCardWork
{
  gchar* assigment;             /* contact role        */
  gchar* org;                   /* organization        */
  gchar* dep;                   /* department          */
  gchar* subdep;                /* sub department      */
  gchar* secretary;             /* contact secretary   */
  gchar* telephone;             /* secreatry telephone */
};

struct _RubricaCardNotes
{
  gboolean married;             /* is contact married?          */
  gboolean birth;               /* is spouse birthday known ?   */
  gboolean ann;                 /* is their anniversary known ? */

  time_t bdate;                 /* spouse's birthday */
  time_t adate;                 /* anniversary       */

  gchar* spouse;                /* spouse name    */
  gchar* children;              /* children notes */
  gchar* notes;                 /* other notes    */
  gchar* pubkey;                /* public key     */
};


struct _RubricaItem 
{
  GObject parent;

  gulong id;                        /* card's unique id */

  gboolean deletable;               /* true if card is marked deletable */
  gboolean deleted;                 /* is this a deleted card ?         */
  gboolean marked;                  /* true if card was marked          */

  RubricaDoc* father;               /* card's father    */

  RubricaRankType  rank;            /* card's rank   */
  RubricaGroupList groups;          /* card's groups */
 
  gchar* card;                      /* card's name   */

  RubricaCardPersonal   personal;
  RubricaCardAddresses  addresses;
  RubricaCardNet        web;
  RubricaCardNet        email;
  RubricaCardTelephone  phone;
  RubricaCardCompany    company;
  RubricaCardWork       work;
  RubricaCardNotes      notes;

  time_t created;                    /* card's creation time             */
  time_t changed;                    /* card's last change time          */
  gint position;                     /* item position in list   */

  RubricaItemPrivate* private;
};

struct _RubricaItemClass
{
  GObjectClass parent;
};



GType        rubrica_item_get_type        (void);

RubricaItem* rubrica_item_new             (RubricaDoc *doc);


void            rubrica_item_add             (RubricaItem* item);
void            rubrica_item_modify          (RubricaItem* item);
void            rubrica_item_delete          (RubricaItem* item);

gboolean        rubrica_item_need_id         (RubricaItem *item);
gulong          rubrica_item_get_id          (RubricaItem *item);
gulong          rubrica_item_ask_id          (RubricaItem *item);
void            rubrica_item_reject_id       (RubricaItem *item);

RubricaDoc*     rubrica_item_get_father      (RubricaItem *item);


gboolean        rubrica_item_is_deletable    (RubricaItem *item);
gboolean        rubrica_item_is_deleted      (RubricaItem *item);
gboolean        rubrica_item_is_marked       (RubricaItem *item);


RubricaRankType rubrica_item_get_rank        (RubricaItem *item);

time_t          rubrica_item_get_creation_time     (RubricaItem* item);
time_t          rubrica_item_get_modification_time (RubricaItem* item);

void            rubrica_item_set_modification_time (RubricaItem* item, 
						    time_t time);

RubricaGroupList rubrica_item_get_groups      (RubricaItem *item);
RubricaGroup*    rubrica_item_get_group       (RubricaGroupList groups);
//RubricaGroup*    rubrica_item_get_next_group  (RubricaGroupList groups);
gchar*           rubrica_item_get_group_label (RubricaGroup* group);
RubricaGroupType rubrica_item_get_group_type  (RubricaGroup* group);
gboolean         rubrica_item_belong_to_group (RubricaItem *item,
						    RubricaGroupType type);
void             rubrica_item_set_father      (RubricaItem *item,
						    RubricaDoc* father);


void             rubrica_item_set_id          (RubricaItem *item, 
						    gulong id);
void             rubrica_item_set_deletable   (RubricaItem *item,  
						    gboolean bool); 
void             rubrica_item_set_deleted     (RubricaItem *item, 
						    gboolean state); 
void             rubrica_item_set_marked      (RubricaItem *item, 
						    gboolean bool);

/*
gint             rubrica_item_get_position    (RubricaItem *item);




*/

void             rubrica_item_set_rank        (RubricaItem *item, 
						    RubricaRankType rank);
// void             rubrica_item_set_position    (RubricaItem *item,
//						    gint position);

// RubricaItem* rubrica_item_get_item_by_id  (RubricaDoc* doc, 
//						    gulong id);

RubricaItem* rubrica_item_modify_item   (RubricaDoc* doc,
						  RubricaItem* item);
RubricaItem* rubrica_item_retrieve_item (RubricaDoc* doc);

void rubrica_item_cut_item   ();
RubricaItem* rubrica_item_copy_item  (RubricaItem *item);
RubricaItem* rubrica_item_paste_item ();


void rubrica_item_build_xmltree   (RubricaDoc* doc);

/*  Free memory
*/
void rubrica_item_free_addresses (RubricaCardAddresses addresses);
void rubrica_item_free_address   (RubricaCardAddress*  address);
void rubrica_item_free           (RubricaItem *item);




#endif
