/*
*  Rubrica
*  file: menu.c
*
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <config.h>
#include <gnome.h>

#include "rubrica_type.h"
#include "callbacks.h"
#include "ui.h"
#include "prefer.h"
#include "menu.h"
#include "utils.h"


#define  RUBRICA_UNDERLINE(x)   gtk_label_set_use_underline (GTK_LABEL(((GtkToolbarChild*) (g_list_last(GTK_TOOLBAR (x)->children)->data))->label), TRUE);


void
rubrica_menu_create_toolbar(GtkWidget *app, GtkAccelGroup *group)
{
  GtkWidget *toolbar1;
  GtkWidget *toolbar2;
//  GtkWidget *toolbar3;

  RubricaToolbarStyle style;
  GtkWidget *bt_new;
  GtkWidget *bt_open;
  GtkWidget *bt_save;
  GtkWidget *bt_save_as;
  GtkWidget *bt_close;
  GtkWidget *bt_add;
  GtkWidget *bt_remove;
  GtkWidget *bt_modify;
  GtkWidget *bt_first;
  GtkWidget *bt_last;
  GtkWidget *bt_find;
//  GtkWidget *bt_cards;
//  GtkWidget *bt_account;
  GtkWidget *bt_quit;

  GtkWidget *tmp_icon;
  
  toolbar1 = gtk_toolbar_new ();
  rubrica_utils_add_toolbar(toolbar1);

  gnome_app_add_toolbar (GNOME_APP (app), GTK_TOOLBAR(toolbar1), "toolbar1",
			 BONOBO_DOCK_ITEM_BEH_NORMAL,
			 BONOBO_DOCK_TOP, 1, 0, 0);
//  gtk_container_set_border_width (GTK_CONTAINER (toolbar1), 1);
//  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_BOTH);
  
  bt_new = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar1),
				     GTK_STOCK_NEW,
				     _("New Rubrica document"),
				     _("New Rubrica document"), 
				     G_CALLBACK(button_new_cb), NULL, -1);
  gtk_widget_show (bt_new);
  
  bt_open = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar1),
				      GTK_STOCK_OPEN,
				      _("Open a document"),
				      _("Open a document"),
				      G_CALLBACK(button_open_cb), NULL, -1);
  gtk_widget_show (bt_open);
  
  bt_save = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar1),
				      GTK_STOCK_SAVE,
				      _("Save document"), 
				      _("Save document"), 
				      G_CALLBACK(button_save_cb), NULL, -1);
  gtk_widget_show (bt_save);
  

  bt_save_as = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar1),
					 GTK_STOCK_SAVE_AS,
					 _("Save as..."),
					 _("Save as..."),
					 G_CALLBACK(button_save_as_cb), 
					 NULL, -1);
  gtk_widget_show (bt_save_as);

  bt_close = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar1),
				       GTK_STOCK_CLOSE,
				       _("Close document"),
				       _("Close document"),
				       G_CALLBACK(button_close_cb), NULL, -1);
  gtk_widget_show (bt_close);
  
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar1));
  
  bt_quit = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar1),
				      GTK_STOCK_QUIT,
				      _("Quit Rubrica"),
				      _("Quit Rubrica"),
				      G_CALLBACK(button_exit_cb), NULL, -1);

  gtk_widget_show (bt_quit);

  toolbar2 = gtk_toolbar_new ();
  rubrica_utils_add_toolbar(toolbar2);
  gtk_widget_show (toolbar2);
  gnome_app_add_toolbar (GNOME_APP(app), GTK_TOOLBAR(toolbar2), "toolbar2",
			 BONOBO_DOCK_ITEM_BEH_NORMAL,
			 BONOBO_DOCK_TOP, 1, 1, 0);
  
  tmp_icon = rubrica_ui_image_new("add.png");
  bt_add = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
				       GTK_TOOLBAR_CHILD_BUTTON, 
				       NULL,
				       _("_Add"), 
				       _("Add an item"),
				       NULL, tmp_icon,
				       G_CALLBACK(button_add_cb), NULL);
  RUBRICA_UNDERLINE(toolbar2);
  gtk_widget_show (bt_add);
  
  tmp_icon = rubrica_ui_image_new("remove.png");  
  bt_remove = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
					  GTK_TOOLBAR_CHILD_BUTTON,
					  NULL,
					  _("R_emove"),
					  _("Remove an item"), 
					  NULL, tmp_icon, 
					  G_CALLBACK(button_remove_cb), NULL);
  RUBRICA_UNDERLINE(toolbar2);  
  gtk_widget_show (bt_remove);
  
  tmp_icon = gtk_image_new_from_stock (GTK_STOCK_CONVERT, 
				       gtk_toolbar_get_icon_size(GTK_TOOLBAR(toolbar2)));
  bt_modify = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
					  GTK_TOOLBAR_CHILD_BUTTON,
					  NULL,
					  _("Modif_y"),
					  _("Modify the selected item"),
					  NULL, tmp_icon, 
					  G_CALLBACK(button_modify_cb), NULL);
  RUBRICA_UNDERLINE(toolbar2);
  gtk_widget_show (bt_modify);
  
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar2));
  
  tmp_icon = gtk_image_new_from_stock (GTK_STOCK_GOTO_TOP, 
				       gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar2)));
  bt_first = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
					 GTK_TOOLBAR_CHILD_BUTTON,
					 NULL,
					 _("Firs_t"),
					 _("Show the first item"), NULL,
					 tmp_icon, 
					 G_CALLBACK(button_browse_cb), 
					 GINT_TO_POINTER(RB_FIRST_CARD));
  RUBRICA_UNDERLINE(toolbar2);
  gtk_widget_show (bt_first);
  
  tmp_icon = gtk_image_new_from_stock (GTK_STOCK_GOTO_BOTTOM, 
				       gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar2)));
  bt_last = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
					GTK_TOOLBAR_CHILD_BUTTON,
					NULL,
					_("_Last"),
					_("Show the last item"), NULL,
					tmp_icon, 
					G_CALLBACK(button_browse_cb),
					GINT_TO_POINTER(RB_LAST_CARD));
  RUBRICA_UNDERLINE(toolbar2);
  gtk_widget_show (bt_last);
  
  bt_find = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar2),
				      GTK_STOCK_FIND,
				      _("Search..."),
				      NULL,
				      G_CALLBACK(button_find_cb), 
				      NULL, -1);
  gtk_widget_show (bt_find);  
  
/*
  toolbar3 = gtk_toolbar_new ();
  rubrica_utils_add_toolbar(toolbar3);
  gtk_widget_show (toolbar3);
  gnome_app_add_toolbar (GNOME_APP(app), GTK_TOOLBAR (toolbar3), "toolbar3",
			 BONOBO_DOCK_ITEM_BEH_NORMAL,
			 BONOBO_DOCK_TOP, 1, 2, 0);
  
  tmp_icon = rubrica_ui_image_new("contacts.png");  
  bt_cards = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar3),
					 GTK_TOOLBAR_CHILD_BUTTON,
					 NULL,
					 _("Ca_rds"),
					 _("Card view"), NULL,
					 tmp_icon, 
					 G_CALLBACK(button_cards_cb), 
					 NULL);
  RUBRICA_UNDERLINE(toolbar3);
  gtk_widget_show (bt_cards);
  
  tmp_icon = gtk_image_new_from_stock (GTK_STOCK_INDEX, 
				       gtk_toolbar_get_icon_size(GTK_TOOLBAR (toolbar3)));
  bt_account = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar3),
					   GTK_TOOLBAR_CHILD_BUTTON,
					   NULL,
					   _("Acco_unt"),
					   _("Account view"), NULL,
					   tmp_icon, 
					   G_CALLBACK(button_account_cb), 
					   NULL);
  RUBRICA_UNDERLINE(toolbar3);
  gtk_widget_show (bt_account);
*/
  
  style = rubrica_preferences_get_toolbarstyle();
  if (style != RTS_GNOME_DEFAULT)
    {
      gtk_toolbar_set_style(GTK_TOOLBAR(toolbar1), style);
      gtk_toolbar_set_style(GTK_TOOLBAR(toolbar2), style);
//      gtk_toolbar_set_style(GTK_TOOLBAR(toolbar3), style);
    }
  else
    {
      style = rubrica_preferences_get_desktop_toolbarstyle();
      
      gtk_toolbar_set_style(GTK_TOOLBAR(toolbar1), style);
      gtk_toolbar_set_style(GTK_TOOLBAR(toolbar2), style);
//      gtk_toolbar_set_style(GTK_TOOLBAR(toolbar3), style);
    }

//  return (GtkWidget *) toolbar;
}


GtkWidget *rubrica_menu_popup_new(RubricaMenu *rm)
{
  g_return_val_if_fail(rm != NULL, NULL);

  return rubrica_ui_popup_menu_new(rm);
}

void rubrica_menu_attach_popup(GtkWidget *parent, GtkMenu *menu)
{
  gtk_menu_attach_to_widget(GTK_MENU(menu), GTK_WIDGET(parent), 
			    rubrica_menu_detach_menu);
  
  g_signal_connect_swapped (GTK_OBJECT (parent), "button_press_event",
			    G_CALLBACK (rubrica_menu_do_popup), 
			    GTK_OBJECT (menu));  
}


gint rubrica_menu_do_popup(GtkWidget *widget, GdkEvent *event)
{
  GtkMenu *menu;
  GdkEventButton *event_button;
  
  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_MENU (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);
  
  menu = GTK_MENU (widget);

  if (event->type == GDK_BUTTON_PRESS)
    {
      event_button = (GdkEventButton *) event;
      if (event_button->button == 3)
	{
	  gtk_widget_show_all(GTK_WIDGET(menu));
	  gtk_menu_popup (menu, NULL, NULL, NULL, NULL, 
			  event_button->button, event_button->time);
	  return TRUE;
	}
    }
  
  return FALSE;  
}

void rubrica_menu_detach_menu(GtkWidget *attach_widget, GtkMenu *menu)
{
  /* do nothing */ 
}
