/*
*  program name: Rubrica -- http://digilander.iol.it/nfragale/download.html
*  file: prefer_int.c
*  
*  build the graphic interface for preferences dialog
*
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <config.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <gnome.h>
#include <glade/glade.h>
#include <gconf/gconf-client.h>
#include <gconf/gconf-value.h>
#include <gconf/gconf-schema.h>

#include "prefer.h"
#include "prefer_cb.h"
#include "prefer_int.h"
#include "prefer_configure.h"
#include "ui.h"
#include "rubrica_type.h"
#include "dialog.h"
#include "utils.h"
#include "view.h"
#include "data_view.h"

static GtkWidget*   dialog   = NULL;
static GtkWidget*   notebook = NULL;
static GtkTooltips* tooltips;

gint cards;
// gint accounts;
gint personal;
gint company;
gint note;


RubricaPreferTree file[] = {
//  {N_("Defaults"),      "folder.png",   1, NULL},
  {N_("File Settings"), "filesett.png", 1, NULL},
  {NULL}    
};


RubricaPreferTree general[] =  {
  {N_("Appearance"),          "misc.png",   3, NULL},
  {N_("Fonts and colours"),   "misc.png",   4, NULL},  
  {N_("Search and ordering"), "search.png", 5, NULL},
//  {N_("User's categories"),   "misc.png",   6, NULL},
  {NULL}
};


RubricaPreferTree view[] =  {
  {N_("Columns view"),       "view-column.png", 8, NULL},
  {N_("Personal Data view"), "personal.png",    9, NULL},
  {N_("Company Data view"),  "company.png",    10, NULL},
  {N_("Notes view"),         "notes.png",      11, NULL},
//  {N_("Accounts view"),      "accounts.png",   12, NULL},
  {NULL}
};


RubricaPreferTree crypto[] =  {
  {N_("Cryptography"),   "crypto.png",  14, NULL},
  {NULL}
};

/*
RubricaPreferTree translation[] =  {
  {N_("Rubrica Dictionary"),   "dictionary.png", 16, NULL},
  {NULL}
};
*/

RubricaPreferTree rpi[] =  {
  {N_("File"),         "file.png",         0, file},
  {N_("General"),      "general.png",      2, general},
  {N_("View"),         "view.png",         7, view},
  {N_("Cryptography"), "crypto.png",      13, crypto},
//  {N_("Translation"),  "translation.png", 15, translation},
  {NULL}
};


  
RubricaPreferItem card_columns[] = {
//  {N_("Categories pixmap"), RUBRICA_KEY_SCC_PIXMAP,     TRUE},
  {N_("Card"),              RUBRICA_KEY_SCC_CARD,       TRUE},
  {N_("First name"),        RUBRICA_KEY_SCC_FIRST_NAME, TRUE},
  {N_("Last name"),         RUBRICA_KEY_SCC_LAST_NAME,  TRUE},
  {N_("Profession"),        RUBRICA_KEY_SCC_PROFESSION, FALSE},
  {N_("City"),              RUBRICA_KEY_SCC_CITY,       FALSE},
  {N_("Country"),           RUBRICA_KEY_SCC_COUNTRY,    FALSE},
  {N_("Web"),               RUBRICA_KEY_SCC_WEB,        TRUE},
  {N_("Email"),             RUBRICA_KEY_SCC_EMAIL,      TRUE},
  {N_("Telephone"),         RUBRICA_KEY_SCC_TELEPHONE,  FALSE},
  {N_("Company"),           RUBRICA_KEY_SCC_COMPANY,    FALSE},
  {N_("Assigment"),         RUBRICA_KEY_SCC_ASSIGMENT,  FALSE},
//  {N_("Categories"),        RUBRICA_KEY_SCC_CATEGORIES, FALSE},
  {NULL}
};


/*
RubricaPreferItem account_columns[] = {
  {N_("Marked accounts"),   RUBRICA_KEY_SAC_MARKED, TRUE},
  {N_("Account name"),      RUBRICA_KEY_SAC_NAME,   TRUE},
  {N_("Web url"),     	    RUBRICA_KEY_SAC_WEB,    TRUE},
  {N_("Email"),       	    RUBRICA_KEY_SAC_EMAIL,  FALSE},
  {N_("Description"), 	    RUBRICA_KEY_SAC_DESC,   FALSE},
  {NULL}
};
*/

RubricaPreferItem personal_data[] = {
  {N_("First name"),        RUBRICA_KEY_SCD_FIRST_NAME,  TRUE},
  {N_("Middle name"),       RUBRICA_KEY_SCD_MIDDLE_NAME, FALSE},
  {N_("Last name"),         RUBRICA_KEY_SCD_LAST_NAME,   TRUE},
  {N_("Nickname"),          RUBRICA_KEY_SCD_NICKNAME,    TRUE},
  {N_("Profession"),        RUBRICA_KEY_SCD_PROFESSION,  TRUE},
  {N_("Name prefix"),       RUBRICA_KEY_SCD_NAME_PREFIX, FALSE},
  {N_("Title"),             RUBRICA_KEY_SCD_TITLE,       FALSE},
  {N_("Birthday"),          RUBRICA_KEY_SCD_BIRTHDAY,    TRUE},
  {N_("Street"),            RUBRICA_KEY_SCD_STREET,      TRUE},
  {N_("Street number"),     RUBRICA_KEY_SCD_STREET_NUM,  FALSE},
  {N_("Zip code"),          RUBRICA_KEY_SCD_ZIP_CODE,    FALSE},
  {N_("City"),              RUBRICA_KEY_SCD_CITY,        TRUE},
  {N_("Province"),          RUBRICA_KEY_SCD_PROVINCE,    TRUE},
  {N_("State"),             RUBRICA_KEY_SCD_STATE,       FALSE},
  {N_("Country"),           RUBRICA_KEY_SCD_COUNTRY,     TRUE},
  {N_("Web urls"),          RUBRICA_KEY_SCD_WEB,         TRUE},
  {N_("Emails"),            RUBRICA_KEY_SCD_EMAIL,       TRUE},
  {N_("Telephone numbers"), RUBRICA_KEY_SCD_TELEPHONE,   TRUE},
  {NULL}
};


RubricaPreferItem company_data[] = {
  {N_("Company name"),          RUBRICA_KEY_SCD_COMPANY,            TRUE},
  {N_("Company street"),        RUBRICA_KEY_SCD_COMPANY_STREET,     TRUE},
  {N_("Street number"),         RUBRICA_KEY_SCD_COMPANY_STREET_NUM, FALSE},
  {N_("Zip code"),              RUBRICA_KEY_SCD_COMPANY_ZIP_CODE,   FALSE},
  {N_("City"),                  RUBRICA_KEY_SCD_COMPANY_CITY,       TRUE},
  {N_("Province"),              RUBRICA_KEY_SCD_COMPANY_PROVINCE,   TRUE},
  {N_("Country"),               RUBRICA_KEY_SCD_COMPANY_COUNTRY ,   TRUE},
  {N_("Web url"),               RUBRICA_KEY_SCD_COMPANY_WEB,        TRUE},
  {N_("Email"),                 RUBRICA_KEY_SCD_COMPANY_EMAIL,      TRUE},
  {N_("Operator"),              RUBRICA_KEY_SCD_OPERATOR,           TRUE},
  {N_("Fax"),                   RUBRICA_KEY_SCD_FAX,                TRUE},
  {N_("Green"),                 RUBRICA_KEY_SCD_GREEN,              TRUE},
  {N_("Customer care"),         RUBRICA_KEY_SCD_CUSTOMER_CARE,      TRUE},
  {N_("Company notes"),         RUBRICA_KEY_SCD_COMPANY_NOTE,       FALSE},
  {N_("Contact assigment"),     RUBRICA_KEY_SCD_ASSIGMENT,          TRUE},
  {N_("Organizzation"),         RUBRICA_KEY_SCD_ORG,                FALSE},
  {N_("Department"),            RUBRICA_KEY_SCD_DEP,                FALSE},
  {N_("Sub department"),        RUBRICA_KEY_SCD_SUB_DEP,            FALSE},
  {N_("Secretary (company contact)"), RUBRICA_KEY_SCD_SECRETARY,    FALSE},
  {N_("Secretary telephone"),         RUBRICA_KEY_SCD_SECR_PHONE,   FALSE},
  {NULL}
};

RubricaPreferItem note_data[] = {
  {N_("Contact is married"),   RUBRICA_KEY_SCD_MARRIED,      TRUE},
  {N_("Spouse name"),          RUBRICA_KEY_SCD_SPOUSE,       TRUE},
  {N_("Spouse birthday"),      RUBRICA_KEY_SCD_SPOUSE_BIRTH, TRUE},
  {N_("Marriage anniversary"), RUBRICA_KEY_SCD_ANNIVERSARY,  TRUE},
//  {N_("Contact has children"), RUBRICA_KEY_SCD_HAS_CHILDREN, FALSE},
  {N_("Children's notes"),     RUBRICA_KEY_SCD_CHILDREN,     FALSE},
//  {N_("Hobbies"),              RUBRICA_KEY_SCD_HOBBIES,      TRUE},
  {N_("Notes"),                RUBRICA_KEY_SCD_NOTE,         TRUE},
  {N_("Public key"),           RUBRICA_KEY_SCD_PUBLIC_KEY,   TRUE},
  {NULL}
};


RubricaOptionType search_on_option[] = {
  {"", N_("Card"),          SEARCH_CARD},
  {"", N_("First Name"),    SEARCH_FIRST_NAME},
  {"", N_("Last Name"),     SEARCH_LAST_NAME},
  {"", N_("Profession"),    SEARCH_PROFESSION},
  {"", N_("City"),          SEARCH_CITY},
  {"", N_("Country"),       SEARCH_COUNTRY},
  {"", N_("Web Address"),   SEARCH_WEB_ADDRESS},
  {"", N_("Email Address"), SEARCH_EMAIL_ADDRESS},
  {"", N_("Telephone"),     SEARCH_TELEPHONE},
  {"", N_("Company"),       SEARCH_COMPANY},
  {"", N_("Categories"),    SEARCH_CATEGORIES},
  {NULL, NULL}
};

RubricaOptionType sort_option[] = {
  {"", N_("in alphabetic order"), NO_GROUP},
  {"", N_("by creation date"),    NO_GROUP},
  {"", N_("by last change date"), NO_GROUP},
  {"", N_("by card importance"),  NO_GROUP},
  {NULL, NULL}
};

RubricaOptionType toolbar_option[] = {
  {"", N_("only icons"), NO_GROUP},
  {"", N_("only text"),  NO_GROUP},
  {"", N_("both text and icons"), NO_GROUP},
  {"", N_("both text and icons alongside each other"), NO_GROUP},
//  {"", N_("according to Gnome default"), NO_GROUP},
  {NULL}
};

/*
RubricaDictionaryItem personal_dic[] = {
  {N_("First name"),    RUBRICA_KEY_DICT_FIRST_NAME},
  {N_("Last name"),     RUBRICA_KEY_DICT_LAST_NAME},
  {N_("Middle name"),   RUBRICA_KEY_DICT_MIDDLE_NAME},
  {N_("Nickname"),      RUBRICA_KEY_DICT_NICKNAME},
  {N_("Name Prefix"),   RUBRICA_KEY_DICT_NAME_PREFIX},
  {N_("Profession"),    RUBRICA_KEY_DICT_PROFESSION},
  {N_("Title"),         RUBRICA_KEY_DICT_TITLE},
  {N_("Card"),          RUBRICA_KEY_DICT_CARD},
  {N_("Categories"),    RUBRICA_KEY_DICT_CARD_GROUP},
  {N_("Personal Data"), RUBRICA_KEY_DICT_DATA},
  {NULL, NULL}
};

RubricaDictionaryItem address_dic[] = {
  {N_("Address"),       RUBRICA_KEY_DICT_ADDRESS},
  {N_("Street"),        RUBRICA_KEY_DICT_STREET},
  {N_("Street number"), RUBRICA_KEY_DICT_STREET_NUMBER},
  {N_("Zip code"),      RUBRICA_KEY_DICT_ZIP_CODE},
  {N_("City"),          RUBRICA_KEY_DICT_CITY},
  {N_("Province"),      RUBRICA_KEY_DICT_PROVINCE},
  {N_("Country"),       RUBRICA_KEY_DICT_COUNTRY},
  {N_("State"),         RUBRICA_KEY_DICT_STATE},
  {NULL, NULL} 
};

RubricaDictionaryItem communication_dic[] = {
  {N_("Web addresses"),   RUBRICA_KEY_DICT_WEB_ADDRESS},
  {N_("Web"),             RUBRICA_KEY_DICT_WEB},
  {N_("Email addresses"), RUBRICA_KEY_DICT_EMAIL_ADDRESS},
  {N_("Email"),           RUBRICA_KEY_DICT_EMAIL},
  {N_("Telephone list"),  RUBRICA_KEY_DICT_TELEPHONE_LIST},
  {N_("Telephone"),       RUBRICA_KEY_DICT_TELEPHONE_NUMBER},
  {N_("Type"),            RUBRICA_KEY_DICT_TELEPHONE_TYPE},
  {N_("Work"),            RUBRICA_KEY_DICT_TELEPHONE_WORK},
  {N_("Home"),            RUBRICA_KEY_DICT_TELEPHONE_HOME},
  {N_("Fax"),             RUBRICA_KEY_DICT_TELEPHONE_FAX},
  {N_("Cellular"),        RUBRICA_KEY_DICT_TELEPHONE_CELL},
  {N_("Other"),           RUBRICA_KEY_DICT_TELEPHONE_OTHER},
  {NULL, NULL} 
};

RubricaDictionaryItem company_dic[] = {
  {N_("Company Name"),    RUBRICA_KEY_DICT_COMPANY_NAME},
  {N_("Company"),         RUBRICA_KEY_DICT_COMPANY},
  {N_("Operator"),        RUBRICA_KEY_DICT_OPERATOR},
  {N_("Green"),           RUBRICA_KEY_DICT_GREEN},
  {N_("Customer care"),   RUBRICA_KEY_DICT_CUSTOMER_CARE},
  {N_("Secretary"),       RUBRICA_KEY_DICT_SECRETARY},
  {N_("Secretary Phone"), RUBRICA_KEY_DICT_SECRETARY_PHONE},
  {N_("Assigment"),       RUBRICA_KEY_DICT_ASSIGMENT},
  {N_("Department"),      RUBRICA_KEY_DICT_DEPARTMENT},
  {N_("Sub department"),  RUBRICA_KEY_DICT_SUB_DEPARTMENT},
  {N_("Organization"),    RUBRICA_KEY_DICT_ORGANIZATION},
  {NULL, NULL} 
};

RubricaDictionaryItem notes_dic[] = {
  {N_("Spouse Name"), RUBRICA_KEY_DICT_SPOUSE_NAME},
  {N_("Birthday"),    RUBRICA_KEY_DICT_BIRTHDAY},
  {N_("Children"),    RUBRICA_KEY_DICT_CHILDREN},
  {N_("Married"),     RUBRICA_KEY_DICT_MARRIED},
  {N_("Anniversary"), RUBRICA_KEY_DICT_ANNIVERSARY},
  {N_("Note"),        RUBRICA_KEY_DICT_NOTES},
  {N_("Public Key"),  RUBRICA_KEY_DICT_PUBLIC_KEY},
  {NULL, NULL} 
};
*/

GtkWidget* rubrica_preferences_get_preferencesbox(void)
{
  return (GtkWidget*) dialog;
}


GtkWidget* rubrica_preferences_get_notebook(void)
{
  return (GtkWidget*) notebook;
}

/*	create the model for preferences index tree 
	and populate tree	
*/
GtkTreeModel*
rubrica_preferences_create_tree_model(GConfClient *client)
{
  GtkTreeStore *model;
  GtkTreeIter iter;
  RubricaPreferTree *mainitem = rpi;

  model = gtk_tree_store_new(RP_LAST_COLUMN, 
			     GDK_TYPE_PIXBUF,  /* pixmap      */
			     G_TYPE_STRING,    /* string      */ 
			     G_TYPE_INT);      /* page number */

  while(mainitem->label)
    {
      RubricaPreferTree *child = mainitem->children;
      GdkPixbuf *image;
     
      image = rubrica_ui_pixbuf_new(mainitem->str);

      gtk_tree_store_append(GTK_TREE_STORE(model), &iter, NULL);
      gtk_tree_store_set(GTK_TREE_STORE(model), &iter, 
			 RP_PIXMAP_COLUMN, image, 
			 RP_STRING_COLUMN, _(mainitem->label), 
			 RP_PAGE_COLUMN,   mainitem->page, 
			 -1);
			 
      while (child->label)
	{
	  GtkTreeIter child_iter;

	  image = rubrica_ui_pixbuf_new(child->str);
	  
	  gtk_tree_store_append(GTK_TREE_STORE(model), &child_iter, &iter);
	  gtk_tree_store_set(GTK_TREE_STORE(model), &child_iter, 
			     RP_PIXMAP_COLUMN, image, 
			     RP_STRING_COLUMN, _(child->label), 
			     RP_PAGE_COLUMN,   child->page, 
			     -1);	  

	  child++;
	}

      mainitem++;
    }

  return GTK_TREE_MODEL(model);
}




void
rubrica_preferences_configure_tree_view(GtkTreeView *treeview)
{
  GtkTreeSelection *selection;
  
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));

  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(treeview));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  rubrica_preferences_add_columns(GTK_TREE_VIEW(treeview), PREF_TREE);
  gtk_tree_view_expand_all(GTK_TREE_VIEW(treeview));

  g_signal_connect(G_OBJECT(selection), "changed", 
		   G_CALLBACK(rubrica_preferences_row_selected), treeview);
}

/* create a list model for card's column manage
 
GtkTreeModel*
rubrica_preferences_create_dict_list(GConfClient *client, RubricaDictType type)
{
  GtkListStore *model;
  GtkTreeIter iter;
  RubricaDictionaryItem *dict = NULL;

  model = gtk_list_store_new(RP_LAST_DICT, G_TYPE_INT, 
			     G_TYPE_STRING, G_TYPE_STRING);

  switch (type)
    {
    case PERS_DICT:
      dict = personal_dic;
      break;
      
    case ADD_DICT:
      dict = address_dic;
      break;
      
    case COMM_DICT:
      dict = communication_dic;
      break;

    case COMP_DICT:
      dict = company_dic;
      break;
        
    case NOTES_DICT:
      dict = notes_dic;
      break;

    default:
      break;
    }
  
  while (dict->label)
    {
      gchar* str;

      str = gconf_client_get_string(client, dict->key, NULL);
      
      if(!str) str = "";
      
      gtk_list_store_append(model, &iter);
      gtk_list_store_set(model, &iter, 
			 RP_BOOL_COL, TRUE,
			 RP_STR_COL, _(dict->label),
			 RP_LOC_COL, str,
			 -1);

      dict++;			 
    }

  return GTK_TREE_MODEL(model);    
}
*/
  

GtkTreeModel*
rubrica_preferences_create_list_model(GConfClient *client, 
				      RubricaTreeType type)
{
  GtkListStore *model;
  GtkTreeIter iter;  
  RubricaPreferItem *listitem = NULL; 

  model = gtk_list_store_new(RP_VIEW_LAST_COLUMN,
			     G_TYPE_BOOLEAN,
			     G_TYPE_STRING, 
			     G_TYPE_STRING);

  switch (type)
    {
    case PREF_CARD_LIST:
      listitem = card_columns;
      break;

/*
    case PREF_ACCOUNT_LIST:
      listitem = account_columns;
      break;
*/

    case PREF_PERSONAL_DATA_LIST:
      listitem = personal_data;
      break;

    case PREF_COMPANY_DATA_LIST:
      listitem = company_data;     
      break;

    case PREF_NOTE_DATA_LIST:
      listitem = note_data;
      break;

/*	
    case PREF_ACCOUNT_DATA_LIST:      
      break;
*/
	
    default:
      g_warning("can't create model in "
		"rubrica_preferences_create_list_model(). Type is: %d",
		(RubricaTreeType) type);

      return NULL;;
      break;
    }

  while (listitem->label)
    {
      GConfValue  *value;
      gchar *key;
      gboolean bool;
      
      key = listitem->key;
      value = gconf_client_get(client, key, NULL);
      if (value == NULL || value->type != GCONF_VALUE_BOOL)
	{
	  bool = listitem->bool;
	  gconf_client_set_bool(client, key, bool, NULL);
	}
      else 
	bool = gconf_client_get_bool(client, key, NULL); 
      
      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
			 RP_VIEW_CHECK_COLUMN, bool,
			 RP_VIEW_LABEL_COLUMN, _(listitem->label), 
			 RP_VIEW_KEY_COLUMN,   key,
			 -1);
      listitem++;
    }

  return GTK_TREE_MODEL(model);    
}


/* create a list view using a card or account or data model 
*/
GtkWidget*
rubrica_preferences_create_list_view_with_model(GtkTreeModel *model, 
						RubricaTreeType type)
{
  GtkWidget *listview;
  GtkTreeSelection *selection;

  listview  = gtk_tree_view_new_with_model(model);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(listview));
  gtk_widget_show(listview);

  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  if (type == PREF_TREE)
    {
      gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(listview), FALSE);
      gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(listview), FALSE);
    }
  else
    {
      gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(listview), TRUE);
      gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(listview), TRUE);
    }

  rubrica_preferences_add_columns(GTK_TREE_VIEW(listview), type);
  gtk_tree_view_expand_all(GTK_TREE_VIEW(listview));

  return listview;
}


/* add the rigth columns to the list view 
*/
void 
rubrica_preferences_add_columns(GtkTreeView *tree, RubricaTreeType type)
{
  GtkCellRenderer *cell;
  GtkTreeModel *model;
  GtkTreeViewColumn *column;
  gint col;

  model = gtk_tree_view_get_model(tree);

  switch (type)
    {
    case PREF_TREE:
      cell = gtk_cell_renderer_pixbuf_new();
      gtk_tree_view_insert_column_with_attributes(tree, -1, NULL, cell, 
						  "pixbuf",
						  RP_PIXMAP_COLUMN, NULL);
      
      cell = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes(tree, -1, NULL, cell, 
						  "text", 
						  RP_STRING_COLUMN, NULL); 
      break;

    case PREF_CARD_LIST:
//    case PREF_ACCOUNT_LIST:
    case PREF_PERSONAL_DATA_LIST:
    case PREF_PERSONAL_MEDIA_LIST:
    case PREF_COMPANY_DATA_LIST:
    case PREF_NOTE_DATA_LIST:
//    case PREF_ACCOUNT_DATA_LIST:
      cell = gtk_cell_renderer_toggle_new();
      col  = gtk_tree_view_insert_column_with_attributes(tree, -1, 
							 _("Show"), 
							 cell, "active", 
							 RP_VIEW_CHECK_COLUMN,
							 NULL); 
      column = gtk_tree_view_get_column(tree, col-1);
      gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column),
				      GTK_TREE_VIEW_COLUMN_FIXED);
      gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(column), 50);
      gtk_tree_view_column_set_max_width (GTK_TREE_VIEW_COLUMN(column), 50);
     
      g_signal_connect(G_OBJECT(cell), "toggled", 
		       G_CALLBACK(rubrica_preferences_manage_columns), model);
      
      cell = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes(tree, -1, 
						  _("Description"), 
						  cell, "text", 
						  RP_VIEW_LABEL_COLUMN, NULL); 
      break;
      
    case PERS_DICT:
    case ADD_DICT:
    case COMM_DICT:
    case COMP_DICT:
    case NOTES_DICT:
      cell = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes(tree, -1, _("String"), cell, 
						  "text",  RP_STR_COL, NULL); 
     
      cell = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes(tree, -1, 
						  _("Localization"), cell, 
						  "text", RP_LOC_COL, 
						  "editable", RP_BOOL_COL,
						  NULL);       
      g_signal_connect(G_OBJECT(cell), "edited", 
		       G_CALLBACK(rubrica_preferences_cell_edit_cb), model);
      break;

    default:
      break;
    }
}


/*    list/tree view callbacks 
*/
void 
rubrica_preferences_manage_columns(GtkCellRendererToggle *cell, 
				   gchar *path_str, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreePath *path;
  GConfClient *client;
  gboolean status;      /* check status */
  gchar *key;

  model  = (GtkTreeModel *) data;
  path   = gtk_tree_path_new_from_string(path_str);
  client = gconf_client_get_default ();

  gtk_tree_model_get_iter (model, &iter, path);
  gtk_tree_model_get (model, &iter,
		      RP_VIEW_CHECK_COLUMN, &status, 
		      RP_VIEW_KEY_COLUMN,   &key, 
		      -1);
  status ^= 1;
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, RP_VIEW_CHECK_COLUMN, 
		     status, -1);
  gconf_client_set_bool (client, key, status, NULL);
}


void 
rubrica_preferences_row_selected(GtkTreeSelection *select, gpointer data)
{
  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(data));
  GtkTreeIter iter;
  gint page;
  
  if (gtk_tree_selection_get_selected(select, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, RP_PAGE_COLUMN, &page, -1); 

      gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), page);      
    }  
}

void       
rubrica_preferences_cell_edit_cb (GtkCellRendererText *cell, 
				  const gchar *path_string,
				  const gchar *new_text, 
				  gpointer data)
{
  GtkTreeModel *model; 
  GtkTreePath  *path; 
  GtkTreeIter iter;
  gchar *old_text;
  
  model  = (GtkTreeModel *) data;
  path   = gtk_tree_path_new_from_string(path_string);

  gtk_tree_model_get_iter(model, &iter, path);

  gtk_tree_model_get(model, &iter, RP_LOC_COL, &old_text, -1);
  g_free (old_text);
  
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, RP_LOC_COL, 
                     new_text, -1);
}


void rubrica_preferences_configure_list(GtkWidget *tree, 
					GtkTreeModel *model, 
					RubricaTreeType type)
{
  GtkTreeSelection *selection;
  
  gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(model));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree), TRUE);
  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
  rubrica_preferences_add_columns(GTK_TREE_VIEW(tree), type);  
}

void rubrica_preferences_configure(GtkWidget *app, 
				   GConfClient *client,
				   GladeXML* xml)
{
  gchar *str;
  GtkTreeModel* model;

  /* default page widget */		
  GtkWidget* check_path;         // show/hide document path
  GtkWidget* check_doc;          // load document at rubrica start
  GtkWidget* fileentry_doc;      // gnome fileentry for loadable document
  GtkWidget* entry_doc;          // file entry for loadable document
  GtkWidget* check_folder;       // save documents in default folder
  GtkWidget* fileentry_folder;   // gnome fileentry for default folder
  GtkWidget* entry_folder;       // entry for default folder
	
  /* file page widget */
  GtkWidget *check_backup;       // make backup
  GtkWidget *check_compress;     // compress documents
  GtkWidget *check_autosave;     // enable autosave
  GtkWidget *radio_inc;          // backup type is incremental
  GtkWidget *radio_bak;          // backup type is .bak
  GtkWidget *spin_rate;          // compression rate
  GtkWidget *spin_autosave;      // time between autosave
  GtkWidget* box_backup;         // radio backup buttons container
  GtkWidget* box_compress;       // compress spin/labes container
  GtkWidget* box_autosave;       // autosave spin/label container
	
  GtkWidget* check_sort;         // sort ascending/descending
  GtkWidget* check_case;         // case sensitive/insensitive search   
  GtkWidget* check_string;       // search on complete/incomplete string
  GtkWidget* sort_menu;          // "sort by" options
  GtkWidget* search_menu;        // "search on" oprtions
  
  GtkWidget* width_spin;         // main window width
  GtkWidget* length_spin;        // main window length
  GtkWidget* check_tooltips;     // enable tooltips
  GtkWidget* check_exit;         // exit without confirming
  GtkWidget* toolbar_menu;       // toolbar style
  
  GtkWidget* header_fp;          // data header font picker
  GtkWidget* data_fp;            // data font picker
  GtkWidget* header_cp;          // data header color picker
  GtkWidget* data_cp;            // data color picker
  GtkWidget* radio_r2l;          // rigth to left gravity
  GtkWidget* radio_l2r;          // left to rigth gravity
  GtkWidget* gravity_frame;      // gravity container, unsensible now
  
  GtkWidget* card_cols;          // card columns
//  GtkWidget* account_cols;       // account columns
  GtkWidget* personal_list;      // personal data hide/show list
  GtkWidget* company_list;       // company data hide/show list
  GtkWidget* notes_list;         // notes hide/show list
//  GtkWidget* account_list;       // account data hide/show list
  
//  GtkWidget* radio_external;     // external dictionary
//  GtkWidget* radio_internal;     // internal dictionary
//  GtkWidget* radio_italian;      // italian dictionary 
//  GtkWidget* radio_english;      // english dictionaty
//  GtkWidget* box_dictionary;     // italian/english radio button container
//  GtkWidget* box_notebook;       // dictionary notebook container 
//  GtkWidget* entry_dict_name;    // dictionary i/o entry

/*
  GtkWidget* personal_view;
  GtkWidget* address_view;
  GtkWidget* communic_view;
  GtkWidget* company_view;
  GtkWidget* notes_view;
*/

//  GtkWidget* load;
//  GtkWidget* save;
//  GtkWidget* rules;

  GtkWidget* tmpbox;


  /*     file settings page 
   */

  /*    documents and folder frame
  */
  check_path = glade_xml_get_widget(xml, "show_check");
  rubrica_preferences_configure_check(client, check_path, RUBRICA_KEY_SHOW); 

  check_doc     = glade_xml_get_widget(xml, "load_check");
  fileentry_doc = glade_xml_get_widget(xml, "load_fileentry");
  entry_doc     = glade_xml_get_widget(xml, "load_entry");
  rubrica_preferences_configure_checked_entry(client, check_doc, 
					      fileentry_doc, 
					      entry_doc, 
					      RUBRICA_KEY_HAVE_DOC, 
					      RUBRICA_KEY_DOC);

  if (gconf_client_get_bool(client, RUBRICA_KEY_HAVE_DOC, NULL))
    str = g_strdup(gconf_client_get_string(client, RUBRICA_KEY_DOC, NULL));
  else
    str = g_strdup("");

  gtk_entry_set_text (GTK_ENTRY (entry_doc), str);
  g_free(str);

  check_folder     = glade_xml_get_widget(xml, "save_check");
  fileentry_folder = glade_xml_get_widget(xml, "save_fileentry");
  entry_folder     = glade_xml_get_widget(xml, "save_entry");
  rubrica_preferences_configure_checked_entry(client,
					      check_folder, 
					      fileentry_folder, 
					      entry_folder,  
					      RUBRICA_KEY_HAVE_FOLDER, 
					      RUBRICA_KEY_FOLDER);
  
  if (gconf_client_get_bool(client, RUBRICA_KEY_HAVE_FOLDER, NULL))
    str = g_strdup(gconf_client_get_string(client, RUBRICA_KEY_FOLDER, NULL));
  else
    str = g_strdup(""); 
  
  gtk_entry_set_text (GTK_ENTRY (entry_folder), str);
  g_free(str);

  g_object_set_data (G_OBJECT(dialog), RUBRICA_KEY_DOC, fileentry_doc);
  g_object_set_data (G_OBJECT(dialog), RUBRICA_KEY_FOLDER, fileentry_folder);

  /*    actions frame 
   */
  radio_inc    = glade_xml_get_widget(xml, "inc_radio");
  radio_bak    = glade_xml_get_widget(xml, "bak_radio");
  check_backup = glade_xml_get_widget(xml, "backup_check");
  box_backup   = glade_xml_get_widget(xml, "box_backup");  

  g_object_set_data(G_OBJECT(dialog), RUBRICA_KEY_BACKUP, box_backup);
  g_object_set_data(G_OBJECT(dialog), "inc_bak", radio_inc);
  g_object_set_data(G_OBJECT(dialog), "bak_bak", radio_bak);

  rubrica_preferences_configure_check_radio(client,
					    check_backup, radio_inc, 
					    radio_bak, 
					    RUBRICA_KEY_BACKUP, 
					    RUBRICA_KEY_BACKUP_TYPE_INC);

  check_compress = glade_xml_get_widget(xml, "compress_check");
  spin_rate      = glade_xml_get_widget(xml, "rate_spin");
  box_compress   = glade_xml_get_widget(xml, "box_compress");
  rubrica_preferences_configure_spin(client, check_compress, 
				     spin_rate, box_compress,
				     RUBRICA_KEY_COMPRESS, 
				     RUBRICA_KEY_RATE);
  g_object_set_data(G_OBJECT(dialog), RUBRICA_KEY_RATE, spin_rate);
  g_object_set_data(G_OBJECT(dialog), "box_compress", box_compress);   

  check_autosave = glade_xml_get_widget(xml, "autosave_check");
  spin_autosave  = glade_xml_get_widget(xml, "autosave_spin");  
  box_autosave   = glade_xml_get_widget(xml, "box_autosave");
  rubrica_preferences_configure_spin(client, check_autosave,
				     spin_autosave, box_autosave, 
				     RUBRICA_KEY_AUTOSAVE, 
				     RUBRICA_KEY_WAIT_TIME);
  gconf_client_notify_add(client, RUBRICA_KEY_WAIT_TIME, 
			  rubrica_preferences_notify_autosave_time_changes, 
			  spin_autosave, NULL, NULL);
  g_object_set_data(G_OBJECT(dialog), RUBRICA_KEY_WAIT_TIME, spin_autosave);
  g_object_set_data(G_OBJECT(dialog), "box_autosave", box_autosave); 



  /*    Search and Ordering page 
   */
  check_sort = glade_xml_get_widget(xml, "sort_check");
  rubrica_preferences_configure_check(client, check_sort, 
				      RUBRICA_KEY_SORT_DIRECTION); 

  check_case = glade_xml_get_widget(xml, "sensitive_check");
  rubrica_preferences_configure_check(client, check_case, 
				      RUBRICA_KEY_SEARCH_SENSITIVE); 

  check_string = glade_xml_get_widget(xml, "string_check");
  rubrica_preferences_configure_check(client, check_string, 
				      RUBRICA_KEY_SEARCH_COMPLETE);

  sort_menu = glade_xml_get_widget(xml, "sort_menu");
  rubrica_ui_option_menu_new(sort_menu, sort_option);
  rubrica_preferences_configure_option_menu(client, sort_menu, 
					    RUBRICA_KEY_SORT_TYPE);

  search_menu = glade_xml_get_widget(xml, "search_menu");
  rubrica_ui_option_menu_new(search_menu, search_on_option);
  rubrica_preferences_configure_option_menu(client, search_menu,
					    RUBRICA_KEY_SEARCH_TYPE);

  /*    Misc page 
   */
  width_spin = glade_xml_get_widget(xml, "orizontal_spin");
  rubrica_preferences_configure_size_spin(client, width_spin,
					  RUBRICA_KEY_WIDTH); 

  length_spin = glade_xml_get_widget(xml, "vertical_spin");
  rubrica_preferences_configure_size_spin(client, length_spin,
					  RUBRICA_KEY_LENGTH);  

  check_tooltips = glade_xml_get_widget(xml, "tooltips_check");
  rubrica_preferences_configure_check(client, check_tooltips, 
				      RUBRICA_KEY_TOOLTIPS);

  gconf_client_notify_add(client, RUBRICA_KEY_TOOLTIPS, 
			  rubrica_preferences_notify_tooltips_changes, 
			  check_tooltips, NULL, NULL);  

  check_exit = glade_xml_get_widget(xml, "exit_check");
  rubrica_preferences_configure_check(client, check_exit, RUBRICA_KEY_EXIT);

  toolbar_menu = glade_xml_get_widget(xml, "toolbar_menu");
  rubrica_ui_option_menu_new(toolbar_menu, toolbar_option);
  rubrica_preferences_configure_option_menu(client, toolbar_menu, 
					    RUBRICA_KEY_TOOLBAR_STYLE);
  gconf_client_notify_add(client, RUBRICA_KEY_TOOLBAR_STYLE, 
			  rubrica_preferences_notify_toolbar_style_changes,
			  toolbar_menu, NULL, NULL); 

  /* appearance page 
   */
  header_fp = glade_xml_get_widget(xml, "header_fontpicker");
  rubrica_preferences_configure_font_picker(client, header_fp, 
					    RUBRICA_KEY_FONT_HEADER);
  gconf_client_notify_add(client, RUBRICA_KEY_FONT_HEADER, 
			  rubrica_preferences_notify_font_change, 
			  RUBRICA_KEY_FONT_HEADER, NULL, NULL);

  data_fp = glade_xml_get_widget(xml, "data_fontpicker");
  rubrica_preferences_configure_font_picker(client, data_fp,
					    RUBRICA_KEY_FONT_DATA);
  gconf_client_notify_add(client, RUBRICA_KEY_FONT_DATA, 
			  rubrica_preferences_notify_font_change, 
			  RUBRICA_KEY_FONT_DATA, NULL, NULL);

  header_cp = glade_xml_get_widget(xml, "header_colorpicker");
  rubrica_preferences_configure_color_picker(client, header_cp,
					     RUBRICA_KEY_COLOUR_HEADER);
  gconf_client_notify_add(client, RUBRICA_KEY_COLOUR_HEADER, 
			  rubrica_preferences_notify_colour_change,
			  RUBRICA_KEY_COLOUR_HEADER, NULL, NULL);
  
  data_cp   = glade_xml_get_widget(xml, "data_colorpicker");
  rubrica_preferences_configure_color_picker(client, data_cp,
					     RUBRICA_KEY_COLOUR_DATA);
  gconf_client_notify_add(client, RUBRICA_KEY_COLOUR_DATA, 
			  rubrica_preferences_notify_colour_change,
			  RUBRICA_KEY_COLOUR_DATA, NULL, NULL);

  radio_r2l = glade_xml_get_widget(xml, "r2l_radio");
  radio_l2r = glade_xml_get_widget(xml, "l2r_radio");
  
  /* temp, to be removed */
  gravity_frame = glade_xml_get_widget(xml, "gravity_frame"); 
  gtk_widget_set_sensitive(GTK_WIDGET(gravity_frame), FALSE);
  
  /*    Columns view  page 
   */
  card_cols    = glade_xml_get_widget(xml, "column_card_list");
//  account_cols = glade_xml_get_widget(xml, "accout_column_list");
  
  model = rubrica_preferences_create_list_model(client, PREF_CARD_LIST);
  rubrica_preferences_configure_list(card_cols, model, PREF_CARD_LIST);
  g_object_unref(G_OBJECT(model));

/*  
  model = rubrica_preferences_create_list_model(client, PREF_ACCOUNT_LIST);
  rubrica_preferences_configure_list(account_cols, model, PREF_ACCOUNT_LIST);
  g_object_unref(G_OBJECT(model));
*/

  /*    Personal data view page 
   */
  personal_list = glade_xml_get_widget(xml, "data_list");
  model = rubrica_preferences_create_list_model(client, 
						PREF_PERSONAL_DATA_LIST);
  rubrica_preferences_configure_list(personal_list, model, 
				     PREF_PERSONAL_DATA_LIST);
  g_object_unref(G_OBJECT(model));
  
  /*    Company data view page
   */
  company_list = glade_xml_get_widget(xml, "company_list");
  model = rubrica_preferences_create_list_model(client, 
						PREF_COMPANY_DATA_LIST);
  rubrica_preferences_configure_list(company_list, model, 
				     PREF_COMPANY_DATA_LIST);
  g_object_unref(G_OBJECT(model));
  
  /*    Notes data view  page 
   */
  notes_list = glade_xml_get_widget(xml, "note_list");
  model = rubrica_preferences_create_list_model(client, PREF_NOTE_DATA_LIST);
  rubrica_preferences_configure_list(notes_list, model, PREF_NOTE_DATA_LIST);
  g_object_unref(G_OBJECT(model));  
  
  /*    Account view  page 

  account_list = glade_xml_get_widget(xml, "account_list");
  model = rubrica_preferences_create_list_model(client, PREF_ACCOUNT_LIST);
  rubrica_preferences_configure_list(account_list, model, PREF_ACCOUNT_LIST);
  g_object_unref(G_OBJECT(model));   
   */
  
  /*    Cyptography page  page 
  */
  
  /*    Dictionary page 

  box_dictionary = glade_xml_get_widget(xml, "box_dictionary");
  box_notebook   = glade_xml_get_widget(xml, "box_notebook");

  g_object_set_data(G_OBJECT(dialog), "box_dictionary", box_dictionary);
  g_object_set_data(G_OBJECT(dialog), "box_notebook", box_notebook);

  radio_external = glade_xml_get_widget(xml, "ext_radio");
  radio_internal = glade_xml_get_widget(xml, "def_radio");
  rubrica_preferences_configure_radio_dictionary(client, 
						 radio_external, 
						 radio_internal,  
						 RUBRICA_KEY_EXTERNAL,
						 RUBRICA_KEY_DEFAULT); 

  radio_italian   = glade_xml_get_widget(xml, "it_radio");
  radio_english   = glade_xml_get_widget(xml, "en_radio");
  rubrica_preferences_configure_radio(client, 2, 
				      radio_italian, RUBRICA_KEY_ITALIAN,  
				      radio_english, RUBRICA_KEY_ENGLISH); 

  entry_dict_name = glade_xml_get_widget(xml, "dictionary_entry");
  rubrica_preferences_configure_entry(client, entry_dict_name, 
				      RUBRICA_KEY_USER_DICTIONARY);  
  box_dictionary = glade_xml_get_widget(xml, "box_dictionary");
  box_notebook   = glade_xml_get_widget(xml, "box_notebook");
  load  = glade_xml_get_widget(xml, "open_button");
  save  = glade_xml_get_widget(xml, "save_button");
  rules = glade_xml_get_widget(xml, "rules_button");

  g_object_set_data (G_OBJECT(dialog), "box_dictionary", box_dictionary);
  g_object_set_data (G_OBJECT(dialog), "box_notebook", box_notebook);

  personal_view = glade_xml_get_widget(xml, "personal_view");
  address_view  = glade_xml_get_widget(xml, "address_view");
  communic_view = glade_xml_get_widget(xml, "communications_view");
  company_view  = glade_xml_get_widget(xml, "company_view");
  notes_view    = glade_xml_get_widget(xml, "notes_view");
  
  model = rubrica_preferences_create_dict_list(client, PERS_DICT);
  rubrica_preferences_configure_list(personal_view, model, PERS_DICT);

  model = rubrica_preferences_create_dict_list(client, ADD_DICT);
  rubrica_preferences_configure_list(address_view, model, ADD_DICT);

  model = rubrica_preferences_create_dict_list(client, COMM_DICT);
  rubrica_preferences_configure_list(communic_view, model, COMM_DICT);

  model = rubrica_preferences_create_dict_list(client, COMP_DICT);
  rubrica_preferences_configure_list(company_view, model, COMP_DICT);

  model = rubrica_preferences_create_dict_list(client, NOTES_DICT);
  rubrica_preferences_configure_list(notes_view, model, NOTES_DICT);

  g_object_unref(G_OBJECT(model)); 
  */

  /* TO BE REMOVED WHEN STABLE RELEASE IS READY
  */
  tmpbox = glade_xml_get_widget(xml, "vbox85");
  gtk_widget_set_sensitive(tmpbox, FALSE);

  tmpbox = glade_xml_get_widget(xml, "vbox92");
  gtk_widget_set_sensitive(tmpbox, FALSE);

  tmpbox = glade_xml_get_widget(xml, "vbox121");
  gtk_widget_set_sensitive(tmpbox, FALSE);

  tmpbox = glade_xml_get_widget(xml, "vbox122");
  gtk_widget_set_sensitive(tmpbox, FALSE);  

  /* ****************************** */

  
  /*      CALLBACKS
   */
  /*    SHOW PATH CALLBACK
   */
  g_signal_connect (G_OBJECT (check_path), "toggled",
		    G_CALLBACK (on_check_button_toggled), RUBRICA_KEY_SHOW);
  
  /*    LOAD DEFAULT DOCUMENT CALLBACKS
   */
  g_signal_connect (G_OBJECT (check_doc), "toggled",
		    G_CALLBACK(on_check_button_toggled),RUBRICA_KEY_HAVE_DOC);
  g_signal_connect (G_OBJECT (entry_doc), "changed",
		    G_CALLBACK (on_entry_changed), RUBRICA_KEY_DOC);
  
  /*    USE DEFAULT FOLDER CALLBACKS
   */
  g_signal_connect (G_OBJECT (check_folder), "toggled",
		    G_CALLBACK (on_check_button_toggled),
		    RUBRICA_KEY_HAVE_FOLDER);
  g_signal_connect (G_OBJECT (entry_folder), "changed",
		    G_CALLBACK (on_entry_changed), RUBRICA_KEY_FOLDER);

  /*    BACKUP CALLBACKS
   */
  g_signal_connect (G_OBJECT (check_backup), "toggled",
		    G_CALLBACK (on_check_button_toggled), RUBRICA_KEY_BACKUP);
  
  g_signal_connect (G_OBJECT (radio_inc), "toggled",
		    G_CALLBACK (on_radio_button_toggled), 
		    RUBRICA_KEY_BACKUP_TYPE_INC);
  g_signal_connect (G_OBJECT (radio_bak), "toggled",
		    G_CALLBACK (on_radio_button_toggled), 
		    RUBRICA_KEY_BACKUP_TYPE_BAK);
  
  /*    COMPRESS CALLBACKS
   */
  g_signal_connect (G_OBJECT (check_compress), "toggled",
		    G_CALLBACK (on_check_button_toggled), 
		    RUBRICA_KEY_COMPRESS);
  g_signal_connect (G_OBJECT (spin_rate), "activate",
		    G_CALLBACK (on_spin_activate), RUBRICA_KEY_RATE);
  g_signal_connect (G_OBJECT (spin_rate), "changed",
		    G_CALLBACK (on_spin_activate), RUBRICA_KEY_RATE);
  
  /*    AUTOSAVE CALLBACKS
   */
  g_signal_connect (G_OBJECT (check_autosave), "toggled",
		    G_CALLBACK (on_check_button_toggled),
		    RUBRICA_KEY_AUTOSAVE);
  g_signal_connect (G_OBJECT (spin_autosave), "activate",
		    G_CALLBACK (on_spin_activate), RUBRICA_KEY_WAIT_TIME);
  g_signal_connect (G_OBJECT (spin_autosave), "changed",
		    G_CALLBACK (on_spin_activate), RUBRICA_KEY_WAIT_TIME);
  
  /*    SORT CALLBACKS
   */
  g_signal_connect (G_OBJECT (check_sort), "toggled", 
		    G_CALLBACK (on_check_button_toggled), 
		    RUBRICA_KEY_SORT_DIRECTION); 
  
  g_signal_connect (G_OBJECT (sort_menu), "changed", 
		    G_CALLBACK (on_option_menu_changed), 
		    RUBRICA_KEY_SORT_TYPE); 
  
  /*     SEARCH CALLBACKS
   */
  g_signal_connect (G_OBJECT (check_case), "toggled", 
		    G_CALLBACK (on_check_button_toggled), 
		    RUBRICA_KEY_SEARCH_SENSITIVE); 
  
  g_signal_connect (G_OBJECT (check_string), "toggled", 
		    G_CALLBACK (on_check_button_toggled), 
		    RUBRICA_KEY_SEARCH_COMPLETE); 
  
  g_signal_connect (G_OBJECT (search_menu), "changed", 
		    G_CALLBACK (on_option_menu_changed), 
		    RUBRICA_KEY_SEARCH_TYPE); 
  
  /*    SPIN BUTTONS CALLBACKS
   */
  g_signal_connect (G_OBJECT (width_spin), "activate",
		    G_CALLBACK (on_spin_activate), RUBRICA_KEY_WIDTH);
  g_signal_connect (G_OBJECT (width_spin), "changed",
		    G_CALLBACK (on_spin_activate), RUBRICA_KEY_WIDTH);
  
  g_signal_connect (G_OBJECT (length_spin), "activate",
		    G_CALLBACK (on_spin_activate), RUBRICA_KEY_LENGTH);  
  g_signal_connect (G_OBJECT (length_spin), "changed",
		    G_CALLBACK (on_spin_activate), RUBRICA_KEY_LENGTH);  
  
  /*    CHECK BUTTONS CALLBACKS
   */
  g_signal_connect (G_OBJECT (check_exit), "toggled",
		    G_CALLBACK (on_check_button_toggled), RUBRICA_KEY_EXIT);
  
  g_signal_connect (G_OBJECT (check_tooltips), "toggled", 
		    G_CALLBACK (on_check_button_toggled), 
		    RUBRICA_KEY_TOOLTIPS);
  
  /*    TOOLBAR STYLE CALLBACK
   */
  g_signal_connect (G_OBJECT (toolbar_menu), "changed", 
		    G_CALLBACK (on_option_menu_changed), 
		    RUBRICA_KEY_TOOLBAR_STYLE);  
  
  /* FONT PICKER CALLBACK 
   */
  g_signal_connect(G_OBJECT (header_fp), "font-set", 
		   G_CALLBACK(on_font_set_cb), RUBRICA_KEY_FONT_HEADER);
  g_signal_connect(G_OBJECT (data_fp), "font-set", 
		   G_CALLBACK(on_font_set_cb), RUBRICA_KEY_FONT_DATA);

  /* COLOR PICKER CALLBACK 
   */
  g_signal_connect(G_OBJECT (header_cp), "color-set", 
		   G_CALLBACK(on_colour_set_cb), RUBRICA_KEY_COLOUR_HEADER); 
  g_signal_connect(G_OBJECT (data_cp), "color-set", 
		   G_CALLBACK(on_colour_set_cb), RUBRICA_KEY_COLOUR_DATA);

/*  
  g_signal_connect(G_OBJECT (radio_external), "toggled",
		   G_CALLBACK(on_radio_button_toggled), RUBRICA_KEY_EXTERNAL);
  g_signal_connect(G_OBJECT(radio_internal), "toggled",
		   G_CALLBACK(on_radio_button_toggled), RUBRICA_KEY_DEFAULT);

  g_signal_connect(G_OBJECT(radio_italian), "toggled",
		   G_CALLBACK(on_radio_button_toggled), RUBRICA_KEY_ITALIAN);
  g_signal_connect(G_OBJECT(radio_english), "toggled",
		   G_CALLBACK(on_radio_button_toggled), RUBRICA_KEY_ENGLISH);
*/

  /*      notebook dictionary buttons callbacks 

  g_signal_connect(G_OBJECT(load), "clicked",
		   G_CALLBACK(on_load_dictionary_clicked), entry_dict_name);
  g_signal_connect(G_OBJECT(save), "clicked",
		   G_CALLBACK(on_save_dictionary_clicked), entry_dict_name);
  g_signal_connect(G_OBJECT(rules), "clicked",
		   G_CALLBACK(on_show_rules_clicked), NULL);
  */
}



/*    preferences interface creation
*/
GtkWidget*
create_preferences_dialog(GtkWidget *app, GConfClient *client)
{
  GladeXML* xml;
  GtkWidget *tree;
  GtkTreeModel *model;

  xml = glade_xml_new (PACKAGE_DATA_DIR"/rubrica/ui/preferences.glade", 
		       NULL, NULL);

  if (!xml)
    {
      g_print("\nimpossibile leggere l'interfaccia grafica %s%s",
	      PACKAGE_DATA_DIR, "/rubrica/ui/preferences.glade");

      return NULL;
    }

  dialog = glade_xml_get_widget(xml, "preferences");
  gtk_widget_hide(dialog);
  notebook = glade_xml_get_widget(xml, "notebook");
//  tooltips = glade_xml_get_widget(xml, "tooltips");

  model    = rubrica_preferences_create_tree_model(client);
  tree     = glade_xml_get_widget(xml, "tree");
  gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(model));
  rubrica_preferences_configure_tree_view(GTK_TREE_VIEW(tree));
  g_object_unref(G_OBJECT(model));

  rubrica_preferences_configure(app, client, xml);

//  rubrica_preferences_create_new_tree_view(model, notebook);   
//  rubrica_utils_add_tooltip(tooltips);

  g_object_set_data (G_OBJECT (app),    "dialog",   dialog);
  g_object_set_data (G_OBJECT (dialog), "treeview", tree);
  g_object_set_data (G_OBJECT (dialog), "notebook", notebook);
  g_object_set_data (G_OBJECT (dialog), "tooltips", tooltips);

  gtk_widget_show(dialog);
  return dialog;
}


