/*
*  Rubrica
*  file: rubrica.c
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>
#include <libxml/tree.h>
#include <libxml/parser.h>

#include "rubrica.h"
#include "item.h"
#include "doctab.h"
#include "rubrica_marshal.h"
#include "data_view.h"
#include "main.h"
#include "dialog.h"
#include "card.h"
#include "ui.h"
#include "callbacks.h"
#include "form_card.h"
#include "utils.h"


GList*      books;                /* addressbooks's list */
RubricaDoc* current;              /* active addressbook  */

GtkWidget*  notebook;             /* addressbooks container */


gboolean have_buffer = FALSE;
gpointer buffer = NULL;
gpointer rtemp  = NULL;

gint position = -1;


/*    signals enumeration 
 */
enum {
  ITEM_ADD,                /* signal: item_added        */
  ITEM_MOD,                /* signal: item_deleted      */
  ITEM_DEL,                /* signal: item_deleted      */
  DOC_MODIFIED,            /* signal: document_modified */
  LAST_SIGNAL
};


static guint rubrica_doc_signals[LAST_SIGNAL] = {0};

static void rubrica_doc_class_init     (RubricaDocClass* klass);
static void rubrica_doc_instance_init  (RubricaDoc* doc);

static void rubrica_doc_dispose        (RubricaDoc* doc);
static void rubrica_doc_finalize       (RubricaDoc* doc);


// static void rubrica_doc_class_finalize (RubricaDocClass* klass);
// static void rubrica_doc_instance_finalize (RubricaDoc* doc);

/*   implementare i metodi della classe
*/





  

static void
rubrica_doc_class_init(RubricaDocClass* klass)
{
  GObjectClass* objclass;

  objclass = G_OBJECT_CLASS(klass);

  objclass->dispose  = rubrica_doc_dispose;
  objclass->finalize = rubrica_doc_finalize;

  rubrica_doc_signals[ITEM_ADD] = 
    g_signal_new("item_added", 
		 RUBRICA_DOC_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaDocClass, item_add),
		 NULL, 
		 NULL,
		 rubrica_doc_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);	

  rubrica_doc_signals[ITEM_MOD] = 
    g_signal_new("item_modified", 
		 RUBRICA_DOC_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaDocClass, item_mod),
		 NULL, 
		 NULL,
		 rubrica_doc_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);	

  rubrica_doc_signals[ITEM_DEL] = 
    g_signal_new("item_deleted", 
		 RUBRICA_DOC_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaDocClass, item_del),
		 NULL, 
		 NULL,
		 rubrica_doc_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);
	
  rubrica_doc_signals[DOC_MODIFIED] = 
    g_signal_new("document_modified", 
		 RUBRICA_DOC_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaDocClass, doc_mod),
		 NULL, 
		 NULL,
		 rubrica_doc_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);	
}

static void
rubrica_doc_instance_init(RubricaDoc* doc)
{
  doc->id        = 1L;

  doc->docs      = NULL;      /* items list                         */
  doc->xmldoc    = NULL;      /* ptr to xml doc                     */

  doc->new       = TRUE;      /* true if this is a new addressbook  */
  doc->modified  = FALSE;     /* true if rubrica is modified */
  doc->empty     = TRUE;      /* true if rub list is empty   */

  doc->name      = NULL;      /* addressbook's name */ 
  doc->path      = NULL;      /* addressbook's path */
  doc->tmpfile   = NULL;      /* temp file's name, used in autosave */

  doc->items     = 0;         /* number of item in current document */
  doc->index     = -1;        /* number of the item currently show  */
  doc->list_path = NULL;      /* currently shown item's path (GtkTreePath) */

  doc->view      = NULL;      /* the tree_view */
  doc->child     = NULL;      /* notebook page child */
  doc->tab       = NULL;      /* notebook tab */ 

  doc->dispose_has_run = FALSE;
}


static void
rubrica_doc_dispose(RubricaDoc* doc)
{
  if (doc->dispose_has_run)
    return;

  doc->dispose_has_run = TRUE;
}

static void
rubrica_doc_finalize(RubricaDoc *doc)
{
  GList* lst;

  for (lst = doc->docs; lst; lst = lst->next)
    {
      if (lst && lst->data)
	g_object_unref(lst->data);
    }  

  if (doc->xmldoc)
    xmlFree(doc->xmldoc);

  if (doc->name)
    g_free(doc->name);

  if (doc->path)
    g_free(doc->path);
  
  if (doc->tmpfile)
    g_free(doc->tmpfile);

  if (doc->list_path)
    g_free(doc->list_path);
}


GType
rubrica_doc_get_instance_type(void)
{
  static GType rubrica_doc_type = 0;

  if (!rubrica_doc_type)
    {
      static const GTypeInfo rubrica_doc_info = 
	{
	  sizeof(RubricaDocClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_doc_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaDoc),
	  0,
	  (GInstanceInitFunc) rubrica_doc_instance_init
	};

      rubrica_doc_type = g_type_register_static(G_TYPE_OBJECT, 
						"RubricaDoc", 
						&rubrica_doc_info, 0);
    }
  
  return rubrica_doc_type;
}



RubricaDoc* 
rubrica_doc_new(void)
{
  RubricaDoc* obj;

  obj = g_object_new(rubrica_doc_get_instance_type(), NULL);

  return obj;
}

void 
rubrica_doc_store_notebook(GtkWidget* nb)
{
  notebook = nb;
}



/* *****************************************  Get
*/


/**
 * rubrica_doc_get_id
 * @doc: a RubricaDoc
 *
 * return the first available id (will be set as item's id).
 */
gulong
rubrica_doc_get_id(RubricaDoc *doc)
{
  gint id;

  g_return_val_if_fail(IS_RUBRICA_DOC(doc), 0L);

  id = doc->id;
  if (id <= doc->items)
    id = doc->id = doc->items+1;
  
  doc->id++;

  return id;
}


/**
 * rubrica_doc_is_new
 * @doc: a RubricaDoc 
 *
 * return TRUE if the document is new, FALSE otherwise
 */
gboolean 
rubrica_doc_is_new(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), FALSE);
  
  return (gboolean) doc->new;
}


/**
 * rubrica_doc_is_empty
 * @doc: a RubricaDoc
 *
 * return TRUE if document is empty, FALSE otherwise. 
 */         
gboolean 
rubrica_doc_is_empty(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), TRUE);
  
  return (gboolean) doc->empty;
}


/**
 * rubrica_doc_is_modified
 * @doc: a RubricaDoc
 *
 * return TRUE if document is modified, FALSE otherwise. 
 */
gboolean 
rubrica_doc_is_modified(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), FALSE);
  
  return (gboolean) doc->modified;
}


/**
 * rubrica_doc_get_path
 * @doc: a RubricaDoc
 *
 * return the document's path (the file path), 
 * or NULL if doc is not a RubricaDoc. 
 */
gchar*
rubrica_doc_get_path(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  
  return (gchar *) doc->path;
}


/**
 * rubrica_doc_get_name
 * @doc: a RubricaDoc
 *
 * return the document's file name, or NULL 
 * if doc is not a RubricaDoc.
 */
gchar*
rubrica_doc_get_name(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  
  return (gchar *) doc->name;
}


/**
 * rubrica_doc_get_tmpfile
 * @doc: a RubricaDoc
 *
 * return the temporary file name used in autosave, or NULL.
 */
gchar*
rubrica_doc_get_tmpfile(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  
  return (gchar *) doc->tmpfile;  
}


/**
 * rubrica_doc_get_items
 * @doc: a RubricaDoc
 *
 * return the items in doc, or 0 if doc is not a RubricaDoc. 
 */
gint 
rubrica_doc_get_items(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), 0);
  
  return (gint) doc->items;
}


/**
 * rubrica_doc_get_index
 * @doc: a RubricaDoc
 * 
 * retrun the index of currently displayed item, 
 * or 0 if doc is not a RubricaDoc
 */
gint 
rubrica_doc_get_index(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), 0);
  
  return (gint) doc->index;
}


/**
 * rubrica_doc_get_list_path
 * @doc: a RubricaDoc
 *
 * return the document list's path (GtkTreeView path), 
 * or NULL if doc is not a RubricaDoc. 
 */
gchar*
rubrica_doc_get_list_path(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
 
  return (gchar*) doc->list_path;
}


/**
 * rubrica_doc_get_view
 * @doc: a RubricaDoc
 *
 * return document's tree view or NULL 
 * if doc is not a RubricaDoc. 
 */  
GtkWidget*
rubrica_doc_get_view(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);

  return (GtkWidget *) doc->view; 
}


/**
 * rubrica_doc_get_doc_list
 * @doc: a RubricaDoc
 *
 * return the item's list contained in doc
 * or NULL (the list is empty)
 */
GList* 
rubrica_doc_get_doc_list(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  
  return doc->docs;
}


/**
 * rubrica_doc_get_xml_tree
 * @doc: a RubricaDoc
 *
 * return the xml tree of the document
 * or NULL if doc isn't a RubricaDoc
 */
xmlDocPtr      
rubrica_doc_get_xml_tree (RubricaDoc* doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  
  if (!doc->xmldoc)
    rubrica_doc_build_xmltree(doc);
    
  return doc->xmldoc;
}

/* *****************************************  Set
*/

/**
 * rubrica_doc_set_new
 * @doc: a RubricaDoc 
 * @state: gboolean
 *
 * set the state of document
 */
inline void 
rubrica_doc_set_new(RubricaDoc *doc, gboolean state)
{
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  doc->new = state;
}


/**
 * rubrica_doc_set_empty
 * @doc: a RubricaDoc 
 * @state: a gboolean (TRUE) if doc is empty, FALSE otherwise
 */
inline void 
rubrica_doc_set_empty(RubricaDoc *doc, gboolean state)
{
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  doc->empty = state;
}


/**
 * rubrica_doc_set_modified
 * @doc: a RubricaDoc 
 * @modify: a gboolean (TRUE) if doc is modified, FALSE otherwise
 */
inline void 
rubrica_doc_set_modified(RubricaDoc *doc, gboolean modify)
{
  GtkWidget *app;
  gchar *title, *name;
  
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  app  = rubrica_get_app();
  name = rubrica_doc_get_name(doc);

  if (modify) 
    {
      title = g_strdup_printf(_("Rubrica -- %s changed --"), name);
      gtk_window_set_title(GTK_WINDOW(app), title);
      rubrica_doc_enable_red(doc);
      g_free(title);
    }
  else      
    {
      gtk_window_set_title(GTK_WINDOW(app), "Rubrica");
      rubrica_doc_enable_green(doc);
    }
  
  doc->modified = modify;
}


/**
 * rubrica_doc_set_path
 * @doc:
 * @str: 
 *
 * set the document's path to str
 */
void 
rubrica_doc_set_path(RubricaDoc *doc, gchar *str)
{
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  if (doc->path)
    g_free(doc->path);

  if (str)
    doc->path = g_strdup(str);
  else
    doc->path = NULL;   
}


/**
 * rubrica_doc_set_name
 * @doc:
 * @str:
 *
 * set documet's name to str
 */
void 
rubrica_doc_set_name(RubricaDoc *doc, gchar *str)
{
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  if (doc->name)
    g_free(doc->name);

  if (str)
    doc->name = g_strdup(str);
  else
    doc->name = NULL;

  rubrica_doc_set_tab_label(doc, str);
}


/**
 * rubrica_doc_set_tmpfile
 * @doc:
 * @str:
 *
 * set the document's temporary filename to str
 */
void 
rubrica_doc_set_tmpfile(RubricaDoc *doc, gchar *str)
{
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  if (doc->tmpfile)
    g_free(doc->tmpfile);

  doc->tmpfile = str ? g_strdup(str) : NULL;
}


/**
 * rubrica_doc_set_items
 * @doc:
 * @n: gint
 *
 * set the document's item number to n
 */
void 
rubrica_doc_set_items(RubricaDoc *doc, gint n)
{
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  doc->items = n;
}

/**
 * rubrica_doc_inc_items
 * @doc:
 *
 * increase the document's item number
 */
void 
rubrica_doc_inc_items(RubricaDoc *doc)
{
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  doc->items++;
}

/**
 * rubrica_doc_dec_items
 * @doc:
 *
 * decrease the document's item number
 */
void 
rubrica_doc_dec_items(RubricaDoc *doc)
{
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  doc->items--;
}


/**
 * rubrica_doc_set_index
 * @doc:
 * @index: gint 
 *
 * store the index of the item currently showed
 * salva l'indice dell'elemento mostrato attualmente
 */
void 
rubrica_doc_set_index(RubricaDoc *doc, gint index)
{
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  doc->index = index;
}


/**
 * rubrica_doc_set_list_path
 * @doc:
 * @path: gchar 
 *
 * store the path (GtkTreePath) of the item currently showed
 * salva il path (GtkTreePath) dell'elemento mostrato attualmente
 */
void  
rubrica_doc_set_list_path(RubricaDoc *doc, gchar *path)
{
  g_return_if_fail(doc != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  if (doc->list_path != NULL)
    g_free(doc->list_path);
  
  if (path)
    doc->list_path = g_strdup(path);
  else
    doc->list_path = NULL;
}


void 
rubrica_doc_set_view(RubricaDoc *doc, GtkTreeView* treeview)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));
  g_return_if_fail(GTK_IS_TREE_VIEW(treeview));

  doc->view = treeview;
}

void 
rubrica_doc_set_child(RubricaDoc *doc, GtkWidget* child)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));
  g_return_if_fail(GTK_IS_WIDGET(child));

  doc->child = child;
}


void 
rubrica_doc_set_doc_tab(RubricaDoc *doc, RubricaDocTab* tab)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));
  g_return_if_fail(tab != NULL);

  doc->tab = tab;

  g_signal_connect(G_OBJECT(doc->tab->close), "clicked",
		   G_CALLBACK(on_close_activate), doc);

}


GList*      
rubrica_doc_get_all_books (void)
{
  return books;
}

RubricaDoc* 
rubrica_doc_get_document  (GList* book_node)
{
  g_return_val_if_fail(book_node != NULL, NULL);

  if (IS_RUBRICA_DOC(book_node->data))
    return (RubricaDoc*) book_node->data;
  /* else */
  return NULL;
}


RubricaDoc*
rubrica_doc_get_current_document (void)
{
  return (RubricaDoc *) current;
}

RubricaDoc* 
rubrica_doc_get_nth_document(gint n)
{
  RubricaDoc* doc;

  if (n > g_list_length(books))
    return NULL;
      
  doc = g_list_nth_data(books, n);
  if (IS_RUBRICA_DOC(doc))
    return doc;

  /* else */
  return NULL; 
}

void 
rubrica_doc_set_doc_as_current(RubricaDoc* doc)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  current = doc;
}


void 
rubrica_doc_append_doc(RubricaDoc* doc,
		       GtkWidget *child, 
		       GtkWidget *box)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));
  g_return_if_fail(child != NULL);
  g_return_if_fail(box != NULL);
  
  books = g_list_append(books, doc);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child, box);
  gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), -1);
}


void 
rubrica_doc_remove_doc(RubricaDoc* doc) 
{
  gint index;

  g_return_if_fail(IS_RUBRICA_DOC(doc));

  index = g_list_index(books, doc);
  books = g_list_remove(books, doc);
  g_object_unref(doc);
  
  gtk_notebook_remove_page(GTK_NOTEBOOK(notebook), index);
}




/* ******************************************  API
 */

/* reject id
 */
void           
rubrica_doc_reject_id (RubricaDoc *doc)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  doc->id--;
}


/*   get the item with id "id"
*/
gpointer
rubrica_doc_get_item_by_id(RubricaDoc *doc, gulong id)
{
  GList* alias;
  RubricaItem* item;

  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  
  for (alias = doc->docs; alias; alias = g_list_next(alias))
    {
      item = (RubricaItem*) alias->data;
      
      if(item && (item->id == id))
	{
	  return item;
	}    
    }
  
  return NULL;  
}

/*   get the nth item from doc
*/
gpointer 
rubrica_doc_get_nth_item(RubricaDoc *doc, gulong n)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  g_return_val_if_fail(n >= 0, NULL);
  g_return_val_if_fail(n < doc->items, NULL);

  return g_list_nth_data(doc->docs, n);
}

/*   get last item from doc
*/
gpointer
rubrica_doc_get_last_item(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  
  return g_list_nth_data(doc->docs, g_list_length(doc->docs));
}



/*   retrieve the new item from form
*/
gpointer 
rubrica_doc_retrieve_item_form(RubricaDoc *doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);

  return rubrica_item_retrieve_item(doc);
}

gpointer
rubrica_doc_get_selected_item (RubricaDoc* doc)
{
  gulong id;

  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);

  id = rubrica_card_get_selected_item_id(doc);

  if (-1 == id)
    return NULL;
  
  return rubrica_doc_get_item_by_id(doc, id);
}

gpointer
rubrica_doc_modify_item(RubricaDoc* doc, gpointer item)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  g_return_val_if_fail(item != NULL, NULL);

  return rubrica_item_modify_item(doc, item);
}

void
rubrica_doc_cut_item(RubricaDoc *doc, gpointer item)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));
  g_return_if_fail(item != NULL);
  
  rubrica_item_cut_item(doc);

  rubrica_doc_dec_items(doc);
  rubrica_doc_set_new(doc, FALSE);
  rubrica_doc_set_modified(doc, TRUE);
  rubrica_doc_set_empty(doc, FALSE);
  
  rubrica_data_view_set_items(doc->items);    
  rubrica_item_set_deleted(RUBRICA_ITEM(item), TRUE);
}


gpointer
rubrica_doc_copy_item(RubricaDoc *doc, gpointer item)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  g_return_val_if_fail(item != NULL, NULL);
  
  return rubrica_item_copy_item(item);
}

void
rubrica_doc_paste_item(RubricaDoc *doc, gpointer obj)
{
/*
  RubricaItem *item;

  g_return_if_fail(IS_RUBRICA_DOC(doc));
  g_return_if_fail(item != NULL);

  item = RUBRICA_ITEM(obj);

  
  

  gint index;

  g_return_if_fail(doc != NULL);

  if (!buffer)
    {
      rubrica_dialog_message(_("I can't paste anything if you "
			       "haven't previously cut or copied"), 
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);

      return;
    }
  
  rubrica_item_set_father(buffer, doc);
  index = rubrica_doc_get_index(doc);
  if (index  == -1)
    rubrica_card_append_item(doc, buffer);
  else
    rubrica_card_insert_item(doc, buffer, index);

  buffer = NULL;
*/
}



void 
rubrica_doc_append_item(RubricaDoc *doc, gpointer item)
{
  g_return_if_fail(item != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  doc->docs = g_list_append(doc->docs, item);

  rubrica_doc_set_new(doc, FALSE);
  rubrica_doc_set_empty(doc, FALSE);
  rubrica_doc_set_modified(doc, TRUE);

  rubrica_doc_inc_items(doc);
// TODO: collegare la vista al segnale document_modified
  rubrica_data_view_set_items(doc->items);   

  g_signal_emit_by_name(doc, "item_added", item, G_TYPE_NONE); 
  g_signal_emit_by_name(doc, "document_modified", NULL, G_TYPE_NONE); 
}


void 
rubrica_doc_replace_item (RubricaDoc *doc, gpointer old, gpointer new)
{
  GList* node;

  g_return_if_fail(doc != NULL);
  g_return_if_fail(old != NULL);
  g_return_if_fail(new != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  node = g_list_find(doc->docs, old);
  node->data = new;  

  rubrica_doc_set_new(doc, FALSE);
  rubrica_doc_set_modified(doc, TRUE);
  rubrica_doc_set_empty(doc, FALSE);  

  g_signal_emit_by_name(doc, "item_modified", new, G_TYPE_NONE);
  g_signal_emit_by_name(doc, "document_modified", NULL, G_TYPE_NONE); 
}



void
rubrica_doc_delete_item (RubricaDoc *doc, gpointer item)
{
  g_return_if_fail(item != NULL);
  g_return_if_fail(IS_RUBRICA_DOC(doc));

  rubrica_doc_dec_items(doc); 
// TODO: collegare la vista al segnale document_modified
  rubrica_data_view_set_items(doc->items); 

  if (rubrica_doc_get_items(doc) == 0)
    rubrica_doc_set_empty(doc, TRUE);
  else
    rubrica_doc_set_empty(doc, FALSE);

  rubrica_doc_set_new(doc, FALSE);
  rubrica_doc_set_modified(doc, TRUE);

  rubrica_item_set_deleted(RUBRICA_ITEM(item), TRUE);

  g_signal_emit_by_name(doc, "item_deleted", NULL, G_TYPE_NONE);
  g_signal_emit_by_name(doc, "document_modified", NULL, G_TYPE_NONE); 
}


void
rubrica_doc_delete_last_item(RubricaDoc *doc) 
{ 
  gpointer item;

  g_return_if_fail(IS_RUBRICA_DOC(doc)); 

  item = rubrica_doc_get_last_item(doc);
  rubrica_doc_delete_item(doc, item);
} 


void
rubrica_doc_delete_nth_item(RubricaDoc *doc, gint n) 
{
  gpointer item;

  g_return_if_fail(IS_RUBRICA_DOC(doc)); 

  item = rubrica_doc_get_nth_item(doc, n);
  rubrica_doc_delete_item(doc, item);
}


void
rubrica_doc_build_xmltree(RubricaDoc* doc)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));
    
  rubrica_item_build_xmltree(doc);
}


void 
rubrica_doc_free_item(RubricaDoc* doc, gpointer item)
{
   g_return_if_fail(IS_RUBRICA_DOC(doc));
   g_return_if_fail(item != NULL);

//   rubrica_item_free_item(doc, item);
}


/* *******************************************  DocTab
 */

GtkWidget*     
rubrica_doc_get_tab_box (RubricaDoc* doc)
{
  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);

  return doc->tab->box;
}


void 
rubrica_doc_set_tab_label (RubricaDoc* doc, gchar* str)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));
  
  rubrica_doc_tab_set_label(doc->tab, str);
}


void           
rubrica_doc_enable_green (RubricaDoc* doc)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));
  
  rubrica_doc_tab_enable_green(doc->tab);
}

void           
rubrica_doc_enable_red (RubricaDoc* doc)
{
  g_return_if_fail(IS_RUBRICA_DOC(doc));
  
  rubrica_doc_tab_enable_red(doc->tab);
}



/* ********************************************* Other
*/



void 
rubrica_doc_free_memory(RubricaDoc *doc)
{


}


void 
rubrica_free_string(gchar *str)
{
  if (str) g_free(str);
}


/*  utility functions
*/
gboolean rubrica_doc_others_docs_are_modified(void)
{  
  RubricaDoc *doc;
  gint cards, n;
  gboolean bool = FALSE;

  cards = g_list_length(books);
  
  for (n = 0; n < cards; n++)
    {
      doc = rubrica_doc_get_nth_document(n);
      
      bool |=  rubrica_doc_is_modified(doc);   
    }

  return bool;  
}


void 
rubrica_doc_close(RubricaDoc *doc)
{    
  GtkWidget *dialog;
  gint choose;
  gchar *mes;

  g_return_if_fail(doc != NULL);

  if (rubrica_doc_is_modified(doc))
    {
      mes = g_strdup_printf(_("Warning. %s\n"
			      "Changes not saved will be lost.\n\n"
			      "Do you want continue?"), 
			    _("Addressbook has changed."));
      
      dialog = rubrica_dialog_query(mes, GTK_BUTTONS_YES_NO);
      gtk_widget_show(dialog);
      g_free(mes);

      choose = gtk_dialog_run(GTK_DIALOG(dialog));
      
      switch(choose)
	{
	case GTK_RESPONSE_YES:
	  gtk_widget_destroy(dialog);
	  /* delete addressbook, go down an delete */
	  break;
	  
	case GTK_RESPONSE_NO:
	default:
	  gtk_widget_destroy(dialog);
	  return;
	  break;
	}  
    }
  
  /*      clean view window 
   */
  rubrica_data_view_clean_view();  
  rubrica_doc_remove_doc(doc);
  
  if (g_list_length(books) == 0)
    rubrica_card_add_page_view(NULL);  
}


