/*
*  Rubrica
*  file: rubrica.h
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_H
#define _RUBRICA_H

#include <gnome.h>
#include <libxml/tree.h>

#include "rubrica_type.h"
#include "doctab.h"


/*  fields attribute: r ==> read  (available get function) 
                      w ==> write (available set function)
*/  


#define RUBRICA_DOC_TYPE            (rubrica_doc_get_instance_type())
#define RUBRICA_DOC(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), RUBRICA_DOC_TYPE, RubricaDoc))
#define RUBRICA_DOC_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), RUBRICA_DOC_TYPE, RubricaDocClass))
#define IS_RUBRICA_DOC(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), RUBRICA_DOC_TYPE))
#define IS_RUBRICA_DOC_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), RUBRICA_DOC_TYPE))
#define RUBRICA_DOC_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), RUBRICA_DOC_TYPE, RubricaDocClass))

typedef struct _RubricaDoc         RubricaDoc;
typedef struct _RubricaDocClass    RubricaDocClass;
typedef struct _RubricaDocPrivate  RubricaDocPrivate;


struct _RubricaDoc
{
  GObject parent;

  gulong id;                /* r first available id */

  GList* docs;              /* r/w list of cards || accounts || ... */
  xmlDocPtr xmldoc;         /* r/w xml doc */
  
  gboolean new;             /* r/w true if this is a new addressbook  */
  gboolean modified;        /* r/w true if rubrica is modified */
  gboolean empty;           /* r/w true if rub list is empty   */

  gchar *name;              /* r/w addressbook's name */
  gchar *path;              /* r/w addressbook's path */
  gchar *tmpfile;           /* r/w temp file's name, used in autosave */

  gint   items;             /* r/w number of item in current document */
  gint   index;             /* r/w number of the item currently show  */
  gchar* list_path;         /* r/w currently shown item's path (GtkTreePath) */

  GtkTreeView*   view;      /* r    the tree_view      */
  GtkWidget*     child;     /* r    notebook page      */
  RubricaDocTab* tab;       /* r/w  notebook tab       */

  /* private */
  gboolean dispose_has_run;
};


struct _RubricaDocClass
{
  GObjectClass parent;

  /* metodi */
  
  /* signals */
  void (*item_add) (RubricaDoc*);
  void (*item_mod) (RubricaDoc*);
  void (*item_del) (void);
  void (*doc_mod)  (RubricaDoc*);
};


GType          rubrica_doc_get_instance_type();

RubricaDoc*    rubrica_doc_new            (void);
void           rubrica_doc_store_notebook (GtkWidget* notebook);

/*    Get
 */
gulong         rubrica_doc_get_id         (RubricaDoc* doc);

gboolean       rubrica_doc_is_new         (RubricaDoc* doc);
gboolean       rubrica_doc_is_empty       (RubricaDoc* doc);
gboolean       rubrica_doc_is_modified    (RubricaDoc* doc);

gchar*         rubrica_doc_get_path       (RubricaDoc* doc);
gchar*         rubrica_doc_get_name       (RubricaDoc* doc);
gchar*         rubrica_doc_get_tmpfile    (RubricaDoc* doc);

gint           rubrica_doc_get_items      (RubricaDoc* doc);
gint           rubrica_doc_get_index      (RubricaDoc* doc);
gchar*         rubrica_doc_get_list_path  (RubricaDoc* doc);

GtkWidget*     rubrica_doc_get_view       (RubricaDoc* doc);
GList*         rubrica_doc_get_doc_list   (RubricaDoc* doc);
xmlDocPtr      rubrica_doc_get_xml_tree   (RubricaDoc* doc);


/*    Set
*/
void           rubrica_doc_set_new        (RubricaDoc* doc, gboolean state);
void           rubrica_doc_set_empty      (RubricaDoc* doc, gboolean state);
void           rubrica_doc_set_modified   (RubricaDoc* doc, gboolean state);

void           rubrica_doc_set_path       (RubricaDoc* doc, gchar *str);
void           rubrica_doc_set_name       (RubricaDoc* doc, gchar *str);
void           rubrica_doc_set_tmpfile    (RubricaDoc* doc, gchar *str);

void           rubrica_doc_set_items      (RubricaDoc* doc, gint n);
void           rubrica_doc_inc_items      (RubricaDoc* doc);
void           rubrica_doc_dec_items      (RubricaDoc* doc);

void           rubrica_doc_set_index      (RubricaDoc* doc, gint index);
void           rubrica_doc_set_list_path  (RubricaDoc* doc, gchar *path);

void           rubrica_doc_set_child      (RubricaDoc* doc, GtkWidget* child);
void           rubrica_doc_set_view       (RubricaDoc* doc, 
					   GtkTreeView* treeview);
void           rubrica_doc_set_doc_tab    (RubricaDoc* doc, 
					   RubricaDocTab* tab);

/* addressbooks list manage
*/
GList*      rubrica_doc_get_all_books        (void);
RubricaDoc* rubrica_doc_get_document         (GList* book_node);
RubricaDoc* rubrica_doc_get_current_document (void);
RubricaDoc* rubrica_doc_get_nth_document     (gint n);
void        rubrica_doc_set_doc_as_current   (RubricaDoc* doc);
void        rubrica_doc_append_doc           (RubricaDoc* doc,
					      GtkWidget *child, 
					      GtkWidget *box);
/*  Items manage
*/
void     rubrica_doc_reject_id          (RubricaDoc* doc);

gpointer rubrica_doc_get_item_by_id     (RubricaDoc* doc, gulong id);
gpointer rubrica_doc_get_nth_item       (RubricaDoc *doc, gulong n);
gpointer rubrica_doc_get_last_item      (RubricaDoc *doc);

gpointer rubrica_doc_retrieve_item_form (RubricaDoc *doc);
gpointer rubrica_doc_get_selected_item  (RubricaDoc* doc);
gpointer rubrica_doc_modify_item        (RubricaDoc* doc, gpointer item);

gpointer rubrica_doc_copy_item          (RubricaDoc* doc, gpointer item);
void     rubrica_doc_cut_item           (RubricaDoc* doc, gpointer item);
void     rubrica_doc_paste_item         (RubricaDoc* doc, gpointer item);

void     rubrica_doc_append_item        (RubricaDoc* doc, gpointer item);
void     rubrica_doc_replace_item       (RubricaDoc *doc, gpointer old, 
					 gpointer new);

void     rubrica_doc_delete_item        (RubricaDoc* doc, gpointer item);
void     rubrica_doc_delete_last_item   (RubricaDoc *doc);
// gboolean    rubrica_doc_delete_nth_item  (RubricaDoc *doc, gint n);

void     rubrica_doc_build_xmltree      (RubricaDoc* doc);
void     rubrica_doc_save               (RubricaDoc* doc);
void     rubrica_doc_save_as            (RubricaDoc* doc);
void     rubrica_doc_free_item          (RubricaDoc* doc, gpointer item);

/*  utils
*/
GtkWidget*     rubrica_doc_get_tab_box        (RubricaDoc* doc);
void           rubrica_doc_set_tab_label      (RubricaDoc* doc, gchar* str);
void           rubrica_doc_enable_green       (RubricaDoc* doc);
void           rubrica_doc_enable_red         (RubricaDoc* doc);


/*  Free memory
*/
void           rubrica_doc_free_memory (RubricaDoc* doc);
void           rubrica_free_string (gchar *str);  // spostare


/* Closing functions
 */
gboolean       rubrica_doc_others_docs_are_modified (void);
void           rubrica_doc_close                    (RubricaDoc* doc);







#endif
